#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: New Tenet Extraction Engine (TEE)
#------------------------------------------------------------------------------
# Module, with a specific tenet extraction engine, to run the extraction 
# process by applying a list of transduction schemes (CTS) on a working 
# structure 
#==============================================================================

from rdflib import Graph
import os, sys, glob
import logging
from pathlib import Path

from importlib.machinery import SourceFileLoader
import importlib.util
import importlib

from utility.timer import timer_return
from novTransduction.nov_rule import NovRule
from novTransduction.nov_sequence import NovSequence

# Logging
logger = logging.getLogger(__name__)

  
#==============================================================================
# Loading Functions
#==============================================================================

def load_cts(config):
    """ Load extraction scheme (CTS) from <extraction_scheme> file """
    
    try:
        cts_module = SourceFileLoader(config.extraction_scheme, 
                                      config.cts_file).load_module()
    
        return cts_module.rule_dir, cts_module.prefix_list, cts_module.scheme
    
    except FileNotFoundError:
        logger.error(' *** Error while loading scheme (load_cts) ***') 
        logger.debug(f' [{config.process_ref}] ----- current work directory: {os.getcwd()}')
        logger.debug(f' [{config.process_ref}] ----- cts_file unknown: {config.cts_file}') 
        
    except Exception:
        logger.error(' *** Error while loading scheme (load_cts) ***') 
        logger.debug(f' [{config.process_ref}] ----- current work directory: {os.getcwd()}')
        logger.debug(f' [{config.process_ref}] ----- extraction_scheme: {config.extraction_scheme}') 
        logger.debug(f' [{config.process_ref}] ----- cts_file: {config.cts_file}') 
    
    
def get_new_rule_set(config, rule_def_set, prefix_list):
    """ Get a set of new rules from <rule_def_set> (rule definition dictionary)  
    """
    
    try:
        rule_set = {}
        
        for rule_key, rule_def in rule_def_set.items():
            rule = NovRule()
            rule.load_dict(rule_def)
            rule.load_prefix_list(prefix_list)
            rule_set[rule_key] = rule
     
        return rule_set
    
    except:
        logger.error(f' [{config.process_ref}] *** Error while loading rule set (get_new_rule_set) *** ') 
        logger.debug(f' [{config.process_ref}] ----- len(rule_def_set): {len(rule_def_set)}')
        logger.debug(f' [{config.process_ref}] ----- rule_key: {rule_key}') 
        logger.debug(f' [{config.process_ref}] ----- rule_def: {rule_def}')   
        logger.debug(f' [{config.process_ref}] ----- len(rule_set): {rule_set}') 

    
def load_rule_set(config, rule_dir, prefix_list):
    """ Load all rules into a set (as dictionary) from definition files  
        in <rule_dir> directory. 
    """
    
    try:
        rule_set = {}
        
        # -- Append rule paths to sys.path
        path_glob_pattern = config.cts_dir + rule_dir + '**/'
        for rule_path in glob.iglob(path_glob_pattern, recursive = True):
            sys.path.append(rule_path)
    
        # -- Load rule modules
        file_glob_pattern = config.cts_dir + rule_dir + '**/*.py'
        for rule_filename in glob.iglob(file_glob_pattern, recursive = True):
            
            if 'query_builder' not in rule_filename:
                               
                # -- old ---
                # spec = importlib.util.spec_from_file_location(rule_module_name,
                #                                               rule_filename)
                # rule_module = importlib.util.module_from_spec(spec)
                # sys.modules[rule_module_name] = rule_module
                # spec.loader.exec_module(rule_module)
                # -- ---
                
                # -- Import module
                rule_module_name = Path(rule_filename).stem
                rule_module = importlib.import_module(f'{rule_module_name}')
                
                # -- Update rule set
                if hasattr(rule_module, 'rule_set'):
                    rule_def_set = rule_module.rule_set
                    new_rule_set = get_new_rule_set(config, rule_def_set, prefix_list)
                    rule_set.update(new_rule_set)
    
        return rule_set
    
    except:
       logger.error(f' [{config.process_ref}] *** Error while loading rule set (load_rule_set) *** ') 
       logger.debug(f' [{config.process_ref}] ----- path_glob_pattern: {path_glob_pattern}') 
       logger.debug(f' [{config.process_ref}] ----- sys.path: {sys.path}') 
       logger.debug(f' [{config.process_ref}] ----- file_glob_pattern: {file_glob_pattern}') 
       logger.debug(f' [{config.process_ref}] ----- rule_filename: {rule_filename}')  
       logger.debug(f' [{config.process_ref}] ----- rule_module_name: {rule_module_name}')
       logger.debug(f' [{config.process_ref}] ----- rule_module: {rule_module}')   
       logger.debug(f' [{config.process_ref}] ----- len(rule_def_set): {len(rule_def_set)}')
       logger.debug(f' [{config.process_ref}] ----- len(new_rule_set): {len(new_rule_set)}') 
       logger.debug(f' [{config.process_ref}] ----- len(rule_set): {len(rule_set)}')  
        

  
#==============================================================================
# Extraction step
#==============================================================================
        
def _prepare_sequence(sequence_def, rule_set):
    sequence = NovSequence()
    sequence.load_sequence_from_dict(sequence_def)
    sequence.load_rule_list(rule_set)
    return sequence
                       
        
     
@timer_return   
def _apply_rule(graph, rule):
    rule_label, extracted_triple_set = rule(graph)
    for triple in extracted_triple_set:
        graph.add(triple)
    return rule_label, graph, extracted_triple_set

    
def _apply_sequence(config, graph, sequence_list):
    """ Apply the rule on the working graph <graph> """ 
    
    try:
        assert len(sequence_list) > 0, f'Houston, we have a problem: it is an empty sequence ({sequence_list})!'
        
        sequence_label = sequence_list[0]
        logger.info(f" [{config.process_ref}] --- Sequence: {sequence_label}")    
        all_new_triple_set = []
    
        for rule in sequence_list[1:]:
            
            graph_length_before = len(graph)
            
            # -- apply rule
            (rule_label, graph, extracted_triple_set), exec_time_date = _apply_rule(graph, rule) 
            all_new_triple_set.extend(extracted_triple_set)
            
            new_triple_count = len(graph) - graph_length_before
            str = f" [{config.process_ref}] ----- {rule_label}: "
            str += f"{new_triple_count}/{len(extracted_triple_set)} new triple"
            if new_triple_count > 1: str += "s" 
            str += f" ({len(graph)}, {exec_time_date})"
            if (new_triple_count > 0):
                logger.info(str)
            else:
                logger.debug(str)
                    
        return graph, all_new_triple_set
    
    except AssertionError as ae:
        logger.error(f' [{config.process_ref}] *** *** **** Assertion Error *** *** *** \n {ae}')  
        
    except:
        logger.error(f" [{config.process_ref}] *** Error while processing extraction (_apply_new_rule_sequence) ***")
        logger.debug(f" [{config.process_ref}] ----- len(sequence): {len(rule.query_list)} ") 
        logger.debug(f" [{config.process_ref}] ----- len(extracted_triple_set): {len(extracted_triple_set)} ") 
        logger.debug(f" [{config.process_ref}] ----- new_triple_count: {new_triple_count} ") 
        logger.debug(f" [{config.process_ref}] ----- exec_time_date: {exec_time_date} ") 
        
        
    
def _serialize_graph(config, graph, step_name):
    """ Serialize <graph> to a file """ 
    
    try:
        uuid_str = config.uuid_str
        work_file = config.output_file.replace('.ttl', '_' + step_name + '.ttl')
        base_ref = f"http://{uuid_str}/{step_name}"
        
        message = f" [{config.process_ref}] --- Serializing graph to {Path(work_file).stem} "
        logger.debug(message)
        logger.debug(f" [{config.process_ref}] ----- step: {step_name}")
        logger.debug(f" [{config.process_ref}] ----- id: {uuid_str}")
        logger.debug(f" [{config.process_ref}] ----- work_file: {work_file}")
        logger.debug(f" [{config.process_ref}] ----- base: {base_ref}")
        
        graph.serialize(destination=work_file, base=base_ref, format='turtle')

    except:
        logger.error(f" [{config.process_ref}]  *** Error while serializing graph (serialize_graph) ***") 
        logger.debug(f" [{config.process_ref}]  ----- work_file: {work_file} ") 
        
     
def apply_step(config, graph, rule_set, step_number, step_name, step_sequence_def):
    """ Apply extraction step on the working graph """
    
    try:
        logger.info(f" [{config.process_ref}] -- Step {step_number}: {step_name}")
        
        # -- Initialize
        step_triple_list = []
        graph_length_before_step = len(graph)
        
        # -- Apply the sequences of the step
        for sequence_def in step_sequence_def:
            graph, triple_list = _apply_sequence(config, graph, sequence_def)
            step_triple_list.extend(triple_list)
        
        # -- Serialize the working graph updated during the step
        if config.technical_dir_path is not None:
            os.makedirs(config.sentence_output_dir, exist_ok=True)
            _serialize_graph(config, graph, step_name)
        
        # -- Log extracted triple number
        new_triple_count = len(graph) - graph_length_before_step
        logger.info(f" [{config.process_ref}] ----- {new_triple_count} triples extracted during {step_name} step")
        
        return graph, step_triple_list
    
    except AssertionError:
        logger.error(f' [{config.process_ref}] *** *** **** Assertion Error *** *** *** \n')  
    
    except:
        logger.error(f" [{config.process_ref}] *** Error while processing extraction (apply_step) ***") 
        logger.debug(f' [{config.process_ref}] ----- step_name = {step_name}') 
        logger.debug(f' [{config.process_ref}] ----- len(step_sequence_def) = {len(step_sequence_def)}') 
        logger.debug(f' [{config.process_ref}] ----- last sequence def = {sequence_def}') 
    
   
#==============================================================================
# Main Function
#==============================================================================   

def apply(config, graph):
    """ Apply extraction process on the working graph """
    
    try:
        
        # -- Loading Extraction Scheme
        logger.info(f" [{config.process_ref}] -- Loading Extraction Scheme ({config.extraction_scheme})")
        rule_dir, prefix_list, scheme = load_cts(config)
        logger.debug(f" [{config.process_ref}] ----- Step number: {len(scheme)}")
        
        # -- Loading Extraction Rules
        logger.info(f" [{config.process_ref}] -- Loading Extraction Rules ({rule_dir}*)")
        rule_set = load_rule_set(config, rule_dir, prefix_list)
        logger.debug(f" [{config.process_ref}] ----- Total rule number: {len(rule_set)}")
        
        # -- Apply each step of the scheme
        new_triple_list = []
        step_number = 0
        for step_name, step_sequence_def in scheme.items():
            step_number += 1
            graph, new_triple_list = apply_step(config, graph, rule_set,
                                                step_number, step_name, step_sequence_def)
        
        # -- Result: file containing only the factoids (last step result)
        if config.technical_dir_path is not None:
            os.makedirs(config.sentence_output_dir, exist_ok=True)
            factoid_file = config.output_file.replace('.ttl', '_factoid.ttl')
            logger.debug(f" [{config.process_ref}] --- Serializing graph to factoid file (" + factoid_file + ")")
            factoid_graph = Graph()
            for new_triple in new_triple_list:
                factoid_graph.add(new_triple) 
            logger.debug(f" [{config.process_ref}] ----- Number of factoids: " + str(len(new_triple_list)))    
            base_ref = f'http://{config.uuid_str}/factoid'
            logger.debug(f" [{config.process_ref}] ----- Graph base: " + base_ref) 
            factoid_graph.serialize(destination=factoid_file, 
                                    base=base_ref, 
                                    format='turtle')
                
        return graph, new_triple_list   
    
    except:
        logger.error(f' [{config.process_ref}]  *** Error while processing extraction (apply) ***') 
        logger.debug(f' [{config.process_ref}] ----- config.extraction_scheme = {config.extraction_scheme}')  
        logger.debug(f' [{config.process_ref}] ----- rule_dir = {rule_dir}')  
        logger.debug(f' [{config.process_ref}] ----- scheme = {scheme}')      
        logger.debug(f' [{config.process_ref}] ----- step = {step_name}, {step_sequence_def}')  