#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: main
#------------------------------------------------------------------------------
# Module providing the main methods of the TENET library.
#==============================================================================

import sys, os, glob
import shutil
from rdflib import Graph
import logging
import logging.config
import multiprocessing_logging
import multiprocessing
import json

from extraction import config, structure, process
from utility.timer import timed

# -- Config File Path
LIB_PATH = os.path.dirname(os.path.abspath(__file__)) + '/'
LOGGING_CONF_FILE_PATH = f'{LIB_PATH}logging.conf'
OWL_CONFIG_FILE_PATH = f'{LIB_PATH}owl_amr_config.xml'
ODRL_CONFIG_FILE_PATH = f'{LIB_PATH}odrl_amr_config.xml'

# -- Logging
logging.config.fileConfig(LOGGING_CONF_FILE_PATH, disable_existing_loggers=False)
logger = logging.getLogger('root')
    
   
#==============================================================================
# Steps
#==============================================================================

def __set_context():
    print(f'Tenet Running in {LIB_PATH}')
    os.chdir(LIB_PATH)


def __set_config(config_dict):
    config_file_path = config_dict['config_file_path']
    source_type = config_dict['source_type']
    source_corpus = config_dict['source_corpus']
    onto_prefix = config_dict['onto_prefix']
    base_output_dir = config_dict['base_output_dir']
    technical_dir_path = config_dict['technical_dir_path']

    logger.info("-- Process Setting ")
    logger.info(f'----- Corpus source: {source_corpus} ({source_type})')
    logger.info(f'----- Base output dir: {base_output_dir}')
    logger.info(f'----- technical dir path: {technical_dir_path}')
    logger.info(f'----- Ontology target (id): {onto_prefix}')
    logger.info(f'----- Current path: {os.getcwd()}')
    logger.debug(f'----- Config file: {config_file_path}')

    base_config = config.Config(config_dict)

    logger.debug(base_config.get_full_config())

    return base_config


def __count_number_of_graph(config):   
    logger.debug("-- Counting number of graph files (sentences) ")
    sentence_count = 0
    for file_ref in glob.glob(config.source_sentence_file, recursive = True):
        sentence_count += 1   
    assert sentence_count > 0, f'no file to analyze in the input directory ({config.source_sentence_file})'
    logger.info("----- Number of Graphs: {0}".format(sentence_count))


def __apply_extraction(config, sentence_file):
    if config.technical_dir_path is not None:
        os.makedirs(config.sentence_output_dir, exist_ok=True)    
    work_graph = structure.prepare_sentence_work(config, sentence_file)    
    _, new_triple_list = process.apply(config, work_graph)
    return new_triple_list
    
    
def __generate_final_ontology(result_triple_list): 
    config.sentence_output_dir = ''
    logger.info("-- Making complete factoid graph by merging the result factoids")
    factoid_graph = Graph()
    for new_triple in result_triple_list:
        factoid_graph.add(new_triple) 
    logger.info(f'----- Total factoid number: {len(result_triple_list)}')
    return factoid_graph
   
    
def __serialize_factoid_graph(config, factoid_graph, out_file_path=None):
    
    logger.info('-- Serializing graph to factoid string')
    base_ref = f'http://{config.uuid_str}/factoid'
    logger.info(f'----- Graph base: {base_ref}')
    ontology_turtle_string = factoid_graph.serialize(base=base_ref, format='turtle')
    
    if out_file_path is not None:
        logger.info('-- Serializing graph to factoid file')
        logger.info(f'----- Ontology Turtle File: {out_file_path}')
        factoid_graph.serialize(destination=out_file_path, base=base_ref, format='turtle')
        
    return ontology_turtle_string    
    

    
#==============================================================================
# Extraction Run
#==============================================================================
        
def run_extraction(arg_dict):
    process_config = config.Config(arg_dict)
    sentence_file = arg_dict['sentence_file']
    logger.info(f'\n [{process_config.process_ref}]     *** extraction from sentence {process_config.sentence_id} *** ')
    process_config.sentence_output_dir = f'-{process_config.sentence_id}'
    
    try:
        new_triple_list = __apply_extraction(process_config, sentence_file)    
        logger.info(f' [{process_config.process_ref}] Success ({len(new_triple_list)} extracted triple(s))')
        return(new_triple_list)
    
    except:
        logger.info(f' [{process_config.process_ref}] Failure')
        return []

    
#==============================================================================
# AMR Main Methods (to create an ontology)
#==============================================================================

@timed
def create_ontology_from_amrld_file(amrld_file_path, 
                                    base_ontology_path=None, 
                                    onto_prefix=None, 
                                    out_file_path=None, 
                                    technical_dir_path=None):
    """
    Method to create an ontology (as Turtle String) from a transduction 
    analysis of an AMRLD file.

    Parameters
    ----------
    amrld_file_path: a path to an AMR-LD Turtle File.
    base_ontology_path: a path to a Base Ontology Turtle File if defined.
    onto_prefix: the target ontology prefix if defined (if not defined a prefix based on the amrld filename is used).
    out_file_path: a file path where the output ontology is written if defined (the function still outputs the string). 
    technical_dir_path: a dir path where some technical and log files are written if defined.

    Returns
    -------
    Ontology Turtle String.

    """
    
    logger.info('[TENET] Extraction Processing')    
    
    # -- Process Initialization
    logger.info('\n\n === Process Initialization === ')
    __set_context()
    if onto_prefix is None: onto_prefix = 'DefaultId'
    base_output_dir = os.path.dirname(out_file_path) if out_file_path is not None else None
    
    config_dict = {
        'config_file_path': OWL_CONFIG_FILE_PATH,
        'source_type': 'amr',
        'source_corpus': amrld_file_path,
        'onto_prefix': onto_prefix,
        'base_output_dir': base_output_dir,
        'technical_dir_path': technical_dir_path
    }

    config = __set_config(config_dict)
    
    assert os.path.exists(amrld_file_path), f'input file does not exists ({amrld_file_path})'
    
    # -- Extraction Processing   
    logger.info('\n\n === Extraction Processing === ')
    config.sentence_output_dir = f'-0'
    result_triple_list = __apply_extraction(config, amrld_file_path)
    
    # -- Final Ontology Generation (factoid_graph)
    logger.info('\n\n === Final Ontology Generation  === ') 
    factoid_graph = __generate_final_ontology(result_triple_list)
    ontology_turtle_string = __serialize_factoid_graph(config, factoid_graph, out_file_path)
        
    # -- Done
    logger.info('\n\n === Done === ')
    if config.technical_dir_path is not None:
        log_file_name = 'tenet.log'
        dest_file_path = f'{config.technical_dir_path}{log_file_name}'
        shutil.copy(log_file_name, dest_file_path)
    
    return ontology_turtle_string


@timed
def create_ontology_from_amrld_dir(amrld_dir_path, 
                                   base_ontology_path=None, 
                                   onto_prefix=None, 
                                   out_file_path=None, 
                                   technical_dir_path=None,
                                   multiprocessing_run=False,
                                   processes=multiprocessing.cpu_count()-1
                                  ):
    """
    Method to create an ontology (as Turtle String) from a transduction 
    analysis of an AMRLD file.

    Parameters
    ----------
    amrld_file_path: a path to an AMR-LD Turtle File.
    base_ontology_path: a path to a Base Ontology Turtle File if defined.
    onto_prefix: the target ontology prefix if defined (if not defined a prefix based on the amrld filename is used).
    out_file_path: a file path where the output ontology is written if defined (the function still outputs the string). 
    technical_dir_path: a dir path where some technical and log files are written if defined.

    Returns
    -------
    Dictionary [filename -> Ontology Turtle String].
    Complete Ontology Turtle String (synthesis of all ontology)

    """

    logger.info('[TENET] Extraction Processing')    
    
    # -- Process Initialization
    logger.info('\n === Process Initialization === ')
    __set_context()
    if onto_prefix is None: onto_prefix = 'DefaultId'
    base_output_dir = os.path.dirname(out_file_path) if out_file_path is not None else None
        
    config_dict = {
        'config_file_path': OWL_CONFIG_FILE_PATH,
        'source_type': 'amr',
        'source_corpus': amrld_dir_path,
        'onto_prefix': onto_prefix,
        'base_output_dir': base_output_dir,
        'technical_dir_path': technical_dir_path
    }

    config = __set_config(config_dict)

    assert os.path.exists(amrld_dir_path), f'input directory does not exists ({amrld_dir_path})'
    __count_number_of_graph(config)
    
    # -- Extraction Processing    
    logger.info('\n === Extraction Processing === ')
    result_triple_list = []
    
    # ----- Sentence File List
    sentence_dir = config.source_sentence_file   
    sentence_file_list = glob.glob(sentence_dir, recursive = True)
    sentence_file_list = sorted(sentence_file_list)

    # ----- Computing Extraction Argument
    mapIterable = []    
    for i in range(len(sentence_file_list)):
        config_dict['sentence_id'] = i+1
        config_dict['sentence_file'] = sentence_file_list[i]
        mapIterable = mapIterable + [config_dict.copy()]
    
    if multiprocessing_run:
        logger.info('Multi-Processing Run ')
        
        # ----- Multiprocessing Logging (must be exec before the pool is created)
        multiprocessing_logging.install_mp_handler()

        # ----- (Multi-processing) Extraction Run 
        with multiprocessing.Pool(processes) as p:
            triplesLists = p.map(run_extraction, mapIterable)    
            
        # ----- Result Triple List Update
        for tripleList in triplesLists :
            result_triple_list = result_triple_list + tripleList
        
    else:
        logger.info('Single-Processing Run ')
            
        # ----- (Single-processing) Extraction Run
        for config_dict in mapIterable:#
            new_triple_list = run_extraction(config_dict)
            result_triple_list.extend(new_triple_list)
      
    # -- Final Ontology Generation (factoid_graph)
    logger.info('\n === Final Ontology Generation  === ') 
    factoid_graph = __generate_final_ontology(result_triple_list)
    ontology_turtle_string = __serialize_factoid_graph(config, factoid_graph, out_file_path)
     
    # -- Done
    logger.info('\n === Done === ')
    if config.technical_dir_path is not None:
        log_file_name = 'tenet.log'
        dest_file_path = f'{config.technical_dir_path}{log_file_name}'
        shutil.copy(log_file_name, dest_file_path)
    
    return ontology_turtle_string


#==============================================================================
# AMR Main Methods (to generate ODRL statements) 
#==============================================================================

@timed
def generate_odrl_from_amrld_file(
        amrld_file_path, onto_prefix=None, out_file_path=None, 
        technical_dir_path=None):
    """
    Method to generate an ODRL statement (as Turtle String) from a transduction 
    analysis of an AMRLD file.

    Parameters
    ----------
    amrld_file_path: a path to an AMR-LD Turtle File.
    onto_prefix: the target ontology prefix if defined (if not defined a prefix based on the amrld filename is used).
    out_file_path: a file path where the ODRL statement is written if defined (the function still outputs the string). 
    technical_dir_path: a dir path where some technical and log files are written if defined.

    Returns
    -------
    Ontology Turtle String.

    """
    
    logger.info('[TENET] Extraction Processing')    
    
    # -- Process Initialization
    logger.info('\n === Process Initialization === ')
    __set_context()
    if onto_prefix is None: onto_prefix = 'DefaultId'
    base_output_dir = os.path.dirname(out_file_path) if out_file_path is not None else None  
    
    config_dict = {
        'config_file_path': ODRL_CONFIG_FILE_PATH,
        'source_type': 'amr',
        'source_corpus': amrld_file_path,
        'onto_prefix': onto_prefix,
        'base_output_dir': base_output_dir,
        'technical_dir_path': technical_dir_path
    }

    config = __set_config(config_dict)

    assert os.path.exists(amrld_file_path), f'input file does not exists ({amrld_file_path})'
    
    # -- Extraction Processing   
    logger.info('\n === Extraction Processing === ')
    config.sentence_output_dir = f'-0'
    result_triple_list = __apply_extraction(config, amrld_file_path)
    
    # -- Final Ontology Generation (factoid_graph)
    logger.info('\n === Final Ontology Generation  === ') 
    factoid_graph = __generate_final_ontology(result_triple_list)
    ontology_turtle_string = __serialize_factoid_graph(config, factoid_graph, out_file_path)
        
    # -- Done
    logger.info('\n === Done === ')
    if config.technical_dir_path is not None:
        log_file_name = 'tenet.log'
        dest_file_path = f'{config.technical_dir_path}{log_file_name}'
        shutil.copy(log_file_name, dest_file_path)
    
    return ontology_turtle_string


@timed
def generate_odrl_from_amrld_dir(
        amrld_dir_path, onto_prefix=None, out_file_path=None, 
        technical_dir_path=None):
    """
    Method to create an ontology (as Turtle String) from a transduction 
    analysis of an AMRLD file.

    Parameters
    ----------
    amrld_file_path: a path to an AMR-LD Turtle File.
    onto_prefix: the target ontology prefix if defined (if not defined a prefix based on the amrld filename is used).
    out_file_path: a file path where the ODRL statement is written if defined (the function still outputs the string). 
    technical_dir_path: a dir path where some technical and log files are written if defined.

    Returns
    -------
    Dictionary [filename -> Ontology Turtle String].
    Complete Ontology Turtle String (synthesis of all ontology)

    """

    logger.info('[TENET] Extraction Processing')    
    
    # -- Process Initialization
    logger.info('\n === Process Initialization === ')
    __set_context()
    if onto_prefix is None: onto_prefix = 'DefaultId'
    base_output_dir = os.path.dirname(out_file_path) if out_file_path is not None else None  
    
    config_dict = {
        'config_file_path': ODRL_CONFIG_FILE_PATH,
        'source_type': 'amr',
        'source_corpus': amrld_dir_path,
        'onto_prefix': onto_prefix,
        'base_output_dir': base_output_dir,
        'technical_dir_path': technical_dir_path
    }

    config = __set_config(config_dict)
    
    assert os.path.exists(amrld_dir_path), f'input directory does not exists ({amrld_dir_path})'
    __count_number_of_graph(config)
    
    # -- Extraction Processing    
    logger.info('\n === Extraction Processing === ')
    sentence_dir = config.source_sentence_file
    sentence_count = 0
    result_triple_list = []
    for sentence_file in glob.glob(sentence_dir, recursive = True):
        sentence_count += 1
        logger.info(f'     *** sentence {sentence_count} *** ')
        config.sentence_output_dir = f'-{sentence_count}'
        new_triple_list = __apply_extraction(config, sentence_file)
        result_triple_list.extend(new_triple_list)
    
    # -- Final Ontology Generation (factoid_graph)
    logger.info('\n === Final Ontology Generation  === ') 
    factoid_graph = __generate_final_ontology(result_triple_list)
    ontology_turtle_string = __serialize_factoid_graph(config, factoid_graph, out_file_path)
        
    # -- Done
    logger.info('\n === Done === ')
    if config.technical_dir_path is not None:
        log_file_name = 'tenet.log'
        dest_file_path = f'{config.technical_dir_path}{log_file_name}'
        shutil.copy(log_file_name, dest_file_path)
    
    return ontology_turtle_string


    
#==============================================================================
# UNL Main Methods 
#==============================================================================

# TODO