#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Transduction Sequence
#------------------------------------------------------------------------------
# Class to define a Transduction Sequence
#==============================================================================

#==============================================================================
# Importing required modules
#==============================================================================

import logging


#==============================================================================
# Constants and Global Variables
#==============================================================================

# -- String
SEQUENCE_STRING = """ *** Transduction Sequence ***
-- label: {0}
-- comment: {1}
{2}
    """
    
# -- Logging
logger = logging.getLogger(__name__)
    

#==============================================================================
# Useful Functions
#==============================================================================

def get_rule_key_list_str(rule_key_list):
    rule_key_list_str = '-- no rule'
    if len(rule_key_list) == 1:
        rule_key_list_str = '-- rule (1): {0}'.format(rule_key_list[0])
    elif len(rule_key_list) > 1:
        rule_key_list_str = '-- rules ({0}):'.format(len(rule_key_list)) 
        rule_num = 0   
        for rule_name in rule_key_list:
            rule_num += 1
            s = '\n----- [{0}] {1}'
            rule_key_list_str += s.format(rule_num, rule_name)
    return rule_key_list_str


#==============================================================================
# Class
#==============================================================================

class NovSequence:
    """ Class to define a Transduction Sequence.
    """
    
    #--------------------------------------------------------------------------
    # Constructor(s)
    #--------------------------------------------------------------------------
           
    def __init__(self, label='', comment='', rule_key_list=[], rule_list=[]):
        self.label = label
        self.comment = comment
        self.rule_key_list = rule_key_list
        self.rule_list = rule_key_list


    #--------------------------------------------------------------------------
    # Loading Method(s)
    #--------------------------------------------------------------------------
        
    def load_sequence_from_dict(self, sequence_def):
        for attr in self.__dict__:
            if attr in sequence_def.keys():
                self.__dict__[attr] = sequence_def[attr] 
                         
    
    #--------------------------------------------------------------------------
    # Accessor(s)
    #--------------------------------------------------------------------------    
    
    def has_rule(self):
        return len(self.rule_key_list) > 0
        
    
    #--------------------------------------------------------------------------
    # Base Method(s)
    #--------------------------------------------------------------------------

    def __len__(self):
        return len(self.rule_key_list)

    def __str__(self):
        rule_key_list_str = get_rule_key_list_str(self.rule_key_list)            
        sequence_str = SEQUENCE_STRING.format(self.label,
                                              self.comment,
                                              rule_key_list_str)
        return sequence_str
        
    
    #--------------------------------------------------------------------------
    # Method(s) to load the rule list corresponding to the sequence
    #--------------------------------------------------------------------------

    def load_rule_list(self, rule_set):
        """ Load the rule list corresponding to the sequence """
    
        try:
            self.rule_list = []
            if self.has_rule(): 
                for rule_key in self.rule_key_list:
                    rule = rule_set[rule_key]
                    self.rule_list.append(rule)
                    
            # post-condition
            assert len(self.rule_list) == len(self.rule_key_list)
            
            return self.rule_list
        
        except:
            logger.error(' *** Error while getting rules (get_rule_list) *** ')  
            logger.debug(' ----- len(rule_set): {0}'.format(len(rule_set))) 
            logger.debug(' ----- has_rule? {0}'.format(self.has_rule()))   
            logger.debug(' ----- last rule_key: {0}'.format(rule_key))  
  
    
#==============================================================================
# Development Test
#==============================================================================
    
if __name__ == '__main__':
    
    print('\n' + ' *** Development Test ***')
    
    print('\n' + ' -- Test Data:')
    print(' ----- sequence_def_1: ')
    sequence_def_1 = {
        'label': 'test-sequence-1',
        'comment': 'sequence with only one rule',
        'rule_file': 'new_amr_ctr/test.py',
        'rule_key_list': ['one-rule-only']}
    print(sequence_def_1)
    print(' ----- sequence_def_2: ')
    sequence_def_2 = {
        'label': 'test-sequence-2',
        'comment': 'sequence with two rules',
        'rule_file': 'new_amr_ctr/test.py',
        'rule_key_list': ['rule-1', 'rule-2']}
    print(sequence_def_2)
    print(' ----- sequence_def_3: ')
    sequence_def_3 = {
        'label': 'sequence_def_3',
        'comment': 'sequence with no rule',
        'rule_file': 'new_amr_ctr/test.py',
        'rule_key_list': []}
    print(sequence_def_3)

    
    print('\n' + ' -- construct an empty sequence')
    sequence1 = Sequence()
    print(sequence1)
    
    print('\n' + ' -- load a sequence definition from a dictionary')
    sequence1.load_sequence_from_dict(sequence_def_1)
    print(sequence1)
    
    print('\n' + ' -- sequence with two rules')
    sequence2 = Sequence()
    sequence2.load_sequence_from_dict(sequence_def_2)
    print(sequence2)
    
    print('\n' + ' -- sequence with no rule')
    sequence3 = Sequence()
    sequence3.load_sequence_from_dict(sequence_def_3)
    print(sequence3)
    
    
    print('\n' + ' -- Sequence length (__len__)')
    print('len(sequence1) = {0}'.format(len(sequence1)))
    print('len(sequence2) = {0}'.format(len(sequence2)))
    print('len(sequence3) = {0}'.format(len(sequence3)))
    
    
    print('\n' + ' -- has_rule?')
    print('sequence1.has_rule() = {0}'.format(sequence1.has_rule()))
    print('sequence2.has_rule() = {0}'.format(sequence2.has_rule()))
    print('sequence3.has_rule() = {0}'.format(sequence3.has_rule()))
    
        
    print('\n' + ' *** - ***')