#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to conjunctive phenomena or (rule 1)
#------------------------------------------------------------------------------
# Net Expansion AMR rule to analyse conjunctive phenomena (or)
# Rule: property(class, or_phenomena) => compositeClass
#==============================================================================

import rdflib
from rdflib import Graph
from rdflib.namespace import FOAF, RDF

import transduction
from transduction import net
from transduction.rdfterm_computer import produce_uriref, produce_literal
from transduction.query_builder import generate_select_query
from scheme.amr_clara_rule.odrl_generation import odrl_vocabulary_reference


#==============================================================================
# Pattern Search: Rule Nets
#==============================================================================  
       
def __search_pattern(graph):
    select_data_list = ['?rule_net']
    clause_list = [f'?rule_net a net:Rule_Net.']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set



#==============================================================================
# Check Methods
#==============================================================================

def __is_property_to_generate(property_net):
    check_1 = property_net.is_deprecated()
    if is_instance(property_net.mother_property_net, list):
        check_2 = len(property_net.mother_property_net) > 0
    else:
        check_2 = False
    return not check_1 or check_2
    
   
    
#==============================================================================
# Generator Methods
#==============================================================================

ACTION_SPACING_CODE = '\n       '


# Policy URI
# -----------------------------------------------------

def __compute_policy_uri(graph, net):    
    structure_name = net.get_attribute_first_value(net.structure)    
    return produce_uriref(graph, f'ext-out:policy_{structure_name}')


# ODRL Rule Type URI
# -----------------------------------------------------

def __compute_odrl_rule_type_uri(graph, net):
    naming = net.get_attribute_first_value(net.rule_relation_name)  
    odrl_rule_type_uri = f'odrl:{naming}'
    return produce_uriref(graph, odrl_rule_type_uri)


# Action Reference
# -----------------------------------------------------

def __compute_action_ref(graph, action_net):
    
    action_ref = ''
    first = True
    
    action_ref_table = odrl_vocabulary_reference.action_table
    for ref in action_net.action_name:
        ref = str(ref)
        new_ref = action_ref_table[ref] if ref in action_ref_table else f'rdf:{ref}'
        action_ref = new_ref if first else f'{action_ref}, {new_ref}'
        first = False
        
    return action_ref
    

# Target Reference
# -----------------------------------------------------

def __compute_target_ref_from_class_net(graph, target_net_uri):
    
    target_net = net.ClassNet(graph, target_net_uri)
    individual_name = 'any'
    class_name = target_net.get_attribute_first_value(target_net.class_name)
    if class_name is None: class_name = 'unknown'
    
    return f'<http://example.com/asset:{individual_name}.{class_name}>'


def __compute_target_ref_from_individual_net(graph, target_net_uri):
    
    target_net = net.IndividualNet(graph, target_net_uri)
    individual_name = target_net.get_attribute_first_value(target_net.individual_label)
    class_net_uri = target_net.get_attribute_first_value(target_net.mother_class_net)
    class_net = net.ClassNet(graph, class_net_uri)
    class_name = class_net.get_attribute_first_value(class_net.class_name)
    if class_name is None: class_name = 'unknown'
    
    return f'<http://example.com/asset:{individual_name}.{class_name}>'


def __compute_target_ref(graph, action_net):
    
    target_ref = ''
    first = True
    
    for target_net_uri in action_net.target_class_net:
        new_ref = __compute_target_ref_from_class_net(graph, target_net_uri)
        target_ref = new_ref if first else f'{target_ref}, {new_ref}'
        first = False   
   
    for target_net_uri in action_net.target_individual_net:
        new_ref = __compute_target_ref_from_individual_net(graph, target_net_uri)
        target_ref = new_ref if first else f'{target_ref}, {new_ref}'
        first = False
    
    return target_ref


# Assignee Reference
# -----------------------------------------------------

def __compute_assignee_ref_from_class_net(graph, assignee_net_uri):
    
    assignee_net = net.ClassNet(graph, assignee_net_uri)
    individual_name = 'any'
    class_name = assignee_net.get_attribute_first_value(assignee_net.class_name)
    if class_name is None: class_name = 'unknown'
    
    return f'<http://example.com/asset:{individual_name}.{class_name}>'


def __compute_assignee_ref_from_individual_net(graph, assignee_net_uri):
    
    assignee_net = net.IndividualNet(graph, assignee_net_uri)
    individual_name = assignee_net.get_attribute_first_value(assignee_net.individual_label)
    class_net_uri = assignee_net.get_attribute_first_value(assignee_net.mother_class_net)
    class_net = net.ClassNet(graph, class_net_uri)
    class_name = class_net.get_attribute_first_value(class_net.class_name)
    if class_name is None: class_name = 'unknown'
    
    return f'<http://example.com/asset:{individual_name}.{class_name}>'


def __compute_assignee_ref(graph, action_net):
    
    assignee_ref = ''
    first = True
    
    for assignee_net_uri in action_net.assignee_class_net:
        new_ref = __compute_assignee_ref_from_class_net(graph, assignee_net_uri)
        assignee_ref = new_ref if first else f'{assignee_ref}, {new_ref}'
        first = False   
   
    for assignee_net_uri in action_net.assignee_individual_net:
        new_ref = __compute_assignee_ref_from_individual_net(graph, assignee_net_uri)
        assignee_ref = new_ref if first else f'{assignee_ref}, {new_ref}'
        first = False
    
    return assignee_ref


# ODRL Action Code
# -----------------------------------------------------

def __compute_odrl_action_code_list(graph, rule_net): 
    assert rule_net.rule_action_net is not None, f'Houston, we have a problem: rule_action_net is None!'
    assert isinstance(rule_net.rule_action_net, list), f'Houston, we have a problem: rule_action_net is not a list!'
    
    action_code_list = []
    
    for action_net_uri in rule_net.rule_action_net:
        action_net_uri = produce_uriref(graph, action_net_uri)
        action_net = net.ActionNet(graph, action_net_uri)
        
        action_ref = __compute_action_ref(graph, action_net)
        if action_ref != '': action_ref = f'{ACTION_SPACING_CODE} odrl:action {action_ref}'
        
        target_ref = __compute_target_ref(graph, action_net)
        if target_ref != '': target_ref = f'{ACTION_SPACING_CODE} odrl:target {target_ref} ;'
        
        assignee_ref = __compute_assignee_ref(graph, action_net)
        if assignee_ref != '': assignee_ref = f'{ACTION_SPACING_CODE} odrl:assignee {assignee_ref} ;'
    
        action_string = f"""[ {target_ref} {assignee_ref} {action_ref} ]"""
        action_code_list.append(produce_literal(graph, action_string))
    
    return action_code_list


def __generate_odrl_triple_definition(graph, rule_net): 
    
    odrl_triple_definition = []
    
    policy_uri = __compute_policy_uri(graph, rule_net)       
    odrl_rule_type_uri = __compute_odrl_rule_type_uri(graph, rule_net)      
    action_definition_list = __compute_odrl_action_code_list(graph, rule_net) 

    for action_definition in action_definition_list:
        odrl_triple_definition.append((policy_uri, odrl_rule_type_uri, action_definition))
    
    return odrl_triple_definition

  
    
#==============================================================================
# Main Method: analyze_phenomena_or_1
#==============================================================================

def generate_odrl_rule(graph):
    
    # -- Rule Initialization 
    rule_label = 'generate ODRL rule' 
    rule_triple_list = []

    # -- Get property net listings
    _, pattern_set = __search_pattern(graph)
    
    # -- Triple Definition for 'not deprecated property net'
    for pattern in pattern_set:
        rule_net = net.RuleNet(graph, pattern.rule_net)
        if not rule_net.is_deprecated():
            rule_triple_list += __generate_odrl_triple_definition(graph, rule_net)
        
    return rule_label, rule_triple_list