#!/usr/bin/python3.10
# -*- coding: Utf-8 -*-

#==============================================================================
# TENET: Rule to add AMR edge for reified concept
#------------------------------------------------------------------------------
# Add AMR edge corresponding to reification of LK non-core relation to concept.
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.rdfterm_computer import ( produce_uriref, produce_literal )


#==============================================================================
# Pattern Search: reified concept{reified leaf, leaf1, leaf2}
#==============================================================================

def __search_pattern(graph):
    select_data_list = ['?rvLabel', 
                        '?roleId0', '?roleId1', 
                        '?v1Label', '?v2Label',
                        '?reifiedLeaf', '?leaf1', '?leaf2']
    clause_list = ['?reifiedLeaf a amr:AMR_Leaf.',
                   '?reifiedLeaf amr:isReifiedLeaf true.',
                   '?reifiedLeaf amr:hasVariable ?reifiedVariable.',
                   '?reifiedLeaf amr:hasConcept ?reifiedConcept.',
                   '?reifiedVariable amr:label ?rvLabel.',
                   '?reifiedConcept amr:fromAmrLk ?r.',
                   '?leaf1 amr:hasVariable ?v1.',
                   '?leaf2 amr:hasVariable ?v2.',
                   '?v1 a amr:AMR_Variable.',
                   '?v1 amr:label ?v1Label.',
                   '?v1 amr:fromAmrLk ?v1Link.',
                   '?v2 a amr:AMR_Variable.',
                   '?v2 amr:label ?v2Label.',
                   '?v2 amr:fromAmrLk ?v2Link.',
                   '?v1Link ?r ?v2Link.',
                   'BIND ("ARG0" AS ?roleId0)',
                   'BIND ("ARG1" AS ?roleId1)']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set


#==============================================================================
# Construct Method(s)
#==============================================================================

def __add_amr_edge(graph,
                   rvLabel, roleId0, roleId1, v1Label, v2Label, 
                   reifiedLeaf, leaf1, leaf2):
    triple_list = []
        
    # -- New edge 0
    new_edge_0_uri = produce_uriref(graph, f'amr:edge_{rvLabel}_{roleId0}_{v1Label}')
    relation = produce_uriref(graph, 'rdf:type')
    value = produce_uriref(graph, 'amr:AMR_Edge')
    triple_list.append((new_edge_0_uri, relation, value))
        
    # -- Relation: hasRoleID for new edge 0
    relation = produce_uriref(graph, 'amr:hasRoleID')
    triple_list.append((new_edge_0_uri, relation, roleId0))
    
    # -- New edge 1
    new_edge_1_uri = produce_uriref(graph, f'amr:edge_{rvLabel}_{roleId1}_{v2Label}')
    relation = produce_uriref(graph, 'rdf:type')
    value = produce_uriref(graph, 'amr:AMR_Edge')
    triple_list.append((new_edge_1_uri, relation, value))
        
    # -- Relation: hasRoleID for new edge 1
    relation = produce_uriref(graph, 'amr:hasRoleID')
    triple_list.append((new_edge_1_uri, relation, roleId1))
    
    # -- New relation between leafs
    triple_list.append((reifiedLeaf, new_edge_0_uri, leaf1))
    triple_list.append((reifiedLeaf, new_edge_1_uri, leaf2))
    
    return triple_list


#==============================================================================
# Main Method
#==============================================================================

def add_amr_edge_for_reified_concept(graph):
    
    # -- Rule Initialization
    rule_label = 'add AMR edge for reified concept'
    
    # -- Search for patterns
    _, pattern_set = __search_pattern(graph)
    
    # -- Selection Analyzing
    rule_triple_list = []
    for pattern in pattern_set:
        rule_triple_list += __add_amr_edge(
            graph, 
            pattern.rvLabel, pattern.roleId0, pattern.roleId1, pattern.v1Label, pattern.v2Label,
            pattern.reifiedLeaf, pattern.leaf1, pattern.leaf2)
    
    return rule_label, rule_triple_list
