#!/usr/bin/python3.10
# -*- coding: Utf-8 -*-

#==============================================================================
# TENET: Rule to add AMR edge for polarity relation
#------------------------------------------------------------------------------
# Add AMR edge corresponding to leaf with polarity relation.
#==============================================================================

from rdflib import Graph, Literal

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.rdfterm_computer import ( produce_uriref, produce_literal )


#==============================================================================
# Pattern Search: leaf{polarity}
#==============================================================================

def __search_pattern(graph):
    select_data_list = ['?leaf', '?varLabel', '?valueLabel']
    clause_list = ['?leaf a amr:AMR_Leaf.',
                   '?leaf amr:hasVariable ?v.',
                   '?leaf amr:hasConcept ?c.',
                   '?v a amr:AMR_Variable.',
                   '?v amr:label ?varLabel.',
                   '?v amr:fromAmrLk ?vLink.',
                   '?vLink ns2:polarity ?vPolarity.',
                   'BIND (REPLACE(?vPolarity, " ", "") AS ?nvl1).',
                   'BIND (REPLACE(?nvl1, "-", "negative") AS ?valueLabel).'
                   ]
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code)
    return query_code, result_set


#==============================================================================
# Construct Method(s)
#==============================================================================

def __add_amr_edge(graph, leaf, varLabel, valueLabel, roleId='polarity'):
    triple_list = []
    
    # -- New value
    new_value_uri = produce_uriref(graph, f'amr:value_{valueLabel}')
    relation = produce_uriref(graph, 'rdf:type')
    value = produce_uriref(graph, 'amr:AMR_Value')
    triple_list.append((new_value_uri, relation, value))
    
    # -- Value label
    relation = produce_uriref(graph, 'rdfs:label')
    value = valueLabel
    triple_list.append((new_value_uri, relation, value))
    
    # -- New edge
    new_edge_uri = produce_uriref(graph, f'amr:edge_{varLabel}_{roleId}_{valueLabel}')
    relation = produce_uriref(graph, 'rdf:type')
    value = produce_uriref(graph, 'amr:AMR_Edge')
    triple_list.append((new_edge_uri, relation, value))
    
    # -- Relation: hasRoleID for new edge
    relation = produce_uriref(graph, 'amr:hasRoleID')
    value = Literal(f'{roleId}')
    triple_list.append((new_edge_uri, relation, value))
    
    # -- New relation between leaf and value
    triple_list.append((leaf, new_edge_uri, new_value_uri))
    
    return triple_list


#==============================================================================
# Main Method
#==============================================================================

def add_amr_edge_for_polarity_relation(graph):
    
    # -- Rule Initialization
    rule_label = 'add AMR edge for polarity relation'
    
    # -- Search for patterns
    _, pattern_set = __search_pattern(graph)
    
    # -- Selection Analyzing (1)
    rule_triple_list = []
    for pattern in pattern_set:
        rule_triple_list += __add_amr_edge(
            graph, pattern.leaf, pattern.varLabel, pattern.valueLabel)
    
    return rule_label, rule_triple_list
