#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to reify roles as AMR Relation concept
#------------------------------------------------------------------------------
# Reify some LK roles to AMR Relation concept for:
# (1) AMR relation with reification,
# (2) non core LK role,
# (3) matching between role and reification relation.
#==============================================================================

from rdflib import Graph, Literal

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.rdfterm_computer import ( produce_uriref, produce_literal )


#==============================================================================
# Pattern Search
#==============================================================================
       
def __search_pattern(graph):
    select_data_list = ['?originConcept', '?relationName', '?reificationConcept']
    clause_list = [f'?reifiableRelation rdfs:subClassOf amr:AMR_Relation .',
                   f'?reifiableRelation amr:hasReification true .',
                   f'?reifiableRelation amr:hasRelationName ?relationName .',
                   f'?reifiableRelation amr:hasReificationConcept ?reificationConcept .',
                   f'?originConcept a ns3:Role .',
                   f'FILTER (?originConcept != ns1:FrameRole) .',
                   f'BIND (strafter(str(?originConcept), str(ns1:)) AS ?rLabel1) .',
                   f'BIND (strafter(str(?originConcept), "#") AS ?rLabel2) .',
                   f'FILTER (?rLabel1 = ?relationName || ?rLabel2 = ?relationName) .']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set

    
#==============================================================================
# Construct Method(s)
#==============================================================================

def __reify_roles_as_concept(
        graph, originConcept, relationName, reificationConcept):
    
    triple_list = []
    
    new_concept_uri = produce_uriref(graph, f'amr:concept_{relationName}')
    
    # -- New concept
    relation = produce_uriref(graph, 'rdfs:subClassOf')
    value = produce_uriref(graph, 'amr:AMR_Predicat_Concept')
    triple_list.append((new_concept_uri, relation, value))
    
    # -- Reification flag
    relation = produce_uriref(graph, 'amr:isReifiedConcept')
    value = Literal(True)
    triple_list.append((new_concept_uri, relation, value))
    
    # -- Label
    relation = produce_uriref(graph, 'amr:label')
    triple_list.append((new_concept_uri, relation, reificationConcept))
    
    # -- AMR-LK trace 
    relation = produce_uriref(graph, 'amr:fromAmrLk')
    triple_list.append((new_concept_uri, relation, originConcept))
    
    # -- Classification of the original concept as LinkedData (for tracing)
    relation = produce_uriref(graph, 'rdfs:subClassOf')
    value = produce_uriref(graph, 'amr:AMR_Linked_Data')
    triple_list.append((originConcept, relation, value)) 
    
    return triple_list

  
    
#==============================================================================
# Main Method
#==============================================================================

def reify_roles_as_concept(graph):
    
    # -- Rule Initialization 
    rule_label = 'reify roles as concept' 

    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Selection Analyzing (1)
    rule_triple_list = []
    for pattern in pattern_set:
        rule_triple_list += __reify_roles_as_concept(
            graph, pattern.originConcept, pattern.relationName, pattern.reificationConcept)
            
    return rule_label, rule_triple_list
 
