#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to reclassify existing LK variable to AMR variable
#------------------------------------------------------------------------------
# Reclassify existing LK variable to AMR variable.
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.rdfterm_computer import ( produce_uriref, produce_literal )


#==============================================================================
# Pattern Search: leaf{variable, concept}
#==============================================================================
       
def __search_pattern(graph):
    select_data_list = ['?v', '?vLabel', '?vName']
    clause_list = [f'?c rdfs:subClassOf* amr:AMR_Concept .',
                   f'?c amr:fromAmrLk ?linkedC.',
                   f'?v a ?linkedC.',
                   f'BIND (strafter(str(?v), "#") AS ?vLabel)',
                   f'OPTIONAL {{ ?v rdfs:label ?vName }}']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set

    
    
#==============================================================================
# Construct Method(s)
#==============================================================================

def __reclassify_existing_variable(
        graph, v, vLabel, vName):
    
    triple_list = []
    
    # -- New variable
    new_variable_uri = produce_uriref(graph, f'amr:variable_{vLabel}')
    relation = produce_uriref(graph, 'rdf:type')
    value = produce_uriref(graph, 'amr:AMR_Variable')
    triple_list.append((new_variable_uri, relation, value))
    
    # -- Label
    relation = produce_uriref(graph, 'amr:label')
    value = produce_literal(graph, vLabel)
    triple_list.append((new_variable_uri, relation, value))
    
    # -- AMR-LK trace 
    relation = produce_uriref(graph, 'amr:fromAmrLk')
    triple_list.append((new_variable_uri, relation, v))
    
    # -- Name relation
    if vName is not None:
        relation = produce_uriref(graph, 'amr:name')
        value = produce_literal(graph, vName)
        triple_list.append((new_variable_uri, relation, value))
    
    # -- Classification of the original concept as LinkedData (for tracing)
    relation = produce_uriref(graph, 'rdfs:subClassOf')
    value = produce_uriref(graph, 'amr:AMR_Linked_Data')
    triple_list.append((v, relation, value)) 
    
    return triple_list

    
#==============================================================================
# Main Method
#==============================================================================

def reclassify_existing_variable(graph):
    
    # -- Rule Initialization 
    rule_label = 'reclassify existing variable' 

    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Selection Analyzing (1)
    rule_triple_list = []
    for pattern in pattern_set:
        rule_triple_list += __reclassify_existing_variable(
            graph, pattern.v, pattern.vLabel, pattern.vName)
            
    return rule_label, rule_triple_list
 
