#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to extract composite classes (rule 1)
#------------------------------------------------------------------------------
# Net Expansion AMR rule to extract composite classes
# Rule: property(arg0:class, arg1:class) => compositeClass
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.naming_computer import define_composite_naming_1, define_restriction_naming


#==============================================================================
# Pattern Search: modality(arg1/arg2:atomProperty) => actionProperty
#==============================================================================

SPECIAL_ACTION_PATTERN_LIST = [('amr:role_ARG1', 'keep', 'notice', 'notice'),
                               ('amr:role_ARG1', 'keep', 'intact-notice', 'notice'),
                               ('amr:role_ARG1', 'keep', 'intact', 'notice')]

def __search_pattern_1(graph):
    query_code = ''
    result_set = []
    for (arg_relation, action_name, property_name, _) in SPECIAL_ACTION_PATTERN_LIST:
        select_data_list = ['?action_property_net', '?right_property_net']
        clause_list = [f'?action_property_net a [rdfs:subClassOf* net:Action_Property_Net].',
                       f'FILTER NOT EXISTS {{ ?action_property_net a net:Deprecated_Net. }}',
                        f'?action_property_net net:hasNaming "{action_name}".',
                        f'?right_property_net a [rdfs:subClassOf* net:Atom_Property_Net].',
                        f'FILTER NOT EXISTS {{ ?right_property_net a net:Deprecated_Net. }}',
                        f'?right_property_net net:hasNaming "{property_name}".',
                        f'?action_property_net {arg_relation} ?right_property_net.'
                       ]
        query_code = generate_select_query(graph, select_data_list, clause_list)
        result_set += graph.query(query_code)
    return query_code, result_set


def __search_pattern_2(graph):
    query_code = ''
    result_set = []
    for (arg_relation, action_name, class_name, _) in SPECIAL_ACTION_PATTERN_LIST:
        select_data_list = ['?action_property_net', '?right_class_net']
        clause_list = [f'?action_property_net a [rdfs:subClassOf* net:Action_Property_Net].',
                       f'FILTER NOT EXISTS {{ ?action_property_net a net:Deprecated_Net. }}',
                       f'?action_property_net net:hasNaming "{action_name}".',
                       f'?right_class_net a [rdfs:subClassOf* net:Class_Net].',
                       f'FILTER NOT EXISTS {{ ?right_class_net a net:Deprecated_Net. }}',
                       f'?right_class_net net:hasNaming "{class_name}".',
                       f'?action_property_net {arg_relation} ?right_class_net.']
        query_code = generate_select_query(graph, select_data_list, clause_list)
        result_set += graph.query(query_code)
    return query_code, result_set



#==============================================================================
# Useful Computation Method(s)
#============================================================================== 

def __compute_naming(action_net, right_net):
    naming = 'special_action'
    for (_, action_name, right_name, special_name) in SPECIAL_ACTION_PATTERN_LIST:
        c1 = (action_name == str(action_net.get_attribute_first_value(action_net.naming)))
        c2 = (right_name == str(right_net.get_attribute_first_value(right_net.naming)))
        if c1 & c2: naming = special_name
    return naming


def __filter_relation_1(relation_list, excluded_s):
    result_list = []
    for relation in relation_list:
        check = True
        (s, p, o) = relation
        if s == o: check = False
        if s == excluded_s: check = False
        if check: result_list.append(relation)
    return result_list

def __filter_relation_2(relation_list, excluded_o):
    result_list = []
    for relation in relation_list:
        check = True
        (s, p, o) = relation
        if s == o: check = False
        if o == excluded_o: check = False
        if check: result_list.append(relation)
    return result_list

def __propagate_relation(target_net, base_net_1, base_net_2):
    input_relation_list = base_net_1.input_relation_list
    input_relation_list += __filter_relation_1(base_net_2.input_relation_list, base_net_1.uri)
    target_net.input_relation_list = input_relation_list
    output_relation_list = __filter_relation_2(base_net_1.output_relation_list, base_net_2.uri)
    output_relation_list += base_net_2.output_relation_list
    target_net.output_relation_list = output_relation_list
    
    
    
#==============================================================================
# Construct Method(s)
#==============================================================================   

def __construct_action_property_net(graph, action_property_net, right_net):

    # -- Net Composition
    new_action_property_net = net.ActionPropertyNet(graph)
    new_action_property_net.compose(action_property_net, right_net)
    
    # -- Data Computation 
    new_action_property_net.property_type = action_property_net.property_type
    new_action_property_net.core_role = action_property_net.core_role
    new_action_property_net.target_argument_node = action_property_net.target_argument_node
    
    # -- naming
    naming = __compute_naming(action_property_net, right_net)
    new_action_property_net.naming = naming
    new_action_property_net.property_name = naming
    new_action_property_net.property_name01 = naming
    new_action_property_net.property_name10 = naming
    new_action_property_net.property_name12 = naming
    
    # -- Relation Propagation
    __propagate_relation(new_action_property_net, action_property_net, right_net) 
    
    # -- Finalization
    new_action_property_net.finalize()
    triple_definition = new_action_property_net.generate_triple_definition()
    
    return new_action_property_net, triple_definition

  
    
#==============================================================================
# Main Method
#==============================================================================

def extract_action_property_2(graph):
    
    # -- Rule Initialization 
    rule_label = 'extract action properties (2)' 
    rule_triple_list = []

    # -- Search for patterns 
    _, pattern_set_1 = __search_pattern_1(graph)
    _, pattern_set_2 = __search_pattern_2(graph)
    pattern_set = pattern_set_1 + pattern_set_2
    
    # -- Pattern Analysis
    
    for pattern in pattern_set_1:
        action_property_net = net.ActionPropertyNet(graph, uri=pattern.action_property_net)
        right_net = net.PropertyNet(graph, uri=pattern.right_property_net)
        _, triple_list = __construct_action_property_net(graph, action_property_net, right_net)
        rule_triple_list += triple_list
        rule_triple_list += action_property_net.deprecate()
        rule_triple_list += right_net.deprecate()
    
    for pattern in pattern_set_2:
        action_property_net = net.ActionPropertyNet(graph, uri=pattern.action_property_net)
        right_net = net.ClassNet(graph, uri=pattern.right_class_net)
        _, triple_list = __construct_action_property_net(graph, action_property_net, right_net)
        rule_triple_list += triple_list
        rule_triple_list += action_property_net.deprecate()
        rule_triple_list += right_net.deprecate()
    
    return rule_label, rule_triple_list