#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to extract atomic classes
#------------------------------------------------------------------------------
# Net Expansion AMR rule to extract atomic classes
# Rule: leaf{variable, concept} => atomicClass
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.naming_computer import define_composite_naming_1, define_restriction_naming


#==============================================================================
# Pattern Search: leaf{variable, concept}
#==============================================================================
       
def __search_pattern(graph):
    select_data_list = ['?leaf', '?conceptName']
    clause_list = [f'?leaf a amr:AMR_Leaf.',
                   f'?leaf amr:hasConcept ?leafConcept.',
                   f'?leafConcept rdfs:subClassOf amr:AMR_Term_Concept.',
                   f'?leafConcept amr:label ?conceptName.']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set


#==============================================================================
# Useful Additional Search
#==============================================================================

def __structure_query_code(graph):
    select_data_list = ['?sentenceId']
    clause_list = []
    clause_list.append(f'?root a amr:AMR_Root.')
    clause_list.append(f'?root amr:hasSentenceID ?sentenceId.')
    query_code = generate_select_query(graph, select_data_list, clause_list)
    return query_code 
    

def __search_structure(graph):
    query_code = __structure_query_code(graph)  
    result_set = graph.query(query_code) 
    return result_set
  

def __search_leaf_in_relation(graph, base_leaf_uri):
    select_data_list = ['?inRelationRole', '?inNet']
    clause_list = [f'?inNet a [rdfs:subClassOf* net:Net].',
                   f'?inNet net:coverBaseNode ?inLeaf.', 
                   ('?inLeaf', '?relation', base_leaf_uri),
                   f'?relation amr:hasAmrRole ?inRelationRole.']
    query_code = generate_select_query(graph, select_data_list, clause_list) 
    result_set = graph.query(query_code) 
    return query_code, result_set
  

def __search_leaf_out_relation(graph, base_leaf_uri):
    select_data_list = ['?outRelationRole', '?outNet']
    clause_list = [f'?outNet a [rdfs:subClassOf* net:Net].',
                   f'?outNet net:coverBaseNode ?outLeaf.', 
                   (base_leaf_uri, '?relation', '?outLeaf'),
                   f'?relation amr:hasAmrRole ?outRelationRole.']
    query_code = generate_select_query(graph, select_data_list, clause_list) 
    result_set = graph.query(query_code) 
    return query_code, result_set



#==============================================================================
# Useful Computation Method(s)
#==============================================================================  

def __get_structure(graph):
    structure = 'unknown'
    structure_set = __search_structure(graph)
    for row in structure_set: 
        structure = row.sentenceId
    return structure


def __propagate_relation(graph, target_net, base_leaf):
    _, in_relation_set = __search_leaf_in_relation(graph, base_leaf)
    for row in in_relation_set:
        target_net.input_relation_list += [(row.inNet, row.inRelationRole, _)]    
    _, out_relation_set = __search_leaf_out_relation(graph, base_leaf)
    for row in out_relation_set:
        target_net.output_relation_list += [(_, row.outRelationRole, row.outNet)] 
    
    
    
#==============================================================================
# Construct Method(s)
#==============================================================================    

def __construct_atom_class_net(graph, leaf, concept_name):

    # -- Net Composition
    atom_class_net = net.AtomClassNet(graph)
    
    # -- Data Computation
    atom_class_net.base_node = leaf
    atom_class_net.node = leaf
    atom_class_net.class_name = concept_name
    atom_class_net.structure = __get_structure(graph)

    # -- Net Naming
    atom_class_net.naming = concept_name
    
    # -- Relation Propagation
    # __propagate_relation(graph, atom_class_net, leaf) 
    
    # -- Finalization
    atom_class_net.finalize()
    triple_definition = atom_class_net.generate_triple_definition()
    
    return atom_class_net, triple_definition

  
    
#==============================================================================
# Main Method
#==============================================================================

def extract_atom_class(graph):
    
    # -- Rule Initialization 
    rule_label = 'extract atom classes' 

    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Selection Analyzing (1)
    rule_triple_list = []
    for pattern in pattern_set:
        
        # -- New Net Construction
        new_class, triple_list = __construct_atom_class_net(graph, pattern.leaf, pattern.conceptName)
        rule_triple_list += triple_list
            
    return rule_label, rule_triple_list