#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to extract ODRL actions (rule 1)
#------------------------------------------------------------------------------
# Net Expansion AMR rule to extract ODRL actions (rule 1)
# Rule: modality(arg1/arg2:atomProperty) => action
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.naming_computer import define_composite_naming_1, define_restriction_naming


#==============================================================================
# Pattern Search: modality(arg1/arg2:atomProperty) => action
#==============================================================================

def __search_pattern(graph):
    query_code = ''
    result_set = []
    for arg_relation in ['amr:role_ARG1', 'amr:role_ARG2']:
        select_data_list = ['?property_net']
        clause_list = [f'?phenomena_net a [rdfs:subClassOf* net:Modality_Phenomena_Net].',
                       f'FILTER NOT EXISTS {{ ?phenomena_net a net:Deprecated_Net. }}',
                       f'?property_net a [rdfs:subClassOf* net:Action_Property_Net].',
                       f'FILTER NOT EXISTS {{ ?property_net a net:Deprecated_Net. }}',
                       f'?phenomena_net {arg_relation} ?property_net.']
        query_code = generate_select_query(graph, select_data_list, clause_list)
        result_set += graph.query(query_code)
    return query_code, result_set



#==============================================================================
# Useful Additional Search
#==============================================================================

def __search_target_class(graph, property_net_uri):
    select_data_list = ['?class_net']
    clause_list = [(property_net_uri, f'amr:role_ARG1', '?class_net'),
                   f'?class_net a [rdfs:subClassOf* net:Class_Net].',
                   f'FILTER NOT EXISTS {{ ?class_net a net:Deprecated_Net. }}']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set


def __search_target_individual(graph, property_net_uri):
    select_data_list = ['?individual_net']
    clause_list = [(property_net_uri, f'amr:role_ARG1', '?individual_net'),
                   f'?individual_net a [rdfs:subClassOf* net:Individual_Net].',
                   f'FILTER NOT EXISTS {{ ?individual_net a net:Deprecated_Net. }}']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set


def __search_subject_class(graph, property_net_uri):
    select_data_list = ['?class_net']
    clause_list = [(property_net_uri, f'amr:role_ARG0', '?class_net'),
                   f'?class_net a [rdfs:subClassOf* net:Class_Net].',
                   f'FILTER NOT EXISTS {{ ?class_net a net:Deprecated_Net. }}']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set


def __search_subject_atom_class(graph, property_net_uri):
    select_data_list = ['?class_net']
    clause_list = [(property_net_uri, f'amr:role_ARG0', '?class_net'),
                   f'?class_net a [rdfs:subClassOf* net:Atom_Class_Net].']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set


def __search_subject_individual(graph, property_net_uri):
    select_data_list = ['?individual_net']
    clause_list = [(property_net_uri, f'amr:role_ARG0', '?individual_net'),
                   f'?individual_net a [rdfs:subClassOf* net:Individual_Net].',
                   f'FILTER NOT EXISTS {{ ?individual_net a net:Deprecated_Net. }}']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set



#==============================================================================
# Useful Computation Method(s)
#============================================================================== 

def __compute_target_class_net(graph, net_composition, property_net_uri):
    result_net_list = [] 
    _, pattern_set = __search_target_class(graph, property_net_uri)
    for pattern in pattern_set:
        result_net_list.append(pattern.class_net)
        net_composition.append(net.ClassNet(graph, uri=pattern.class_net))        
    return result_net_list if len(result_net_list) > 0 else None


def __compute_target_individual_net(graph, net_composition, property_net_uri):   
    result_net_list = []   
    _, pattern_set = __search_target_individual(graph, property_net_uri)
    for pattern in pattern_set:
        result_net_list.append(pattern.individual_net)
        net_composition.append(net.IndividualNet(graph, uri=pattern.individual_net))   
    return result_net_list if len(result_net_list) > 0 else None
    

def __compute_assignee_class_net(graph, net_composition, property_net_uri): 
    
    result_net_list = []  
    cover_node_list = []
    
    _, pattern_set = __search_subject_class(graph, property_net_uri)
    for pattern in pattern_set:
        result_net_list.append(pattern.class_net)
        class_net = net.ClassNet(graph, uri=pattern.class_net)
        cover_node_list += class_net.node
        net_composition.append(class_net) 
    
    # # Special case: deprecated atom class net because corresonding to composite_class
    # _, pattern_set = __search_subject_atom_class(graph, property_net_uri)  
    # for pattern in pattern_set:
    #     atom_class_net = net.AtomClassNet(graph, uri=pattern.class_net)
    #     cover_node = atom_class_net.get_attribute_first_value(atom_class_net.node)
    #     if cover_node not in cover_node_list:
    #         result_net_list.append(pattern.class_net)
    #         cover_node_list += atom_class_net.node
    #         net_composition.append(atom_class_net) 
        
    return result_net_list if len(result_net_list) > 0 else None
    

def __compute_assignee_individual_net(graph, net_composition, property_net_uri):
    result_net_list = []
    _, pattern_set = __search_subject_individual(graph, property_net_uri)
    for pattern in pattern_set:
        result_net_list.append(pattern.individual_net)
        net_composition.append(net.IndividualNet(graph, uri=pattern.individual_net))
    return result_net_list if len(result_net_list) > 0 else None


def __filter_relation(relation_list):
    result_list = []
    for relation in relation_list:
        check = True
        (s, p, o) = relation
        if s == o: check = False
        if check: result_list.append(relation)
    return result_list


def __propagate_relation(target_net, base_net):
    target_net.input_relation_list = base_net.input_relation_list
    
    
    
#==============================================================================
# Construct Method(s)
#============================================================================== 

def __construct_action_net(graph, property_net):

    # -- Net Composition
    action_net = net.ActionNet(graph)
    net_composition = [property_net]
    
    # -- Data Computation
    action_net.action_name = property_net.property_name
    action_net.target_class_net = __compute_target_class_net(graph, net_composition, property_net.uri)
    action_net.target_individual_net = __compute_target_individual_net(graph, net_composition, property_net.uri)
    action_net.assignee_class_net = __compute_assignee_class_net(graph, net_composition, property_net.uri)
    action_net.assignee_individual_net = __compute_assignee_individual_net(graph, net_composition, property_net.uri)

    # -- Net Naming
    action_net.naming = property_net.naming
    
    # -- Relation Propagation
    __propagate_relation(action_net, property_net)
    
    # -- Finalization
    action_net.compose(*net_composition)
    action_net.finalize()
    triple_definition = action_net.generate_triple_definition()
    
    return action_net, triple_definition

  
    
#==============================================================================
# Main Method
#==============================================================================

def extract_odrl_action(graph):
    
    # -- Rule Initialization 
    rule_label = 'extract ODRL actions' 
    rule_triple_list = []

    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
        
    # -- Pattern Analysis
    for pattern in pattern_set:
        property_net = net.AtomPropertyNet(graph, uri=pattern.property_net)
                       
        # -- New Net Construction
        _, triple_list = __construct_action_net(graph, property_net)
        rule_triple_list += triple_list
    
    return rule_label, rule_triple_list