#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to extract composite classes (rule 1)
#------------------------------------------------------------------------------
# Net Expansion AMR rule to extract composite classes
# Rule: property(arg0:class, arg1:class) => compositeClass
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.naming_computer import define_composite_naming_1, define_restriction_naming


#==============================================================================
# Pattern Search: modality phenomena
#==============================================================================

PHENOMENA_TYPE_RELATION = 'net:hasPhenomenaType' 

MODALITY_TABLE = [('permission', 'amr:phenomena_modality_possible'),
                  ('obligation', 'amr:phenomena_modality_obligation'),
                  ('prohibition', 'amr:phenomena_modality_prohibition')]

def __get_query_code(graph, phenomena_uri, arg_relation):
    select_data_list = ['?phenomena_net', '?action_net']
    clause_list = [f'?phenomena_net a [rdfs:subClassOf* net:Phenomena_Net].',
                    f'?phenomena_net {PHENOMENA_TYPE_RELATION} {phenomena_uri}.',
                    f'FILTER NOT EXISTS {{ ?phenomena_net a net:Deprecated_Net. }}',
                    f'?phenomena_net {arg_relation} ?action_net.',
                    f'?action_net a [rdfs:subClassOf* net:Action_Net].']
    return generate_select_query(graph, select_data_list, clause_list)


def __search_pattern(graph, phenomena_uri):
    query_code = ''
    result_set = []
    for arg_relation in ['amr:role_ARG1', 'amr:role_ARG2']:
        query_code = __get_query_code(graph, phenomena_uri, arg_relation)
        result_set += graph.query(query_code)
    return query_code, result_set



#==============================================================================
# Useful Computation Method(s)
#============================================================================== 

def __compute_rule_action_net(rule_net, action_net_uri):
    action_net_list = rule_net.rule_action_net
    if not isinstance(action_net_list, list): action_net_list = []
    action_net_list.append(action_net_uri)
    return action_net_list


def __define_naming(rule_relation, action_net):
    return f'{rule_relation}'
    
    
    
#==============================================================================
# Construct Method(s)
#============================================================================== 

def __construct_rule_net(graph, rule_relation, phenomena_net, action_net):

    # -- Net Composition
    rule_net = net.RuleNet(graph)
    rule_net.compose(phenomena_net, action_net)
    
    # -- Data Computation
    rule_net.rule_relation_name = rule_relation
    rule_net.rule_action_net = __compute_rule_action_net(rule_net, action_net.uri)
    
    # -- Relation Propagation
    # None

    # -- Net Naming
    rule_net.naming = __define_naming(rule_relation, action_net)
    
    # -- Finalization
    rule_net.finalize()
    triple_definition = rule_net.generate_triple_definition()
    
    return rule_net, triple_definition

  
    
#==============================================================================
# Main Method
#==============================================================================

def extract_odrl_rule(graph):
    
    # -- Rule Initialization 
    rule_label = 'extract ODRL rules' 
    rule_triple_list = []
    
    for (rule_relation, phenomena_uri) in MODALITY_TABLE:

        # -- Search for patterns 
        _, pattern_set = __search_pattern(graph, phenomena_uri)
        
        # -- Pattern Analysis
        for pattern in pattern_set:
            phenomena_net = net.PhenomenaNet(graph, uri=pattern.phenomena_net)
            action_net = net.ActionNet(graph, uri=pattern.action_net)
                       
            # -- New Net Construction
            _, triple_list = __construct_rule_net(
                graph, rule_relation, phenomena_net, action_net)
            rule_triple_list += triple_list
    
    return rule_label, rule_triple_list