#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to negative polarity phenomena (rule 1)
#------------------------------------------------------------------------------
# Net Expansion AMR rule to analyse negative polarity phenomena 
# Rule: polarity(property, 'negative') => compositeClass
#==============================================================================

import rdflib
from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.naming_computer import define_axiom_naming
from transduction.naming_computer import define_composite_naming_2


#==============================================================================
# Pattern Search: polarity(property, 'negative')
#==============================================================================

POLARITY_RELATION = 'amr:role_polarity' 

def __search_pattern(graph):
    select_data_list = ['?property_net']
    clause_list = [f'?property_net a [rdfs:subClassOf* net:Property_Net].',
                   f'?property_net {POLARITY_RELATION} ?value_net.',
                   ('?value_net', 'net:hasValueLabel', rdflib.term.Literal('negative'))]
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set
    


#==============================================================================
# Relation Propagation
#==============================================================================

def __filter_relation(relation_list):
    result_list = []
    for relation in relation_list:
        check = True
        (s, p, o) = relation
        if s == o: check = False
        if p == POLARITY_RELATION: check = False
        if check: result_list.append(relation)
    return result_list


def __propagate_relation(target_net, base_net):
    target_net.input_relation_list = base_net.input_relation_list
    out_relation_list = __filter_relation(base_net.output_relation_list)
    target_net.output_relation_list = out_relation_list
    
    
    
#==============================================================================
# Construct Methods: construction of composite class net
#==============================================================================

def __construct_disjoint_axiom_net(graph, property_net_1, property_net_2):
    
    axiom_net = net.AxiomNet(graph)
    axiom_net.compose(property_net_1, property_net_2)
    
    # -- Data Computation
    axiom_net.axiom_name = 'disjointProperty'
    axiom_net.axiom_uri = 'owl:propertyDisjointWith'
    axiom_net.axiom_net_argument = [property_net_1.uri, property_net_2.uri]

    # -- Relation Propagation: None

    # -- Net Naming
    arg_net_list = [property_net_1, property_net_2]
    axiom_net.naming = define_axiom_naming(axiom_net, arg_net_list)
    
    # -- Finalization
    axiom_net.finalize()
    triple_list = axiom_net.generate_triple_definition()
    
    return axiom_net, triple_list
    
    

def __construct_negative_property_net(graph, property_net_1):

    # -- Net Composition
    composite_property_net = net.CompositePropertyNet(graph)
    composite_property_net.compose(property_net_1)
    
    # -- Data Computation
    composite_property_net.property_type = 'owl:ObjectProperty'

    # -- Restriction Computation
    triple_list_1 = []
    # restriction_net, triple_list_1 = __construct_restriction_net(graph, property_net_1)
    # composite_class_net.restriction = restriction_net.uri

    # -- Net Naming
    composite_property_net.naming = define_composite_naming_2('not', property_net_1)
    
    # -- Relation Propagation
    __propagate_relation(composite_property_net, property_net_1) 
    
    # -- Finalization
    composite_property_net.finalize()
    triple_list_2 = composite_property_net.generate_triple_definition()
    result_triple_list = triple_list_1 + triple_list_2
    
    return composite_property_net, result_triple_list

  
    
#==============================================================================
# Main Method
#==============================================================================

def analyze_phenomena_polarity_1(graph):
    
    # -- Rule Initialization 
    rule_label = 'analyze "polarity" phenomena (1)' 
    rule_triple_list = []

    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Pattern Analysis
    for pattern in pattern_set:
        
        # -- Net Selection
        property_net = net.PropertyNet(graph, uri=pattern.property_net)
        
        # -- New Negative Property Net
        negative_property_net, triple_list_1 = __construct_negative_property_net(graph, property_net)
        
        # -- New Axiom Net
        _, triple_list_2 = __construct_disjoint_axiom_net(graph, property_net, negative_property_net)
        _, triple_list_3 = __construct_disjoint_axiom_net(graph, negative_property_net, property_net)
        
        # -- Update Rule Triple List
        rule_triple_list += triple_list_1 + triple_list_2 + triple_list_3
        
        # -- Deprecation: Origin Class Net
        rule_triple_list += property_net.deprecate()
    
    return rule_label, rule_triple_list