#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to negative polarity phenomena (rule 3)
#------------------------------------------------------------------------------
# Net Expansion AMR rule to analyse negative polarity phenomena 
# Rule: polarity(phenomena, 'negative') => phenomena
#==============================================================================

import rdflib
from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.naming_computer import define_axiom_naming
from transduction.naming_computer import define_composite_naming_2


#==============================================================================
# Pattern Search: polarity(phenomena, 'negative')
#==============================================================================

POLARITY_RELATION = 'amr:role_polarity' 
PHENOMENA_TYPE_RELATION = 'net:hasPhenomenaType' 
POSSIBLE_PHENOMENA_URI = 'amr:phenomena_modality_prohibition'

def __search_pattern(graph):
    query_code = ''
    result_set = []
    for arg_relation in ['amr:role_ARG1', 'amr:role_ARG2']:
        select_data_list = ['?phenomena_net', '?value_net']
        clause_list = [f'?phenomena_net a [rdfs:subClassOf* net:Phenomena_Net].',
                        f'FILTER NOT EXISTS {{ ?phenomena_net a net:Deprecated_Net. }}',
                        f'?phenomena_net {PHENOMENA_TYPE_RELATION} {POSSIBLE_PHENOMENA_URI}.',
                        f'?property_net a [rdfs:subClassOf* net:Atom_Property_Net].',
                        f'?phenomena_net {arg_relation} ?property_net.',
                        f'?property_net {POLARITY_RELATION} ?value_net.',
                        f'FILTER NOT EXISTS {{ ?value_net a net:Deprecated_Net. }}',
                        ('?value_net', 'net:hasValueLabel', rdflib.term.Literal('negative'))
                       ]
        query_code = generate_select_query(graph, select_data_list, clause_list)
        result_set += graph.query(query_code) 
    return query_code, result_set

    

#==============================================================================
# Useful Computation Method(s)
#==============================================================================  

def __filter_relation(relation_list):
    result_list = []
    for relation in relation_list:
        check = True
        (s, p, o) = relation
        if s == o: check = False
        if p == POLARITY_RELATION: check = False
        if check: result_list.append(relation)
    return result_list


def __propagate_relation(target_net, base_net):
    target_net.input_relation_list = base_net.input_relation_list
    out_relation_list = __filter_relation(base_net.output_relation_list)
    target_net.output_relation_list = out_relation_list



#==============================================================================
# Construct Method(s)
#============================================================================== 

def __construct_phenomena_net(graph, origin_phenomena_net, value_net):

    # -- Net Composition
    new_phenomena_net = net.PhenomenaNet(graph)
    new_phenomena_net.compose(origin_phenomena_net, value_net)
    
    # -- Data Computation
    new_phenomena_net.phenomena_type = 'amr:phenomena_modality_obligation'
    new_phenomena_net.phenomena_ref = f'not-{origin_phenomena_net.phenomena_ref}'

    # -- Net Naming
    new_phenomena_net.naming = 'obligation-modality'
    
    # -- Relation Propagation
    __propagate_relation(new_phenomena_net, origin_phenomena_net) 
    
    # -- Finalization
    new_phenomena_net.finalize()
    triple_definition = new_phenomena_net.generate_triple_definition()
    
    return new_phenomena_net, triple_definition

  
    
#==============================================================================
# Main Method
#==============================================================================

def analyze_phenomena_polarity_4(graph):
    
    # -- Rule Initialization 
    rule_label = 'analyze "polarity" phenomena (4)' 
    rule_triple_list = []

    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Pattern Analysis
    for pattern in pattern_set:
        origin_phenomena_net = net.PhenomenaNet(graph, uri=pattern.phenomena_net)
        value_net = net.ValueNet(graph, uri=pattern.value_net)
        
        # -- New Negative Property Net
        _, triple_list_1 = __construct_phenomena_net(graph, origin_phenomena_net, value_net)
        rule_triple_list += triple_list_1
        
        # -- Deprecation: Origin Class Net and Value Net
        rule_triple_list += origin_phenomena_net.deprecate()
        rule_triple_list += value_net.deprecate()
    
    return rule_label, rule_triple_list