#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to generate OWL individual
#------------------------------------------------------------------------------
# Generative rule to build OWL individuals
# Rule: individualNet => OWL individual
#==============================================================================

import rdflib
from rdflib import Graph, Namespace, BNode
from rdflib.namespace import FOAF, RDF, RDFS, OWL

import transduction
from transduction import net
from transduction.rdfterm_computer import ( produce_uriref, produce_literal )

from transduction.query_builder import (
    generate_select_query,
    generate_ask_query
)

DEFAULT_CLASS_TYPE = 'base-out:Undetermined_Thing'
FEATURE_CLASS_TYPE = 'base-out:Feature'


#==============================================================================
# Method(s) to search for patterns: Individual Net
#==============================================================================  
       
def __search_pattern(graph):
    
    # SPARQL query construction
    select_data_list = ['?individual_net']
    clause_list = [(f'?individual_net a net:Individual_Net.')]
    query_code = generate_select_query(graph, select_data_list, clause_list)

    # SPARQL query execution
    rule_pattern_set = graph.query(query_code) 

    # Result (pattern set)
    return query_code, rule_pattern_set



#==============================================================================
# Methods to ask about net (using SPARQL ASK query)
#==============================================================================

# def is_class_net(graph, net_uri):
    
#     # SPARQL query construction
#     clause_list = ['?classUri a [rdfs:subClassOf* net:Class_Net].']
#     query_code = generate_ask_query(graph, clause_list)

#     # SPARQL query execution
#     result = graph.query(query_code, initBindings={'classUri': net_uri})

#     # Result (True or False)
#     return query_code, bool(result)


# def is_individual_net(graph, net_uri):
    
#     # SPARQL query construction
#     clause_list = ['?classUri a [rdfs:subClassOf* net:Individual_Net].']
#     query_code = generate_ask_query(graph, clause_list)

#     # SPARQL query execution
#     result = graph.query(query_code, initBindings={'classUri': net_uri})

#     # Result (True or False)
#     return query_code, bool(result) 



#==============================================================================
# Check Method(s)
#==============================================================================

def __is_class_to_generate(class_net):
    check_1 = class_net.is_deprecated()
    if isinstance(class_net.mother_class_net, list):
        check_2 = len(class_net.mother_class_net) > 0
    else:
        check_2 = False
    return not check_1 or check_2
    
   
    
#==============================================================================
# Generator Methods
#==============================================================================

# -- Base Methods to compute URI
# ------------------------------------------

def __compute_individual_uri(individual_net):
    if isinstance(individual_net.individual_uri, list) and len(individual_net.individual_uri) > 0:
        class_uri = individual_net.individual_uri[0]
    else: # computation of individual_uri
        net_naming = individual_net.get_attribute_first_value(individual_net.naming)
        individual_uri = produce_uriref(individual_net.support_graph, f'ext-out:{net_naming}')
    return individual_uri


def __compute_class_uri(class_net):
    if isinstance(class_net.class_uri, list) and len(class_net.class_uri) > 0:
        class_uri = class_net.class_uri[0]
    else: # computation of class_uri
        net_naming = class_net.get_attribute_first_value(class_net.naming)
        class_uri = produce_uriref(class_net.support_graph, f'ext-out:{net_naming}')
    return class_uri


def __compute_class_type_uri(net):
    if isinstance(net.class_type, list) and len(net.class_type) > 0:
        class_type = net.class_type[0]
    else: 
        class_type = f'{DEFAULT_CLASS_TYPE}'
    return produce_uriref(net.support_graph, class_type)


def __get_mother_class_uri(graph, mother_class_net_uri):
    mother_class_net = net.ClassNet(graph, uri=mother_class_net_uri)
    return __compute_class_uri(mother_class_net)


# def __compute_class_type_uri(class_net):
#     if isinstance(class_net.class_type, list) and len(class_net.class_type) > 0:
#         class_type = class_net.class_type[0]
#     else: # default class_type
#         class_type = f'{DEFAULT_CLASS_TYPE}'
#     return produce_uriref(class_net.support_graph, class_type)


# def __compute_property_uri(property_net):
#     if isinstance(property_net.property_uri, list) and len(property_net.property_uri) > 0:
#         property_uri = property_net.property_uri[0]
#     else: # computation of class_uri
#         net_naming = property_net.get_attribute_first_value(property_net.naming)
#         property_uri = produce_uriref(property_net.support_graph, f'ext-out:{net_naming}')
#     return property_uri



# -- Base Method to define triple
# --------------------------------

def __define_triple(subject_uri, relation, data):    
    triple_list = []    
    for predicat in data:
        triple_list.append((subject_uri, relation, predicat))        
    return triple_list



# -- Methods to generate OWL triple
# -----------------------------------

def __generate_owl_typing_relation(graph, new_individual_uri, net):
    
    triple_list = []
    relation = produce_uriref(graph, 'rdf:type')
    predicat = None
        
    if net.mother_class_net:
        for mother_class_net_uri in net.mother_class_net:
            predicat = __get_mother_class_uri(graph, mother_class_net_uri)
            if predicat is not None:
                triple_list.append((new_individual_uri, relation, predicat))
                
    if net.class_type:
        predicat = __compute_class_type_uri(net)
        triple_list.append((new_individual_uri, relation, predicat))
        
        
    return triple_list



# -- Main Generation Methods
# --------------------------------

def __generate_owl_triple_definition(graph, net):
    
    triple_list = []
             
    new_individual_uri = __compute_individual_uri(net)
    
    rdf_type_uri = produce_uriref(graph, RDF.type)
    individual_type_list = [produce_uriref(graph, 'owl:Individual')]
    triple_list += __define_triple(new_individual_uri, rdf_type_uri, individual_type_list)
    
    relation = produce_uriref(graph, 'rdfs:label')
    triple_list += __define_triple(new_individual_uri, relation, net.individual_label)
    
    relation = produce_uriref(graph, 'base-out:fromStructure')
    triple_list += __define_triple(new_individual_uri, relation, net.structure)
    
    triple_list += __generate_owl_typing_relation(graph, new_individual_uri, net)

    return triple_list

  
    
#==============================================================================
# Main Method: generate_owl_individual
#==============================================================================

def generate_owl_individual(graph):
    
    # -- Rule Initialization 
    rule_label = 'generate OWL individual' 
    rule_triple_list = []

    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)

    # -- OWL Generation
    for pattern in pattern_set:
        individual_net = net.IndividualNet(graph, uri=pattern.individual_net)
        if not individual_net.is_deprecated():
            triple_list = __generate_owl_triple_definition(graph, individual_net)
            rule_triple_list += triple_list

    return rule_label, rule_triple_list