#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to generate OWL relation (property)
#------------------------------------------------------------------------------
# Generative rule to build OWL relations (properties)
# Rule: relationNet => OWL relation
#==============================================================================

import rdflib
from rdflib import Graph, Namespace, BNode
from rdflib.namespace import FOAF, RDF, RDFS, OWL

import transduction
from transduction import net
from transduction.rdfterm_computer import ( produce_uriref, produce_literal )

from transduction.query_builder import (
    generate_select_query,
    generate_ask_query
)


#==============================================================================
# Methods to search for patterns: Relation Net
#==============================================================================

def __search_relation_pattern(graph):
    
    # SPARQL query construction
    select_data_list = ['?relation_net', '?subject_net', '?predicate_net', '?object_net']
    clause_list = [
        f'?relation_net a net:Relation_Net.', 
        f'?relation_net net:hasSubjectNet ?subject_net.',
        f'?relation_net net:hasPredicateNet ?predicate_net.',
        f'?relation_net net:hasObjectNet ?object_net.'
        ]
    query_code = generate_select_query(graph, select_data_list, clause_list)

    # SPARQL query execution
    relation_pattern_set = graph.query(query_code)

    # Result (pattern set)
    return relation_pattern_set


#==============================================================================
# Generator Methods
#==============================================================================

# -- Base Method to compute URI
# ------------------------------------------

# def __compute_individual_uri(individual_net):
#     return rdflib.URIRef("http://example.org/defaultIndividual")

def __compute_individual_uri(individual_net):
    if isinstance(individual_net.individual_uri, list) and len(individual_net.individual_uri) > 0:
        class_uri = individual_net.individual_uri[0]
    else: # computation of individual_uri
        net_naming = individual_net.get_attribute_first_value(individual_net.naming)
        individual_uri = produce_uriref(individual_net.support_graph, f'ext-out:{net_naming}')
    return individual_uri


# def __compute_property_uri(property_net):
#     return rdflib.URIRef("http://example.org/defaultProperty")

def __compute_property_uri(property_net):
    if isinstance(property_net.property_uri, list) and len(property_net.property_uri) > 0:
        property_uri = property_net.property_uri[0]
    else: # computation of property_uri
        net_naming = property_net.get_attribute_first_value(property_net.naming)
        property_uri = produce_uriref(property_net.support_graph, f'ext-out:{net_naming}')
    return property_uri


# -- Base Method to define triple
# --------------------------------

def __define_relation_triple(subject, predicate, object):    
    return (subject, predicate, object)


# -- Methods to generate OWL triple
# -----------------------------------

def __generate_owl_triple_definition(
        graph, subject_net, predicate_net, object_net):
    
    # --
    subject_net = net.IndividualNet(graph, uri=subject_net)
    predicate_net = net.PropertyNet(graph, uri=predicate_net)
    object_net = net.IndividualNet(graph, uri=object_net)
    
    # Compute URIs for each component
    subject_uri = __compute_individual_uri(subject_net)
    predicate_uri = __compute_property_uri(predicate_net)
    object_uri = __compute_individual_uri(object_net)
    
    result_triple = __define_relation_triple(subject_uri, predicate_uri, object_uri)
    return [result_triple]




#==============================================================================
# Main Method: generate_owl_relation
#==============================================================================

def generate_owl_relation(graph):
    
    # -- Rule Initialization 
    rule_label = 'generate OWL relation (property)' 
    rule_triple_list = []

    # -- Get relation net listings
    pattern_set = __search_relation_pattern(graph)
    
    for pattern in pattern_set:
        triple_list = __generate_owl_triple_definition(
            graph, pattern.subject_net, pattern.predicate_net, pattern.object_net)
        rule_triple_list += triple_list

    return rule_label, rule_triple_list
