#!/usr/bin/python3.10
# -*- coding: Utf-8 -*-

#==============================================================================
# TENET: Rule to add AMR edge for core relation
#------------------------------------------------------------------------------
# Add AMR edge corresponding to LK core relation between two leafs.
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.rdfterm_computer import ( produce_uriref, produce_literal )


#==============================================================================
# Pattern Search: core relation{leaf1, leaf2}
#==============================================================================

def __search_pattern(graph):
    select_data_list = ['?v1Label', '?v2Label', '?roleId', '?leaf1', '?leaf2']
    clause_list = ['?r a ?amrLkRelation .',
                   'VALUES ?amrLkRelation {ns1:FrameRole ns3:Role} .',
                   'FILTER NOT EXISTS {{',
                   '?newConcept rdfs:subClassOf amr:AMR_Predicat_Concept .',
                   '?newConcept amr:isReifiedConcept true .',
                   '?newConcept amr:fromAmrLk ?r .',
                   '}}',
                   '?leaf1 amr:hasVariable ?v1 .',
                   '?leaf2 amr:hasVariable ?v2 .',
                   '?v1 a amr:AMR_Variable .',
                   '?v1 amr:label ?v1Label .',
                   '?v1 amr:fromAmrLk ?v1Link .',
                   '?v2 a amr:AMR_Variable .',
                   '?v2 amr:label ?v2Label .',
                   '?v2 amr:fromAmrLk ?v2Link .',
                   '?v1Link ?r ?v2Link .',
                   f'BIND (strafter(str(?r), "#") AS ?rLabel1)',
                   f'BIND (strafter(str(?r), str(ns1:)) AS ?rLabel2)',
                   f'BIND (IF (STRLEN(?rLabel1) > 0, ?rLabel1, ?rLabel2) AS ?rLabel)',
                   f'BIND (IF (CONTAINS(?rLabel, "."), strafter(?rLabel, "."), ?rLabel) AS ?roleId)']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set



#==============================================================================
# Construct Method(s)
#==============================================================================

def __add_amr_edge(graph, v1Label, v2Label, roleId, leaf1, leaf2):
    triple_list = []
    
    # -- New edge
    new_edge_uri = produce_uriref(graph, f'amr:edge_{v1Label}_{v2Label}')
    relation = produce_uriref(graph, 'rdf:type')
    value = produce_uriref(graph, 'amr:AMR_Edge')
    triple_list.append((new_edge_uri, relation, value))
        
    # -- Relation: hasRoleID
    relation = produce_uriref(graph, 'amr:hasRoleID')
    triple_list.append((new_edge_uri, relation, roleId))
    
    # -- New relation between leafs
    triple_list.append((leaf1, new_edge_uri, leaf2))
    
    return triple_list

#==============================================================================
# Main Method
#==============================================================================

def add_amr_edge_for_core_relation(graph):
    
    # -- Rule Initialization
    rule_label = 'add AMR edge for core relation'
    
    # -- Search for patterns
    _, pattern_set = __search_pattern(graph)
    
    # -- Selection Analyzing
    rule_triple_list = []
    for pattern in pattern_set:
        rule_triple_list += __add_amr_edge(
            graph, pattern.v1Label, pattern.v2Label, pattern.roleId, pattern.leaf1, pattern.leaf2)
    
    return rule_label, rule_triple_list
