#!/usr/bin/python3.10
# -*- coding: Utf-8 -*-

#==============================================================================
# TENET: Rule to add root instance in AMR_Root
#------------------------------------------------------------------------------
# Add root instance in AMR_Root.
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.rdfterm_computer import ( produce_uriref, produce_literal )


#==============================================================================
# Pattern Search: r{ns3:root / ns3:has-id / ns3:has-sentence} --
#                 lkRoot / varRoot{amr:fromAmrLk}
#==============================================================================

def __search_pattern(graph):
    select_data_list = ['?leaf', '?graphId', '?graphSentence', '?originRoot']
    clause_list = ['?originRoot a ns3:AMR.',
                   '?originRoot ns3:root ?lkRoot.',
                   '?originRoot ns3:has-id ?graphId.',
                   '?originRoot ns3:has-sentence ?graphSentence.',
                   '?leaf amr:hasVariable ?varRoot.',
                   '?varRoot a amr:AMR_Variable.',
                   '?varRoot amr:fromAmrLk ?lkRoot']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code)
    return query_code, result_set



#==============================================================================
# Construct Method(s)
#==============================================================================

def __add_amr_root(graph, leaf, graphId, graphSentence, originRoot):
    triple_list = []
    
    # -- New root
    new_root_uri = produce_uriref(graph, f'amr:root_{graphId}')
    relation = produce_uriref(graph, 'rdf:type')
    value = produce_uriref(graph, 'amr:AMR_Root')
    triple_list.append((new_root_uri, relation, value))
    
    # -- Relation: hasRootLeaf for new root
    relation = produce_uriref(graph, 'amr:hasRootLeaf')
    value = leaf
    triple_list.append((new_root_uri, relation, value))
    
    # -- Relation: hasSentenceID for new root
    relation = produce_uriref(graph, 'amr:hasSentenceID')
    value = graphId
    triple_list.append((new_root_uri, relation, value))
    
    # -- Relation: hasSentenceStatement for new root
    relation = produce_uriref(graph, 'amr:hasSentenceStatement')
    value = graphSentence
    triple_list.append((new_root_uri, relation, value))
    
    # -- Linking new variable to LK role (for tracing) 
    relation = produce_uriref(graph, 'amr:fromAmrLk')
    triple_list.append((new_root_uri, relation, originRoot))
    
    return triple_list


#==============================================================================
# Main Method
#==============================================================================

def add_amr_root(graph):
    
    # -- Rule Initialization
    rule_label = 'add AMR root'
    
    # -- Search for patterns
    _, pattern_set = __search_pattern(graph)
    
    # -- Selection Analyzing (1)
    rule_triple_list = []
    for pattern in pattern_set:  
        rule_triple_list += __add_amr_root(
            graph, pattern.leaf, pattern.graphId, pattern.graphSentence, pattern.originRoot)
    
    return rule_label, rule_triple_list
