#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to extract atomic individuals
#------------------------------------------------------------------------------
# Net Expansion AMR rule to extract atomic individuals
# Rule: leaf{variable, concept} => atomicClass
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.naming_computer import define_composite_naming_1, define_restriction_naming


#==============================================================================
# Pattern Search: Leaf{variable, concept} corresponding to individual
#==============================================================================

def __search_pattern(graph):
    select_data_list = ['?baseLeaf', '?phenomenaLabel', '?relationConceptPhenomena', '?relationConceptLabel']
    clause_list = [f'?baseLeaf a amr:AMR_Leaf.',
                   f'?baseLeaf amr:hasConcept ?concept.',
                   f'?concept rdfs:subClassOf amr:AMR_Relation_Concept.',
                   f'?concept amr:hasPhenomenaLink ?relationConceptPhenomena.',
                   f'?concept amr:label ?relationConceptLabel.',
                   f'?relationConceptPhenomena amr:label ?phenomenaLabel.']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set



#==============================================================================
# Useful Additional Search
#==============================================================================

def __structure_query_code(graph):
    select_data_list = ['?sentenceId']
    clause_list = []
    clause_list.append(f'?root a amr:AMR_Root.')
    clause_list.append(f'?root amr:hasSentenceID ?sentenceId.')
    query_code = generate_select_query(graph, select_data_list, clause_list)
    return query_code 
    

def __search_structure(graph):
    query_code = __structure_query_code(graph)  
    result_set = graph.query(query_code) 
    return result_set
  

def __search_leaf_in_relation(graph, base_leaf_uri):
    select_data_list = ['?inRelationRole', '?inNet']
    clause_list = [f'?inNet a [rdfs:subClassOf* net:Net].',
                   f'?inNet net:coverBaseNode ?inLeaf.', 
                   ('?inLeaf', '?relation', base_leaf_uri),
                   f'?relation amr:hasAmrRole ?inRelationRole.']
    query_code = generate_select_query(graph, select_data_list, clause_list) 
    result_set = graph.query(query_code) 
    return query_code, result_set
  

def __search_leaf_out_relation(graph, base_leaf_uri):
    select_data_list = ['?outRelationRole', '?outNet']
    clause_list = [f'?outNet a [rdfs:subClassOf* net:Net].',
                   f'?outNet net:coverBaseNode ?outLeaf.', 
                   (base_leaf_uri, '?relation', '?outLeaf'),
                   f'?relation amr:hasAmrRole ?outRelationRole.']
    query_code = generate_select_query(graph, select_data_list, clause_list) 
    result_set = graph.query(query_code) 
    return query_code, result_set

 
    
#==============================================================================
# Useful Computation Method(s)
#==============================================================================  

def __get_structure(graph):
    structure = 'unknown'
    structure_set = __search_structure(graph)
    for row in structure_set: 
        structure = row.sentenceId
    return structure


def __propagate_relation(graph, target_net, base_leaf): 
    _, in_relation_set = __search_leaf_in_relation(graph, base_leaf)
    for row in in_relation_set:
        target_net.input_relation_list += [(row.inNet, row.inRelationRole, _)]       
    _, out_relation_set = __search_leaf_out_relation(graph, base_leaf)
    for row in out_relation_set:
        target_net.output_relation_list += [(_, row.outRelationRole, row.outNet)] 



#==============================================================================
# Construct Method(s)
#============================================================================== 

def __construct_phenomena_net(
        graph, base_leaf, phenomena_label, relation_concept_phenomena, relation_concept_label):

    # -- Net Composition
    phenomena_net = net.PhenomenaNet(graph)
    
    # -- Data Computation
    phenomena_net.base_node = base_leaf
    phenomena_net.node = base_leaf
    phenomena_net.phenomena_type = relation_concept_phenomena
    phenomena_net.phenomena_ref = relation_concept_label
    phenomena_net.structure = __get_structure(graph)

    # -- Net Naming
    phenomena_net.naming = phenomena_label
    
    # -- Relation Propagation
    # __propagate_relation(graph, phenomena_net, base_leaf) 
    
    # -- Finalization
    phenomena_net.finalize()
    triple_definition = phenomena_net.generate_triple_definition()
    
    return phenomena_net, triple_definition

  
    
#==============================================================================
# Main Method
#==============================================================================

def extract_atom_phenomena(graph):
    
    # -- Rule Initialization 
    rule_label = 'extract atom phenomena' 

    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Selection Analyzing (1)
    rule_triple_list = []
    for pattern in pattern_set:
        
        # -- New Net Construction (from 3 nets)
        new_class, triple_list = __construct_phenomena_net(
            graph, 
            pattern.baseLeaf, 
            pattern.phenomenaLabel, 
            pattern.relationConceptPhenomena,
            pattern.relationConceptLabel)
            
        # -- Resulting List Update
        # class_net_list.append(new_class)
        rule_triple_list += triple_list
            
    return rule_label, rule_triple_list