#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to extract atomic properties
#------------------------------------------------------------------------------
# Net Expansion AMR rule to extract atomic properties
# Rule: leaf{variable, amrTermConcept} => atomicProperty
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction import rdfterm_computer
from transduction.query_builder import generate_select_query
from transduction.naming_computer import define_composite_naming_1, define_restriction_naming


#==============================================================================
# Pattern Search: Leaf{variable, amrPrecicateConcept}
#==============================================================================

def __search_pattern(graph):
    select_data_list = ['?baseLeaf', '?predicateLabel']
    clause_list = [f'?baseLeaf a amr:AMR_Leaf.',
                   f'?baseLeaf amr:hasConcept ?leafConcept.',
                   f'?leafConcept rdfs:subClassOf amr:AMR_Predicat_Concept.',
                   f'?leafConcept amr:label ?predicateLabel.']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set


#==============================================================================
# Useful Additional Search
#==============================================================================

def __structure_query_code(graph):
    select_data_list = ['?sentenceId']
    clause_list = []
    clause_list.append(f'?root a amr:AMR_Root.')
    clause_list.append(f'?root amr:hasSentenceID ?sentenceId.')
    query_code = generate_select_query(graph, select_data_list, clause_list)
    return query_code 
    

def __search_structure(graph):
    query_code = __structure_query_code(graph)  
    result_set = graph.query(query_code) 
    return result_set


def __target_argument_pattern_query_code(graph, base_leaf_uri):
    select_data_list = ['?argLeaf']
    clause_list = []
    clause_list.append((base_leaf_uri, '?relation', '?argLeaf'))
    clause_list.append(f'?relation amr:hasAmrRole ?someArgRole.')
    query_code = generate_select_query(graph, select_data_list, clause_list)
    return query_code 
    

def __search_target_argument(graph, base_leaf_uri):
    query_code = __target_argument_pattern_query_code(graph, base_leaf_uri)  
    result_set = graph.query(query_code) 
    return result_set
  

def __search_leaf_in_relation(graph, base_leaf_uri):
    select_data_list = ['?inRelationRole', '?inNet']
    clause_list = [f'?inNet a [rdfs:subClassOf* net:Net].',
                   f'?inNet net:coverBaseNode ?inLeaf.', 
                   ('?inLeaf', '?relation', base_leaf_uri),
                   f'?relation amr:hasAmrRole ?inRelationRole.']
    query_code = generate_select_query(graph, select_data_list, clause_list) 
    result_set = graph.query(query_code) 
    return query_code, result_set
  

def __search_leaf_out_relation(graph, base_leaf_uri):
    select_data_list = ['?outRelationRole', '?outNet']
    clause_list = [f'?outNet a [rdfs:subClassOf* net:Net].',
                   f'?outNet net:coverBaseNode ?outLeaf.', 
                   (base_leaf_uri, '?relation', '?outLeaf'),
                   f'?relation amr:hasAmrRole ?outRelationRole.']
    query_code = generate_select_query(graph, select_data_list, clause_list) 
    result_set = graph.query(query_code) 
    return query_code, result_set



#==============================================================================
# Useful Computation Method(s)
#==============================================================================  

def __get_structure(graph):
    structure = 'unknown'
    structure_set = __search_structure(graph)
    for row in structure_set: 
        structure = row.sentenceId
    return structure


def __define_naming(predicate_label):
    naming = predicate_label.split('-')[0]
    return naming


def __propagate_relation(graph, target_net, base_leaf):
    
    _, in_relation_set = __search_leaf_in_relation(graph, base_leaf)
    for row in in_relation_set:
        target_net.input_relation_list += [(row.inNet, row.inRelationRole, _)] 
        
    _, out_relation_set = __search_leaf_out_relation(graph, base_leaf)
    for row in out_relation_set:
        target_net.output_relation_list += [(_, row.outRelationRole, row.outNet)] 
    
    
    
#==============================================================================
# Construct Method(s)
#==============================================================================    

def __construct_atom_property_net(graph, base_leaf, predicate_label):

    # -- Net Composition
    atom_property_net = net.AtomPropertyNet(graph)
    
    # -- Data Computation
    atom_property_net.base_node = base_leaf
    atom_property_net.node = base_leaf
    atom_property_net.property_type = 'owl:ObjectProperty'
    atom_property_net.core_role = 'true'
    atom_property_net.structure = __get_structure(graph)
    
    # -- Target Argument
    target_argument_set = __search_target_argument(graph, base_leaf)
    for data in target_argument_set:
        atom_property_net.target_argument_node += [data.argLeaf]

    # -- Net Naming
    naming = __define_naming(predicate_label)
    atom_property_net.naming = naming
    atom_property_net.property_name = naming
    atom_property_net.property_name01 = f'{naming}ing'
    atom_property_net.property_name10 = f'{naming}-by'
    atom_property_net.property_name12 = f'{naming}-of'
    
    # -- Relation Propagation
    # __propagate_relation(graph, atom_property_net, base_leaf) 
    
    # -- Finalization
    atom_property_net.finalize()
    triple_definition = atom_property_net.generate_triple_definition()
    
    return atom_property_net, triple_definition

  
    
#==============================================================================
# Main Method
#==============================================================================

def extract_atom_property(graph):
    
    # -- Rule Initialization 
    rule_label = 'extract atomic properties' 

    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Selection Analyzing (1)
    rule_triple_list = []
    for pattern in pattern_set:
        
        # -- New Net Construction (from 3 nets)
        new_net, triple_list = __construct_atom_property_net(
            graph, pattern.baseLeaf, pattern.predicateLabel)
            
        # -- Resulting List Update
        rule_triple_list += triple_list
            
    return rule_label, rule_triple_list