#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to extract atomic individuals
#------------------------------------------------------------------------------
# Net Expansion AMR rule to extract atomic individuals
# Rule: leaf{variable, concept} => atomicClass
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.naming_computer import define_composite_naming_1, define_restriction_naming


#==============================================================================
# Pattern Search: Leaf{variable, concept} corresponding to individual
#==============================================================================
       
def __search_pattern(graph):
    select_data_list = ['?net']
    clause_list = [f'?net a [rdfs:subClassOf* net:Net].']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set


#==============================================================================
# Useful Additional Search
#==============================================================================  

def __search_leaf_in_leaf_relation(graph, base_leaf_uri):
    select_data_list = ['?inRelationRole', '?inNet']
    clause_list = [f'?inNet a [rdfs:subClassOf* net:Net].',
                   f'?inNet net:coverBaseNode ?inLeaf.',
                   f'?inLeaf a amr:AMR_Leaf.',
                   ('?inLeaf', '?relation', base_leaf_uri),
                   f'?relation amr:hasAmrRole ?inRelationRole.']
    query_code = generate_select_query(graph, select_data_list, clause_list) 
    result_set = graph.query(query_code) 
    return query_code, result_set


def __search_leaf_in_value_relation(graph, base_leaf_uri):
    select_data_list = ['?inRelationRole', '?inNet']
    clause_list = [f'?inNet a [rdfs:subClassOf* net:Net].',
                   f'?inNet net:coverAmrValue ?inValue.', 
                   f'?inValue a amr:AMR_Value.',
                   ('?inValue', '?relation', base_leaf_uri),
                   f'?relation amr:hasAmrRole ?inRelationRole.']
    query_code = generate_select_query(graph, select_data_list, clause_list) 
    result_set = graph.query(query_code) 
    return query_code, result_set


def __search_leaf_out_leaf_relation(graph, base_leaf_uri):
    select_data_list = ['?outRelationRole', '?outNet']
    clause_list = [f'?outNet a [rdfs:subClassOf* net:Net].',
                   f'?outNet net:coverBaseNode ?outLeaf.', 
                   f'?outLeaf a amr:AMR_Leaf.',
                   (base_leaf_uri, '?relation', '?outLeaf'),
                   f'?relation amr:hasAmrRole ?outRelationRole.']
    query_code = generate_select_query(graph, select_data_list, clause_list) 
    result_set = graph.query(query_code) 
    return query_code, result_set


def __search_leaf_out_value_relation(graph, base_leaf_uri):
    select_data_list = ['?outRelationRole', '?outNet']
    clause_list = [f'?outNet a [rdfs:subClassOf* net:Net].',
                   f'?outNet net:coverAmrValue ?outValue.', 
                   f'?outValue a amr:AMR_Value.',
                   (base_leaf_uri, '?relation', '?outValue'),
                   f'?relation amr:hasAmrRole ?outRelationRole.']
    query_code = generate_select_query(graph, select_data_list, clause_list) 
    result_set = graph.query(query_code) 
    return query_code, result_set

 
    
#==============================================================================
# Useful Computation Method(s)
#==============================================================================          

def __propagate_relation(graph, target_net, base_leaf):
    
    # -- In Relation Propagation
    _, in_relation_set_1 = __search_leaf_in_leaf_relation(graph, base_leaf)    
    for row in in_relation_set_1:
        target_net.input_relation_list += [(row.inNet, row.inRelationRole, _)] 
    _, in_relation_set_2 = __search_leaf_in_value_relation(graph, base_leaf)
    for row in in_relation_set_2:
        target_net.input_relation_list += [(row.inNet, row.inRelationRole, _)] 
       
    # -- Out Relation Propagation
    _, out_relation_set_1 = __search_leaf_out_leaf_relation(graph, base_leaf)  
    for row in out_relation_set_1:
        target_net.output_relation_list += [(_, row.outRelationRole, row.outNet)] 
    _, out_relation_set_2 = __search_leaf_out_value_relation(graph, base_leaf)
    for row in out_relation_set_2:
        target_net.output_relation_list += [(_, row.outRelationRole, row.outNet)] 

  
    
#==============================================================================
# Main Method
#==============================================================================

def propagate_atom_relation(graph):
    
    # -- Rule Initialization 
    rule_label = 'propagate atom relations' 

    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Selection Analyzing (1)
    rule_triple_list = []
    for pattern in pattern_set:
        target_net = net.Net(graph, pattern.net)
        if len(target_net.base_node) > 0:
            __propagate_relation(graph, target_net, target_net.base_node[0])
            rule_triple_list += target_net.generate_net_relation_triples()
            
    return rule_label, rule_triple_list