#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to extract atomic individuals
#------------------------------------------------------------------------------
# Net Expansion AMR rule to extract atomic individuals
# Rule: leaf{variable, concept} => atomicClass
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.naming_computer import define_composite_naming_1, define_restriction_naming


#==============================================================================
# Pattern Search: Leaf{variable, concept} corresponding to individual
#==============================================================================
       
def __search_pattern(graph):
    select_data_list = ['?value', '?valueLabel']
    clause_list = [f'?value a amr:AMR_Value.',
                   f'?value rdfs:label ?valueLabel.']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set


#==============================================================================
# Useful Additional Search
#==============================================================================

def __structure_query_code(graph):
    select_data_list = ['?sentenceId']
    clause_list = []
    clause_list.append(f'?root a amr:AMR_Root.')
    clause_list.append(f'?root amr:hasSentenceID ?sentenceId.')
    query_code = generate_select_query(graph, select_data_list, clause_list)
    return query_code 
    

def __search_structure(graph):
    query_code = __structure_query_code(graph)  
    result_set = graph.query(query_code) 
    return result_set
  

def __search_leaf_in_relation(graph, base_leaf_uri):
    select_data_list = ['?inRelationRole', '?inNet']
    clause_list = [f'?inNet a [rdfs:subClassOf* net:Net].',
                   f'?inNet net:coverBaseNode ?inLeaf.', 
                   ('?inLeaf', '?relation', base_leaf_uri),
                   f'?relation amr:hasAmrRole ?inRelationRole.']
    query_code = generate_select_query(graph, select_data_list, clause_list) 
    result_set = graph.query(query_code) 
    return query_code, result_set
  

def __search_leaf_out_relation(graph, base_leaf_uri):
    select_data_list = ['?outRelationRole', '?outNet']
    clause_list = [f'?outNet a [rdfs:subClassOf* net:Net].',
                   f'?outNet net:coverBaseNode ?outLeaf.', 
                   (base_leaf_uri, '?relation', '?outLeaf'),
                   f'?relation amr:hasAmrRole ?outRelationRole.']
    query_code = generate_select_query(graph, select_data_list, clause_list) 
    result_set = graph.query(query_code) 
    return query_code, result_set

 
    
#==============================================================================
# Useful Computation Method(s)
#==============================================================================  

def __get_structure(graph):
    structure = 'unknown'
    structure_set = __search_structure(graph)
    for row in structure_set: 
        structure = row.sentenceId
    return structure
  

def __define_naming(value_label):
    value_label = value_label.replace(' ', '#')
    value_label = value_label.replace('-', '#')
    word_set = value_label.split('#')
    first = True
    for word in word_set:
        if first:
            naming = word
            first = False
        else:
            naming += word.capitalize()
    return naming



#==============================================================================
# Construct Method(s)
#============================================================================== 

def __construct_value_net(graph, amr_value, value_label):

    # -- Net Composition
    value_net = net.ValueNet(graph)
    
    # -- Data Computation
    value_net.amr_value = amr_value
    value_net.value_label = value_label
    value_net.structure = __get_structure(graph)

    # -- Net Naming
    value_net.naming = __define_naming(value_label)
    
    # -- Finalization
    value_net.finalize()
    triple_definition = value_net.generate_triple_definition()
    
    return value_net, triple_definition

  
    
#==============================================================================
# Main Method
#==============================================================================

def extract_atom_value(graph):
    
    # -- Rule Initialization 
    rule_label = 'extract atom values' 

    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Selection Analyzing (1)
    rule_triple_list = []
    for pattern in pattern_set:
        
        # -- New Net Construction (from 3 nets)
        new_class, triple_list = __construct_value_net(graph, pattern.value, pattern.valueLabel)
            
        # -- Resulting List Update
        # class_net_list.append(new_class)
        rule_triple_list += triple_list
            
    return rule_label, rule_triple_list