#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to extract composite properties (rule 1)
#------------------------------------------------------------------------------
# Net Expansion AMR rule to extract composite properties
# Rule: property(arg1:property) => compositeProperty
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query, generate_select_distinct_query

FEATURE_CLASS_TYPE = 'base-out:Feature'


#==============================================================================
# Pattern Search: property(arg1:property)
#==============================================================================

def __search_pattern(graph):
    select_data_list = ['?feature_property_net', '?target_property_net']
    clause_list = ['?target_property_net a [rdfs:subClassOf* net:Property_Net].',
                   'FILTER NOT EXISTS {{ ?target_property_net a net:Deprecated_Net. }}',
                   '?feature_property_net a [rdfs:subClassOf* net:Property_Net].',
                   'FILTER NOT EXISTS {{ ?feature_property_net a net:Deprecated_Net. }}',
                   '?feature_property_net amr:role_ARG1 ?target_property_net',
                   'FILTER NOT EXISTS {{ ?feature_property_net amr:role_ARG0 ?other_net_0. }}',
                   'FILTER NOT EXISTS {{ ?feature_property_net amr:role_ARG2 ?other_net_2. }}',
                   'FILTER NOT EXISTS {{ ?feature_property_net amr:role_ARG3 ?other_net_3. }}']
    query_code = generate_select_distinct_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set



#==============================================================================
# Net Checking
#==============================================================================

def __verify_disjoint_cover(net_0, net_1):
    check = True
    for node in net_0.node:
        if node in net_1.node: check = False
    return check



#==============================================================================
# Useful Computation Method(s)
#============================================================================== 

def __extract_naming(net, default='none'):
    naming = f'{default}'
    naming_list = net.naming
    if len(naming_list) > 0:
        assert len(naming_list) == 1, 'Houston, we have a problem: too many naming'
        naming = naming_list[0]
    return naming

def __define_feature_naming(property_net):
    name = __extract_naming(property_net, default='something')
    return f'{name}'

def __define_composite_naming(net_1, net_2):
    name_1 = __extract_naming(net_1, default='something')
    name_2 = __extract_naming(net_2, default='something')
    return f'{name_2}-{name_1}'


def __filter_relation(relation_list):
    result_list = []
    for relation in relation_list:
        check = True
        (s, p, o) = relation
        if s == o: check = False
        if check: result_list.append(relation)
    return result_list


def __propagate_relation(target_net, base_net):
    # -- target_net.input_relation_list = base_net.input_relation_list
    out_relation_list = __filter_relation(base_net.output_relation_list)
    target_net.output_relation_list = out_relation_list
    
    
    
#==============================================================================
# Construct Method(s)
#==============================================================================

def __construct_feature_individual_net(graph, feature_property_net):

    # -- Net Composition
    individual_net = net.IndividualNet(graph)
    individual_net.compose(feature_property_net)
    
    # -- Data Computation
    individual_net.class_type = FEATURE_CLASS_TYPE
    # individual_net.mother_class_net = class_net.uri

    # -- Net Naming
    individual_name = __define_feature_naming(feature_property_net)
    individual_net.naming = individual_name
    individual_net.individual_label = individual_name
    
    # -- Finalization
    individual_net.finalize()
    result_triple_list = individual_net.generate_triple_definition()
    
    return individual_net, result_triple_list 
    
    

def __construct_composite_property_net(graph, base_property_net, feature_class_net):

    # -- Net Composition
    composite_property_net = net.CompositePropertyNet(graph)
    composite_property_net.compose(base_property_net, feature_class_net)
    
    # -- Data Computation
    composite_property_net.mother_property_net = base_property_net.uri
    
    # -- Relation Propagation
    __propagate_relation(composite_property_net, base_property_net) 

    # -- Net Naming
    composite_property_net.naming = __define_composite_naming(base_property_net, feature_class_net)
    
    # -- Finalization
    composite_property_net.finalize()
    result_triple_list = composite_property_net.generate_triple_definition()
    
    return composite_property_net, result_triple_list

  
    
#==============================================================================
# Main Method
#==============================================================================

def extract_composite_property_1(graph):
    
    # -- Rule Initialization 
    rule_label = 'extract composite properties (1)' 

    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Selection Analyzing (1)
    rule_triple_list = []
    for pattern in pattern_set:
        
        # -- Net Selection
        feature_property_net = net.PropertyNet(graph, uri=pattern.feature_property_net)
        base_property_net = net.PropertyNet(graph, uri=pattern.target_property_net)
        
        # -- Condition: disjoint cover
        if __verify_disjoint_cover(feature_property_net, base_property_net):
            
            # -- Feature Individual Net
            feature_individual_net, triple_list_1 = __construct_feature_individual_net(
                graph, feature_property_net)
            
            # -- Composite Property Net
            composite_property_net, triple_list_2 = __construct_composite_property_net(
                graph, base_property_net, feature_individual_net)
            
            rule_triple_list += (triple_list_1 + triple_list_2)
        
        # -- Deprecation
        rule_triple_list += feature_property_net.deprecate()
        rule_triple_list += base_property_net.deprecate()
    
    return rule_label, rule_triple_list