#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to refine mother relation between classes
#------------------------------------------------------------------------------
# AMR rule to refine mother relations for parent classes related to the same 
# node
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.rdfterm_computer import produce_uriref, produce_literal

ENTITY_CLASS_TYPE = 'base-out:Entity'


#==============================================================================
# Pattern Search: domain
#==============================================================================

def __search_pattern(graph):
    query_code = ''
    result_set = []
    for arg_relation in ['amr:role_domain']:
        select_data_list = ['?target_class_net', '?mother_class_net']
        clause_list = ['?target_class_net a [rdfs:subClassOf* net:Class_Net].',
                       '?mother_class_net a [rdfs:subClassOf* net:Class_Net].',  
                       'FILTER ( ?target_class_net != ?mother_class_net ).',
                       '?target_class_net net:hasMotherClassNetToRefine ?mother_class_net.'
                       ]
        query_code = generate_select_query(graph, select_data_list, clause_list)
        result_set += graph.query(query_code)
    return query_code, result_set


#==============================================================================
# Useful Additional Search
#==============================================================================

def __search_parent_class_related_to_same_node(graph, class_net_uri):
    select_data_list = ['?parent_class_net']
    clause_list = [
        '?parent_class_net a [rdfs:subClassOf* net:Class_Net].',
        f'?parent_class_net net:hasMotherClassNet <{class_net_uri}>.',
        '?parent_class_net net:coverBaseNode ?sameBaseNode.',
        f'<{class_net_uri}> net:coverBaseNode ?sameBaseNode.'        
    ]
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set
    
    
#==============================================================================
# Construct Method(s)
#==============================================================================

def __classify_net_as_entity(graph, net_uri):
    class_net = net.ClassNet(graph, uri=net_uri)
    class_net.class_type = ENTITY_CLASS_TYPE    
    return class_net.generate_triple_definition()

def __associate_mother_relation(graph, left_net_uri, right_net_uri):
    left_class_net = net.ClassNet(graph, uri=left_net_uri)
    right_class_net = net.ClassNet(graph, uri=right_net_uri)
    left_class_net.mother_class_net = right_class_net.uri
    return left_class_net.generate_triple_definition()



#==============================================================================
# Main Rule Method: classify-net-from-domain
#==============================================================================

def refine_mother_relation(graph):
   
    # -- Rule Initialization 
    rule_label = "Refine mother relations for parent classes related to the same node"
    rule_triple_list = []
    
    query_code, pattern_set = __search_pattern(graph)
    
    for pattern in pattern_set:
        
        _, parent_class_set = __search_parent_class_related_to_same_node(graph, pattern.mother_class_net)
        
        for row in parent_class_set:
            
            if pattern.target_class_net != row.parent_class_net:
                # Associate mother relation between target and found parent
                new_triples = __associate_mother_relation(graph, pattern.target_class_net, row.parent_class_net)
                rule_triple_list.extend(new_triples)
                
        if len(parent_class_set) == 0: # no parent found
            # Associate mother relation between target and origin mother
            new_triples = __associate_mother_relation(graph, pattern.target_class_net, pattern.mother_class_net)
            rule_triple_list.extend(new_triples)
    
    return rule_label, rule_triple_list

