#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# refine_restriction_1: Rule for heuristic deduction
#------------------------------------------------------------------------------
# Heuristic deduction rule to refine restriction net value
#==============================================================================

from transduction import net
from transduction.query_builder import generate_select_query
from transduction.naming_computer import define_composite_naming_1, define_restriction_naming


#==============================================================================
# Pattern Search: 
#    restriction(compositeClass, restrictionNet)
#==============================================================================

def __search_pattern(graph):
    select_data_list = ['?composite_class_net', 
                        '?restriction_net', 
                        '?predicate_property_net', 
                        '?new_net_value']
    clause_list = [      
        '?composite_class_net a net:Composite_Class_Net.',
        '?composite_class_net net:hasRestriction ?restriction_net.',   
        '?restriction_net a net:Restriction_Net.',
        '?restriction_net net:hasRestrictionNetValue ?origin_net_value.',
        '?restriction_net net:hasRestrictionOnProperty ?predicate_property_net.'
        '?new_net_value net:hasMotherClassNet ?origin_net_value.'
        '?new_net_value a [rdfs:subClassOf* net:Class_Net].'
    ]
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set


#==============================================================================
# Useful Additional Search
#==============================================================================

# def __search_individual(graph):
#     query_code = ''
#     result_set = []
#     select_data_list = ['?individualNet', '?classNet']
#     clause_list = [
#         '?individualNet a net:Individual_Net.',
#         '?individualNet net:coverBaseNode ?sameBaseLeaf.',
#         '?classNet a [rdfs:subClassOf* net:Class_Net].',
#         '?classNet net:coverBaseNode ?sameBaseLeaf.'
#     ]
#     query_code = generate_select_query(graph, select_data_list, clause_list)
#     result_set = graph.query(query_code)
#     return query_code, result_set

def __search_individual_from_class(graph, class_net_uri):
    select_data_list = ['?individual_net']
    clause_list = [
        '?individual_net a net:Individual_Net.',
        f'?individual_net net:hasMotherClassNet <{class_net_uri}>.' 
    ]
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return result_set



#==============================================================================
# Net Checking
#==============================================================================

def __verify_disjoint_cover(class_net_0, class_net_1):
    check = True
    for node in class_net_0.node:
        if node in class_net_1.node: check = False
    return check



#==============================================================================
# Useful Computation Method(s)
#============================================================================== 

def __filter_relation(relation_list):
    result_list = []
    for relation in relation_list:
        check = True
        (s, p, o) = relation
        if s == o: check = False
        if check: result_list.append(relation)
    return result_list


def __propagate_relation(target_net, base_net):
    # -- target_net.input_relation_list = base_net.input_relation_list
    out_relation_list = __filter_relation(base_net.output_relation_list)
    target_net.output_relation_list = out_relation_list
    
    
    
#==============================================================================
# Construct Method(s)
#==============================================================================

def __construct_restriction_net(graph, property_net, target_class_net):
    
    # -- Net Composition
    restriction_net = net.RestrictionNet(graph)
    restriction_net.compose(property_net, target_class_net)
    
    # -- Data Computation
    restriction_net.restriction_property = property_net.uri
    restriction_net.restriction_net_value = target_class_net.uri

    # -- Relation Propagation: None

    # -- Net Naming
    restriction_net.naming = define_restriction_naming(property_net, target_class_net)
    
    # -- Finalization
    restriction_net.finalize()
    triple_list = restriction_net.generate_triple_definition()
    
    return restriction_net, triple_list
    
    

def __construct_composite_class_net(
        graph, mother_class_net, base_class_net, core_property_net, target_class_net):

    # -- Net Composition
    composite_class_net = net.CompositeClassNet(graph)
    composite_class_net.compose(base_class_net, core_property_net, target_class_net)
    
    # -- Data Computation
    # composite_class_net.mother_class_net = [base_class_net.uri, mother_class_net.uri] 
    composite_class_net.mother_class_net = mother_class_net.uri
    composite_class_net.root_class_net = base_class_net.root_class_net

    # -- Restriction Computation
    restriction_net, triple_list_1 = __construct_restriction_net(graph, core_property_net, target_class_net)
    composite_class_net.restriction = restriction_net.uri
    
    # -- Relation Propagation
    __propagate_relation(composite_class_net, base_class_net) 

    # -- Net Naming
    composite_class_net.naming = define_composite_naming_1(
        base_class_net, core_property_net, target_class_net)
    
    # -- Finalization
    composite_class_net.finalize()
    triple_list_2 = composite_class_net.generate_triple_definition()
    result_triple_list = triple_list_1 + triple_list_2
    
    return composite_class_net, result_triple_list


def __propagate_individual(graph, origin_composite_net, new_class):
    result_triples = []
    pattern_set = __search_individual_from_class(graph, origin_composite_net.uri)
    for pattern in pattern_set:
        individual_net_uri = pattern.individual_net
        individual_net = net.IndividualNet(graph, uri=individual_net_uri)
        individual_net.mother_class_net = new_class.uri
        result_triples.extend(individual_net.generate_triple_definition())
    return result_triples


#==============================================================================
# Main Method
#==============================================================================

def refine_composite_class(graph):
    
    # -- Rule Initialization 
    rule_label = 'refine composite classes' 

    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)

    # -- Selection Analyzing (1)
    rule_triple_list = []
    for pattern in pattern_set:
        
        # -- Net Selection
        origin_composite_net = net.ClassNet(graph, uri=pattern.composite_class_net)
        core_property_net = net.PropertyNet(graph, uri=pattern.predicate_property_net)
        target_class_net = net.ClassNet(graph, uri=pattern.new_net_value)
        
        # -- Base class net of the composite class
        assert isinstance(origin_composite_net.mother_class_net, list), 'Houston, we have a problem: incorrect net attribute'
        assert len(origin_composite_net.mother_class_net) > 0, 'Houston, we have a problem: attribute is an empty list'
        base_class_net_uri = origin_composite_net.mother_class_net[0]
        base_class_net = net.ClassNet(graph, uri=base_class_net_uri)
        
        # -- Condition: disjoint cover
        if __verify_disjoint_cover(base_class_net, target_class_net):
            
            # -- New Composite Class Net
            new_class, triple_list = __construct_composite_class_net(
                graph, origin_composite_net, base_class_net, core_property_net, target_class_net)
            rule_triple_list += triple_list
            
            # Propagate individual by completing Individual Net
            new_triples = __propagate_individual(graph, origin_composite_net, new_class)
            rule_triple_list.extend(new_triples)

    return rule_label, rule_triple_list

 
