#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# relation_deducer_1: Rule for heuristic deduction
#------------------------------------------------------------------------------
# Heuristic deduction rule to deduce individual and relation from restriction
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net

from transduction.query_builder import (
    generate_select_query,
    generate_ask_query
    )

from transduction.naming_computer import ( 
    define_individual_naming, 
    define_relation_naming
    )

ENTITY_CLASS_TYPE = 'base-out:Entity'


#==============================================================================
# Pattern Search: 
#    restriction(compositeClass, restrictionNet)
#==============================================================================

def __search_pattern(graph):
    select_data_list = ['?subject_class_net', '?predicate_property_net', '?object_net']
    clause_list = [      
        '?subject_class_net a net:Composite_Class_Net.',
        '?subject_class_net net:hasRestriction ?restriction_net.',   
        '?restriction_net a net:Restriction_Net.',
        '?restriction_net net:hasRestrictionNetValue ?object_net.',
        '?restriction_net net:hasRestrictionOnProperty ?predicate_property_net.'
    ]
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set


#==============================================================================
# Useful Additional Search
#==============================================================================

def __search_structure(graph):
    select_data_list = ['?sentenceId']
    clause_list = ['?root a amr:AMR_Root.', 
                   '?root amr:hasSentenceID ?sentenceId.']
    query_code = generate_select_query(graph, select_data_list, clause_list) 
    result_set = graph.query(query_code) 
    return result_set


def __search_individual_from_class(graph, class_net_uri):
    select_data_list = ['?individual_net']
    clause_list = [
        '?individual_net a net:Individual_Net.',
        f'?individual_net net:hasMotherClassNet <{class_net_uri}>.' 
    ]
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return result_set


def __search_subclass_of_class_net(graph, class_net_uri):
    select_data_list = ['?subclass_net']
    clause_list = [
        f'?subclass_net net:hasMotherClassNet* <{class_net_uri}>.'
    ]
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code)
    return result_set


#==============================================================================
# Methods to ask about net (using SPARQL ASK query)
#==============================================================================

def is_class_net(graph, net_uri):
    
    # SPARQL query construction
    clause_list = ['?classUri a [rdfs:subClassOf* net:Class_Net].']
    query_code = generate_ask_query(graph, clause_list)

    # SPARQL query execution
    result = graph.query(query_code, initBindings={'classUri': net_uri})

    # Result (True or False)
    return bool(result)


def is_individual_net(graph, net_uri):
    
    # SPARQL query construction
    clause_list = ['?classUri a [rdfs:subClassOf* net:Individual_Net].']
    query_code = generate_ask_query(graph, clause_list)

    # SPARQL query execution
    result = graph.query(query_code, initBindings={'classUri': net_uri})

    # Result (True or False)
    return bool(result) 



#==============================================================================
# Useful Computation Method(s)
#==============================================================================

def __get_structure(graph):
    structure = 'unknown'
    structure_set = __search_structure(graph)
    for row in structure_set: 
        structure = row.sentenceId
    return structure


def __define_individual_label(net, reference):
    class_label = 'thing'
    naming_list = net.naming
    if len(naming_list) > 0:
        assert len(naming_list) == 1, 'Houston, we have a problem: too many naming'
        class_label = naming_list[0]
    return f'{class_label} ({reference})'

        

#==============================================================================
# Construct Method(s)
#==============================================================================

def __construct_individual_net(graph, mother_class_net):
    
    # -- Net Composition
    individual_net = net.IndividualNet(graph)
    
    # -- Main mother class
    root_class_net = mother_class_net
    if mother_class_net.root_class_net:
        root_class_net = net.ClassNet(graph, uri=mother_class_net.root_class_net[0])
    
    # -- Data Computation
    individual_net.base_node = mother_class_net.base_node
    individual_net.node = mother_class_net.node
    structure_ref = __get_structure(graph)
    individual_net.individual_label = __define_individual_label(root_class_net, structure_ref)
    individual_net.structure = structure_ref
    individual_net.class_type = ENTITY_CLASS_TYPE
    
    # -- Mother classes
    mother_class_net_list = [mother_class_net.uri]
    subclass_net_set = __search_subclass_of_class_net(graph, mother_class_net.uri)
    for row in subclass_net_set:
        mother_class_net_list.append(row.subclass_net)
    individual_net.mother_class_net = mother_class_net_list

    # -- Net Naming
    individual_net.naming = define_individual_naming(root_class_net, structure_ref)
    
    # -- Finalization
    individual_net.finalize()
    triple_definition = individual_net.generate_triple_definition()
    
    return individual_net, triple_definition


def __construct_relation_net(graph, subject_net, predicate_net, object_net):
    
    # -- Net Composition
    relation_net = net.RelationNet(graph)
    relation_net.compose(subject_net, predicate_net, object_net)
    
    # -- Data Computation
    relation_net.subject_net = subject_net.uri
    relation_net.predicate_net = predicate_net.uri
    relation_net.object_net = object_net.uri  
    
    # -- Net Naming
    relation_net.naming = define_relation_naming(subject_net, predicate_net, object_net)
    
    # -- Relation Propagation
    # __propagate_relation(graph, subject_net, base_leaf) 
    
    # -- Finalization
    relation_net.finalize()
    triple_definition = relation_net.generate_triple_definition()
    
    return relation_net, triple_definition


#==============================================================================
# Processing Method for two specific cases
#==============================================================================

def deduce_relation_for_object_individual(graph, pattern):
    rule_triple_list = []  
    
    # Useful Nets
    subject_class_net = net.ClassNet(graph, uri=pattern.subject_class_net)
    predicate_property_net = net.PropertyNet(graph, uri=pattern.predicate_property_net)
    object_individual_net = net.ClassNet(graph, uri=pattern.object_net)
    
    # -- Check for existing (subject) Individual
    existing_subject_individuals = list(__search_individual_from_class(graph, pattern.subject_class_net))
        
    # If either a subject or an object individual exists, then proceed
    if existing_subject_individuals:

        # Handle subject individual
        if existing_subject_individuals:
            subject_individual_net = net.IndividualNet(graph, uri=existing_subject_individuals[0].individual_net)
        else:
            subject_individual_net, subject_individual_triple_list = __construct_individual_net(graph, subject_class_net)
            rule_triple_list += subject_individual_triple_list
        
        # Create the relation
        new_relation_net, relation_triple_list = __construct_relation_net(
            graph, subject_individual_net, predicate_property_net, object_individual_net)
        rule_triple_list += relation_triple_list
        
    return rule_triple_list
    
    
def deduce_relation_for_object_class(graph, pattern):  
    rule_triple_list = []  
    
    # Useful Nets
    subject_class_net = net.ClassNet(graph, uri=pattern.subject_class_net)
    predicate_property_net = net.PropertyNet(graph, uri=pattern.predicate_property_net)
    object_class_net = net.ClassNet(graph, uri=pattern.object_net)
    
    # -- Check for existing (subject) Individual
    existing_subject_individuals = list(__search_individual_from_class(graph, pattern.subject_class_net))

    # -- Check for existing (object) Individual
    existing_object_individuals = list(__search_individual_from_class(graph, pattern.object_net))
        
    # If either a subject or an object individual exists, then proceed
    if existing_subject_individuals or existing_object_individuals:

        # Handle subject individual
        if existing_subject_individuals:
            subject_individual_net = net.IndividualNet(graph, uri=existing_subject_individuals[0].individual_net)
        else:
            subject_individual_net, subject_individual_triple_list = __construct_individual_net(graph, subject_class_net)
            rule_triple_list += subject_individual_triple_list

        # Handle object individual
        if existing_object_individuals:
            object_individual_net = net.IndividualNet(graph, uri=existing_object_individuals[0].individual_net)
        else:
            object_individual_net, object_individual_triple_list = __construct_individual_net(graph, object_class_net)
            rule_triple_list += object_individual_triple_list
        
        # Create the relation
        new_relation_net, relation_triple_list = __construct_relation_net(
            graph, subject_individual_net, predicate_property_net, object_individual_net)
        rule_triple_list += relation_triple_list
        
    return rule_triple_list


#==============================================================================
# Main Method
#==============================================================================

def deduce_individual_and_relation_from_restriction(graph):
    
    # -- Rule Initialization 
    rule_label = 'deduce individual and relation from restriction' 

    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)

    # -- Selection Analyzing (1)
    rule_triple_list = []
    for pattern in pattern_set:
       
        if is_individual_net(graph, pattern.object_net):
            rule_triple_list += deduce_relation_for_object_individual(graph, pattern)
            
        if is_class_net(graph, pattern.object_net):
            rule_triple_list += deduce_relation_for_object_class(graph, pattern)

    return rule_label, rule_triple_list


def deduce_individual_and_relation_from_restriction_Recursively(graph):
    
    # -- Rule Initialization 
    rule_label = 'deduce individual and relation from restriction (recursively)' 
    
    # -- Init recursion parameters
    triple_list_length = 0
    recursion_max = 10
    recursion_count = 0
    recursion_condition = True
    
    # -- Recursion run
    while recursion_condition:
        recursion_count += 1
        _, rule_triple_list = deduce_individual_and_relation_from_restriction(graph)
        # print(f'{len(rule_triple_list)} | {triple_list_length} | {recursion_count} | {recursion_max}')
        recursion_condition = ((len(rule_triple_list) > triple_list_length) &
                               (recursion_count < recursion_max))  
        triple_list_length = len(rule_triple_list)
        for triple in rule_triple_list:
            graph.add(triple)
        
    return rule_label, rule_triple_list

 
