#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to conjunctive phenomena degree-91 (rule 1)
#------------------------------------------------------------------------------
# Extraction AMR rule to analyse Degree-91 phenomena (comparative and 
# superlative)
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_distinct_query
from transduction.rdfterm_computer import produce_uriref
from transduction.naming_computer import define_restriction_naming

ENTITY_CLASS_TYPE = 'base-out:Entity'
FEATURE_CLASS_TYPE = 'base-out:Feature'
DEGREE_CLASS_TYPE = 'base-out:Degree'
FEATURE_PROPERTY_NET_URI = 'net:predefinedProperty_hasFeature'
DEGREE_PROPERTY_NET_URI = 'net:predefinedProperty_hasDegree'



#==============================================================================
# Pattern Search: degree91(ARG1: entity, ARG2: attribute, ARG3: degree)
#==============================================================================

DEGREE_PHENOMENA_URI = 'amr:phenomena_degree'  
ARG_RELATION = [None, 'amr:role_ARG1', 'amr:role_ARG2', 'amr:role_ARG3',
                'amr:role_ARG4', 'amr:role_ARG5', 'amr:role_ARG6'] 
             
def __search_pattern(graph):
    select_data_list = ['?phenomena_net', 
                        '?entity_class_net', 
                        '?attribute_class_net', 
                        '?degree_property_net'
                        ]
    clause_list = ['?phenomena_net a [rdfs:subClassOf* net:Phenomena_Net].',
                   f'?phenomena_net net:hasPhenomenaType {DEGREE_PHENOMENA_URI}.'
                   f'?phenomena_net {ARG_RELATION[1]} ?entity_class_net.',
                   '?entity_class_net a [rdfs:subClassOf* net:Class_Net].',
                   'FILTER NOT EXISTS {{ ?entity_class_net a net:Deprecated_Net. }}',
                   f'?phenomena_net {ARG_RELATION[2]} ?attribute_class_net.',
                   '?attribute_class_net a [rdfs:subClassOf* net:Class_Net].',
                   f'?phenomena_net {ARG_RELATION[3]} ?degree_property_net.',
                   '?degree_property_net a [rdfs:subClassOf* net:Property_Net].'
                   ]
    query_code = generate_select_distinct_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set



#==============================================================================
# Useful Additional Search
#==============================================================================

def __search_degree_arg4(graph, degree_net_uri):
    select_data_list = ['?comparated_class_net']
    clause_list = [(degree_net_uri, ARG_RELATION[4], '?comparated_class_net'),
                   '?comparated_class_net a [rdfs:subClassOf* net:Class_Net].', 
                   'FILTER NOT EXISTS {{ ?comparated_class_net a net:Deprecated_Net. }}']
    query_code = generate_select_distinct_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set


def __search_degree_arg5(graph, degree_net_uri):
    select_data_list = ['?superlative_class_net']
    clause_list = [(degree_net_uri, ARG_RELATION[5], '?superlative_class_net'),
                   '?superlative_class_net a [rdfs:subClassOf* net:Class_Net].', 
                   'FILTER NOT EXISTS {{ ?superlative_class_net a net:Deprecated_Net. }}']
    query_code = generate_select_distinct_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set


def __search_degree_arg6(graph, degree_net_uri):
    select_data_list = ['?reference_net']
    clause_list = [(degree_net_uri, ARG_RELATION[6], '?reference_net'),
                   '?reference_net a [rdfs:subClassOf* net:Class_Net].',
                   'FILTER NOT EXISTS {{ ?reference_net a net:Deprecated_Net. }}']
    query_code = generate_select_distinct_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set


#==============================================================================
# Useful Computation Method(s)
#==============================================================================  

# -- Methods to define naming

def __extract_naming(net, default='none'):
    naming = f'{default}'
    naming_list = net.naming
    if len(naming_list) > 0:
        assert len(naming_list) == 1, 'Houston, we have a problem: too many naming'
        naming = naming_list[0]
    return naming

def __define_feature_restriction_naming(class_net):
    name = __extract_naming(class_net, default='something')
    return f'{name}-feature'

def __define_composite_naming(net_1, net_2):
    name_1 = __extract_naming(net_1, default='something')
    name_2 = __extract_naming(net_2, default='something')
    return f'{name_2}-{name_1}'

def __define_comparison_property_naming(net_1, net_2):
    name_1 = __extract_naming(net_1, default='something')
    if len(name_1) == 1: name_1 = name_1[0].upper()
    if len(name_1) >1: name_1 = name_1[0].upper() + name_1[1:]
    name_2 = __extract_naming(net_2, default='something')
    if len(name_2) == 1: name_2 = name_2[0].upper()
    if len(name_2) >1: name_2 = name_2[0].upper() + name_2[1:]
    return f'is{name_2}{name_1}Than'


# -- Methods to propagate relation

def __filter_relation(relation_list, phenomena_net):
    result_list = []
    for relation in relation_list:
        check = True
        (s, p, o) = relation
        if s == o: check = False
        if s == phenomena_net.uri: check = False
        if check: result_list.append(relation)
    return result_list

def __propagate_relation(target_net, base_net, phenomena_net):
    target_net.input_relation_list = base_net.input_relation_list
    out_relation_list = __filter_relation(base_net.output_relation_list, phenomena_net)
    target_net.output_relation_list = out_relation_list
    
    
    
#==============================================================================
# Construct Method(s)
#==============================================================================

def __construct_restriction_net(graph, property_net, target_class_net):
    
    # -- Net Composition
    restriction_net = net.RestrictionNet(graph)
    restriction_net.compose(property_net, target_class_net)
    
    # -- Data Computation
    restriction_net.restriction_property = property_net.uri
    restriction_net.restriction_net_value = target_class_net.uri

    # -- Relation Propagation: None

    # -- Net Naming
    restriction_net.naming = define_restriction_naming(property_net, target_class_net)
    
    # -- Finalization
    restriction_net.finalize()
    triple_list = restriction_net.generate_triple_definition()
    
    return restriction_net, triple_list


def __construct_feature_restriction_net(graph, individual_net):
    
    restriction_net = net.RestrictionNet(graph)
    restriction_net.compose(individual_net)
    
    # -- Data Computation
    restriction_net.restriction_property = produce_uriref(graph, FEATURE_PROPERTY_NET_URI)
    restriction_net.restriction_net_value = individual_net.uri

    # -- Relation Propagation: None

    # -- Net Naming
    restriction_net.naming = __define_feature_restriction_naming(individual_net)
    
    # -- Finalization
    restriction_net.finalize()
    triple_list = restriction_net.generate_triple_definition()
    
    return restriction_net, triple_list
    
    

def __construct_composite_class_net(graph, class_net_1, individual_net, phenomena_net):

    # -- Net Composition
    composite_class_net = net.CompositeClassNet(graph)
    composite_class_net.compose(class_net_1, individual_net)
    
    # -- Data Computation
    composite_class_net.mother_class_net = class_net_1.uri
    composite_class_net.root_class_net = class_net_1.root_class_net

    # -- Restriction Computation
    triple_list_1 = []
    restriction_net, triple_list_1 = __construct_feature_restriction_net(graph, individual_net)
    composite_class_net.restriction = restriction_net.uri 

    # -- Net Naming
    composite_class_net.naming = __define_composite_naming(class_net_1, individual_net)
    
    # -- Relation Propagation
    __propagate_relation(composite_class_net, class_net_1, phenomena_net)
    
    # -- Finalization
    composite_class_net.finalize()
    triple_list_2 = composite_class_net.generate_triple_definition()
    result_triple_list = triple_list_1 + triple_list_2
    
    return composite_class_net, result_triple_list
    
    

def __construct_degree_individual_net(graph, degree_property_net):

    # -- Net Composition
    individual_net = net.IndividualNet(graph)
    individual_net.compose(degree_property_net)
    
    # -- Data Computation
    individual_net.class_type = DEGREE_CLASS_TYPE
    # individual_net.mother_class_net = class_net.uri

    # -- Net Naming
    individual_net.naming = f'{__extract_naming(degree_property_net, default="something")}'
    individual_net.individual_label = f'{__extract_naming(degree_property_net, default="something")}'
    
    # -- Finalization
    individual_net.finalize()
    result_triple_list = individual_net.generate_triple_definition()
    
    return individual_net, result_triple_list  
    
    

def __construct_feature_individual_net(graph, feature_class_net, degree_individual_net):

    # -- Net Composition
    individual_net = net.IndividualNet(graph)
    individual_net.compose(feature_class_net, degree_individual_net)
    
    # -- Data Computation
    individual_net.class_type = FEATURE_CLASS_TYPE
    # individual_net.mother_class_net = class_net.uri

    # -- Net Naming
    individual_name = __define_composite_naming(feature_class_net, degree_individual_net)
    individual_net.naming = individual_name
    individual_net.individual_label = individual_name
    
    # -- Finalization
    individual_net.finalize()
    result_triple_list = individual_net.generate_triple_definition()
    
    return individual_net, result_triple_list    
    
    

def __construct_comparison_property_net(graph, feature_class_net, degree_individual_net):

    # -- Net Composition
    property_net = net.AtomPropertyNet(graph)
    property_net.compose(feature_class_net, degree_individual_net)
    
    # -- Relation Propagation : None

    # -- Net Naming
    property_net.naming = __define_comparison_property_naming(feature_class_net, degree_individual_net)
    
    # -- Finalization
    property_net.finalize()
    result_triple_list = property_net.generate_triple_definition()
    
    return property_net, result_triple_list
    

def __associate_restriction_relation(graph, class_net, restriction_net):
    class_net.restriction = restriction_net.uri
    result_triple_list = class_net.generate_triple_definition()
    return class_net, result_triple_list

def __associate_mother_relation_to_refine(graph, left_class_net, right_class_net):
    left_class_net.mother_class_net_to_refine = right_class_net.uri
    result_triple_list = left_class_net.generate_triple_definition()
    return left_class_net, result_triple_list


def __add_feature_reference_to_degree_net(graph, degree_net, feature_net):
    degree_net.feature_net = feature_net.uri
    return degree_net, degree_net.generate_triple_definition()


#==============================================================================
# Main Method
#==============================================================================

def analyze_phenomena_degree91_1(graph):
    
    # -- Rule Initialization 
    rule_label = 'analyze "degree-91" phenomena (1)' 
    rule_triple_list = []
    
    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Pattern Analysis
    for pattern in pattern_set:
        phenomena_net = net.PhenomenaNet(graph, uri=pattern.phenomena_net)
        entity_class_net = net.ClassNet(graph, uri=pattern.entity_class_net)
        attribute_class_net = net.ClassNet(graph, uri=pattern.attribute_class_net)
        degree_property_net = net.PropertyNet(graph, uri=pattern.degree_property_net)
        
        # -- New Net Construction(s)
        
        degree_individual_net, triple_list_1 = __construct_degree_individual_net(graph, degree_property_net)
        
        feature_individual_net, triple_list_2 = __construct_feature_individual_net(
            graph, attribute_class_net, degree_individual_net)
        
        new_entity_class_net, triple_list_3 = __construct_composite_class_net(
            graph, entity_class_net, feature_individual_net, phenomena_net)  
        
        rule_triple_list += (triple_list_1 + triple_list_2 + triple_list_3)
        
        # -- Update Degree Phenomena Net by adding feature net reference
        _, degree_triple_list = __add_feature_reference_to_degree_net(
            graph, phenomena_net, feature_individual_net)
        rule_triple_list += degree_triple_list
        
        # -- Comparison Handle (ARG4)
        comparison_property_net, triple_list_4a = __construct_comparison_property_net( 
            graph, attribute_class_net, degree_individual_net)
        _, comparated_class_set = __search_degree_arg4(graph, phenomena_net.uri)
        for row in comparated_class_set:
            comparated_class_net = net.ClassNet(graph, uri=row.comparated_class_net)
            restriction_net, triple_list_4b = __construct_restriction_net(
                graph, comparison_property_net, comparated_class_net)
            _, triple_list_4c = __associate_restriction_relation(graph, new_entity_class_net, restriction_net)
            rule_triple_list += (triple_list_4a + triple_list_4b + triple_list_4c)
        
        # -- Superlative Handle (ARG5)
        _, superlative_set = __search_degree_arg5(graph, phenomena_net.uri)
        for row in superlative_set:
            superlative_class_net = net.ClassNet(graph, uri=row.superlative_class_net)
            _, triple_list_5 = __associate_mother_relation_to_refine(
                graph, new_entity_class_net, superlative_class_net)
            rule_triple_list += triple_list_5
        
        # -- Reference Handle (ARG6)
        _, reference_set = __search_degree_arg6(graph, phenomena_net.uri)
        for row in reference_set:
            pass #TODO: uri=row.reference_net
        
        # -- Deprecation: Origin Class Nets
        rule_triple_list += entity_class_net.deprecate()
        rule_triple_list += attribute_class_net.deprecate()
        rule_triple_list += degree_property_net.deprecate()
    
    return rule_label, rule_triple_list