#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to analyze phenomena mod (rule 1)
#------------------------------------------------------------------------------
# Net Expansion AMR rule to analyse modifier phenomena (mod)
# Rule: mod(class, class) => compositeClass
#==============================================================================

import rdflib
from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_distinct_query
from transduction.rdfterm_computer import produce_uriref, produce_literal

ENTITY_CLASS_TYPE = 'base-out:Entity'
FEATURE_CLASS_TYPE = 'base-out:Feature'
FEATURE_PROPERTY_URI = 'base-out:hasFeature'
FEATURE_PROPERTY_NET_URI = 'net:predefinedProperty_hasFeature'


#==============================================================================
# Pattern Search: mod(class, class)
#==============================================================================

MOD_RELATION = 'amr:role_mod' 

def __search_pattern(graph):
    select_data_list = ['?class_net_1', '?class_net_2']
    clause_list = [f'?class_net_1 a [rdfs:subClassOf* net:Class_Net].',
                   f'?class_net_2 a [rdfs:subClassOf* net:Class_Net].',
                   f'?class_net_1 {MOD_RELATION} ?class_net_2.']
    query_code = generate_select_distinct_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set


#==============================================================================
# Useful Additional Search
#==============================================================================

def __search_mod(graph, class_net_uri):
    select_data_list = ['?mod_class_net']
    clause_list = [(class_net_uri, MOD_RELATION, '?mod_class_net'),
                   '?mod_class_net a [rdfs:subClassOf* net:Class_Net].', 
                   'FILTER NOT EXISTS {{ ?mod_class_net a net:Deprecated_Net. }}']
    query_code = generate_select_distinct_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set

    

#==============================================================================
# Useful Computation Method(s)
#============================================================================== 
  
def __extract_naming(net, default='none'):
    naming = f'{default}'
    naming_list = net.naming
    if len(naming_list) > 0:
        assert len(naming_list) == 1, 'Houston, we have a problem: too many naming'
        naming = naming_list[0]
    return naming

def __define_feature_restriction_naming(class_net):
    name = __extract_naming(class_net, default='something')
    return f'{name}-feature'

def __define_composite_naming(class_net_1, class_net_2_set):
    name_1 = __extract_naming(class_net_1, default='something')
    result_name = f'{name_1}'
    for class_net_2 in class_net_2_set:
        name_2 = __extract_naming(class_net_2, default='something')
        result_name = f'{name_2}-{result_name}'
    return result_name


def __filter_relation(relation_list):
    result_list = []
    for relation in relation_list:
        check = True
        (s, p, o) = relation
        if s == o: check = False
        if p == MOD_RELATION: check = False
        if check: result_list.append(relation)
    return result_list

def __propagate_relation(target_net, base_net):
    target_net.input_relation_list = base_net.input_relation_list
    out_relation_list = __filter_relation(base_net.output_relation_list)
    target_net.output_relation_list = out_relation_list
    
    
    
#==============================================================================
# Construct Method(s)
#============================================================================== 

def __construct_feature_restriction_net(graph, individual_net):
    
    restriction_net = net.RestrictionNet(graph)
    restriction_net.compose(individual_net)
    
    # -- Data Computation
    restriction_net.restriction_property = produce_uriref(graph, FEATURE_PROPERTY_NET_URI)
    restriction_net.restriction_net_value = individual_net.uri

    # -- Relation Propagation: None

    # -- Net Naming
    restriction_net.naming = __define_feature_restriction_naming(individual_net)
    
    # -- Finalization
    restriction_net.finalize()
    triple_list = restriction_net.generate_triple_definition()
    
    return restriction_net, triple_list
    
    

def __construct_composite_class_net(graph, class_net_1, feature_individual_net_list):

    # -- Net Composition
    composite_class_net = net.CompositeClassNet(graph)
    composing_net_list = [class_net_1] + feature_individual_net_list
    composite_class_net.compose(*composing_net_list)
    
    # -- Data Computation
    composite_class_net.mother_class_net = class_net_1.uri
    composite_class_net.root_class_net = class_net_1.root_class_net

    # -- Restriction Computation
    triple_list_1 = []
    restriction_list = []
    for feature_individual_net in feature_individual_net_list:
        restriction_net, triple_list = __construct_feature_restriction_net(graph, feature_individual_net)
        restriction_list.append(restriction_net.uri)
        triple_list_1 += triple_list
    composite_class_net.restriction = restriction_list

    # -- Net Naming
    composite_class_net.naming = __define_composite_naming(class_net_1, feature_individual_net_list)
    
    # -- Relation Propagation
    __propagate_relation(composite_class_net, class_net_1)
    
    # -- Finalization
    composite_class_net.finalize()
    triple_list_2 = composite_class_net.generate_triple_definition()
    result_triple_list = triple_list_1 + triple_list_2
    
    return composite_class_net, result_triple_list
    
    

def __construct_feature_individual_net(graph, class_net):

    # -- Net Composition
    individual_net = net.IndividualNet(graph)
    individual_net.compose(class_net)
    
    # -- Data Computation
    individual_net.class_type = FEATURE_CLASS_TYPE
    # individual_net.mother_class_net = class_net.uri

    # -- Net Naming
    individual_net.naming = f'{__extract_naming(class_net, default="something")}'
    individual_net.individual_label = f'{__extract_naming(class_net, default="something")}'
    
    # -- Finalization
    individual_net.finalize()
    result_triple_list = individual_net.generate_triple_definition()
    
    return individual_net, result_triple_list

  
    
#==============================================================================
# Main Method: analyze_phenomena_or_1
#==============================================================================

def analyze_phenomena_mod_1(graph):
    
    # -- Rule Initialization 
    rule_label = 'analyze modifier phenomena (mod)' 

    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Selection Analyzing (1)
    rule_triple_list = []
    for pattern in pattern_set:
        class_net_1 = net.ClassNet(graph, uri=pattern.class_net_1)
        
        # -- Feature Handle (MOD)
        _, mod_class_net_set = __search_mod(graph, class_net_1.uri)
        feature_individual_net_list = []
        for row in mod_class_net_set:
            mod_class_net = net.ClassNet(graph, uri=row.mod_class_net)
            individual_net, triple_list_1 = __construct_feature_individual_net(graph, mod_class_net)
            feature_individual_net_list.append(individual_net)
            rule_triple_list += triple_list_1
            rule_triple_list += mod_class_net.deprecate()
        
        # -- New Net Construction(s)
        _, triple_list_2 = __construct_composite_class_net(graph, class_net_1, feature_individual_net_list)  
        rule_triple_list += triple_list_2
        rule_triple_list += class_net_1.deprecate()
    
    return rule_label, rule_triple_list