#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to analyze a specific pattern
#------------------------------------------------------------------------------
# Extraction AMR Rule to analyze a specific pattern linking a degree-91 
# phenomena and a remain property
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query, generate_select_distinct_query
from transduction.rdfterm_computer import produce_uriref, produce_literal

ENTITY_CLASS_TYPE = 'base-out:Entity'
FEATURE_CLASS_TYPE = 'base-out:Feature'
DEGREE_CLASS_TYPE = 'base-out:Degree'
FEATURE_PROPERTY_NET_URI = 'net:predefinedProperty_hasFeature'
DEGREE_PROPERTY_NET_URI = 'net:predefinedProperty_hasDegree'



#==============================================================================
# Pattern Search: degree91(ARG1: entity, ARG2: attribute, ARG3: degree)
#==============================================================================

AND_PHENOMENA_URI = 'amr:phenomena_conjunction_and' 
DEGREE_PHENOMENA_URI = 'amr:phenomena_degree' 
REMAIN_PROPERTY_NAMING = '"remain"'  
OP_RELATION = [None, 'amr:role_op1', 'amr:role_op2'] 
             
def __search_pattern(graph):
    select_data_list = ['?and_phenomena_net',
                        '?degree_phenomena_net', 
                        '?remain_property_net']
    clause_list = ['?and_phenomena_net a [rdfs:subClassOf* net:Phenomena_Net].',
                   f'?and_phenomena_net net:hasPhenomenaType {AND_PHENOMENA_URI}.',
                   f'?and_phenomena_net {OP_RELATION[1]} ?degree_phenomena_net.',
                   '?degree_phenomena_net a [rdfs:subClassOf* net:Phenomena_Net].',
                   f'?degree_phenomena_net net:hasPhenomenaType {DEGREE_PHENOMENA_URI}.',
                   f'?and_phenomena_net {OP_RELATION[2]} ?remain_property_net.',
                   '?remain_property_net a [rdfs:subClassOf* net:Property_Net].',
                   f'?remain_property_net net:hasNaming {REMAIN_PROPERTY_NAMING}.']
    query_code = generate_select_distinct_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set



#==============================================================================
# Useful Additional Search
#==============================================================================

ARG_RELATION = [None, 'amr:role_ARG1', 'amr:role_ARG2', 'amr:role_ARG3',
                'amr:role_ARG4', 'amr:role_ARG5', 'amr:role_ARG6'] 


def __search_remain_arg1(graph, remain_net_uri):
    select_data_list = ['?remain_class_net']
    clause_list = [(remain_net_uri, ARG_RELATION[1], '?remain_class_net'),
                   '?remain_class_net a [rdfs:subClassOf* net:Class_Net].', 
                   'FILTER NOT EXISTS {{ ?remain_class_net a net:Deprecated_Net. }}']
    query_code = generate_select_distinct_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set


def __search_degree_feature(graph, degree_net_uri):
    select_data_list = ['?feature_individual_net']
    clause_list = [(degree_net_uri, 'net:hasFeatureNet', '?feature_individual_net'),
                   '?feature_individual_net a [rdfs:subClassOf* net:Individual_Net].', 
                   f'?feature_individual_net net:hasClassType {FEATURE_CLASS_TYPE}.',
                   'FILTER NOT EXISTS {{ ?feature_individual_net a net:Deprecated_Net. }}']
    query_code = generate_select_distinct_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set


def __search_degree_arg5(graph, degree_net_uri):
    select_data_list = ['?superlative_class_net']
    clause_list = [(degree_net_uri, ARG_RELATION[5], '?superlative_class_net'),
                   '?superlative_class_net a [rdfs:subClassOf* net:Class_Net].']
                   # 'FILTER NOT EXISTS {{ ?superlative_class_net a net:Deprecated_Net. }}']
    query_code = generate_select_distinct_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set


#==============================================================================
# Useful Computation Method(s)
#==============================================================================  

def __filter_mother_class(graph, class_net_uri_set):
    result_list = []
    
    class_net_uri_set_length = len(class_net_uri_set)
    for row1 in class_net_uri_set:
        class_net_uri_1 = row1.remain_class_net
        check = True
        for row2 in class_net_uri_set:
            class_net_uri_2 = row2.remain_class_net
            if class_net_uri_1 != class_net_uri_2:
                class_net_2 = net.ClassNet(graph, uri=class_net_uri_2)
                if class_net_uri_1 in class_net_2.mother_class_net:
                    check = False
        if check : result_list.append(class_net_uri_1)
       
    return result_list
    
    
def __extract_naming(net, default='none'):
    naming = f'{default}'
    naming_list = net.naming
    if len(naming_list) > 0:
        assert len(naming_list) == 1, 'Houston, we have a problem: too many naming'
        naming = naming_list[0]
    return naming


def __define_opposite_feature_naming(class_net):
    feature_name = __extract_naming(class_net, default='something')
    return f'not-{feature_name}'


def __define_feature_restriction_naming(class_net):
    name = __extract_naming(class_net, default='something')
    return f'{name}-feature'

def __define_composite_naming(net_1, net_2):
    name_1 = __extract_naming(net_1, default='something')
    name_2 = __extract_naming(net_2, default='something')
    return f'{name_2}-{name_1}'


def __filter_relation(relation_list, phenomena_net):
    result_list = []
    for relation in relation_list:
        check = True
        (s, p, o) = relation
        if s == o: check = False
        if s == phenomena_net.uri: check = False
        if check: result_list.append(relation)
    return result_list

def __propagate_relation(target_net, base_net, phenomena_net):
    target_net.input_relation_list = base_net.input_relation_list
    out_relation_list = __filter_relation(base_net.output_relation_list, phenomena_net)
    target_net.output_relation_list = out_relation_list
    
    
    
#==============================================================================
# Construct Method(s)
#==============================================================================

def __construct_opposite_feature_individual_net(graph, feature_individual_net):

    # -- Net Composition
    opposite_individual_net = net.IndividualNet(graph)
    opposite_individual_net.compose(feature_individual_net)
    
    # -- Data Computation
    opposite_individual_net.class_type = FEATURE_CLASS_TYPE
    # individual_net.mother_class_net = class_net.uri

    # -- Net Naming
    opposite_naming = __define_opposite_feature_naming(feature_individual_net)
    opposite_individual_net.naming = opposite_naming
    opposite_individual_net.individual_label = opposite_naming
    
    # -- Finalization
    opposite_individual_net.finalize()
    result_triple_list = opposite_individual_net.generate_triple_definition()
    
    return opposite_individual_net, result_triple_list  


def __construct_feature_restriction_net(graph, individual_net):
    
    restriction_net = net.RestrictionNet(graph)
    restriction_net.compose(individual_net)
    
    # -- Data Computation
    restriction_net.restriction_property = produce_uriref(graph, FEATURE_PROPERTY_NET_URI)
    restriction_net.restriction_net_value = individual_net.uri

    # -- Relation Propagation: None

    # -- Net Naming
    restriction_net.naming = __define_feature_restriction_naming(individual_net)
    
    # -- Finalization
    restriction_net.finalize()
    triple_list = restriction_net.generate_triple_definition()
    
    return restriction_net, triple_list
    
        

def __add_feature_restriction(graph, target_class_net, feature_individual_net):

    # -- Restriction Computation
    triple_list_1 = []
    restriction_net, triple_list_1 = __construct_feature_restriction_net(graph, feature_individual_net)
    target_class_net.restriction = restriction_net.uri 
    
    # -- Finalization
    target_class_net.finalize()
    triple_list_2 = target_class_net.generate_triple_definition()
    result_triple_list = triple_list_1 + triple_list_2
    
    return target_class_net, result_triple_list  
    


def __associate_mother_relation_to_refine(graph, left_class_net, right_class_net):
    left_class_net.mother_class_net_to_refine = right_class_net.uri
    result_triple_list = left_class_net.generate_triple_definition()
    return left_class_net, result_triple_list



#==============================================================================
# Main Method
#==============================================================================

def analyze_specific_pattern_1(graph):
    
    # -- Rule Initialization 
    rule_label = 'analyze a specific pattern (link between degree-91 and remain)' 
    rule_triple_list = []
    
    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Pattern Analysis
    for pattern in pattern_set:
        degree_phenomena_net = net.PhenomenaNet(graph, uri=pattern.degree_phenomena_net)
        remain_property_net = net.PropertyNet(graph, uri=pattern.remain_property_net)
                
        # -- Degree91 Feature Handle
        _, feature_individual_set = __search_degree_feature(graph, degree_phenomena_net.uri)
        feature_individual_net_list = []
        for row in feature_individual_set:
            feature_individual_net = net.IndividualNet(graph, uri=row.feature_individual_net)
            feature_individual_net, new_triples = ( 
                __construct_opposite_feature_individual_net(graph, feature_individual_net))
            feature_individual_net_list.append(feature_individual_net)
            rule_triple_list.extend(new_triples)
            
        # -- Search for target class net (remain ARG1)
        _, arg1_class_set = __search_remain_arg1(graph, remain_property_net.uri)
        target_class_list = __filter_mother_class(graph, arg1_class_set)
            
        # -- Adding Restriction on remain ARG1
        for feature_individual_net in feature_individual_net_list:
            for target_class_uri in target_class_list:
                target_class_net = net.ClassNet(graph, uri=target_class_uri)
                _, new_triples = __add_feature_restriction(graph, target_class_net, feature_individual_net)
                rule_triple_list.extend(new_triples)
        
        # -- Superlative Handle (ARG5)
        _, superlative_set = __search_degree_arg5(graph, degree_phenomena_net.uri)
        for row in superlative_set:
            superlative_class_net = net.ClassNet(graph, uri=row.superlative_class_net)
            for target_class_uri in target_class_list:
                target_class_net = net.ClassNet(graph, uri=target_class_uri)
                _, triple_list_5 = __associate_mother_relation_to_refine(
                    graph, target_class_net, superlative_class_net)
                rule_triple_list += triple_list_5
        
        # -- Deprecation: Origin Class Nets
        rule_triple_list += degree_phenomena_net.deprecate()
        rule_triple_list += remain_property_net.deprecate()
    
    return rule_label, rule_triple_list