#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: UNL CTR at 'OWL Generation' level
#------------------------------------------------------------------------------
# Module grouping compositional transduction rules (CTR) for the analysis of 
# UNL structures, at 'OWL Generation' level
#==============================================================================

#==============================================================================
# Prefix using in CTS
#==============================================================================

prefix = """
PREFIX owl: <http://www.w3.org/2002/07/owl#>
PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>
PREFIX unl: <https://unl.tetras-libre.fr/rdf/schema#>
PREFIX net: <https://tenet.tetras-libre.fr/semantic-net#>
PREFIX cprm: <https://tenet.tetras-libre.fr/config/parameters#>
PREFIX req: <https://tenet.tetras-libre.fr/frame/requirement-ontology#>
PREFIX sys: <https://tenet.tetras-libre.fr/system-ontology/>
PREFIX fprm: <https://tenet.tetras-libre.fr/frame/parameters#>
    
"""


#==============================================================================
# CTR Set (CTR as Sparql querries)
#==============================================================================

rule = {}


# ---------------------------------------------
# OWL Generation: Atom Class
# ---------------------------------------------

rule['generate-atom-class'] = {
    'label': "generate-atom-class",
    'comment': "Generate Atom Class in the target ontology",
    'order': 3.11,
    'query': """
    CONSTRUCT { 
       # Classification
       ?newClassUri rdfs:subClassOf ?parentUri.
       ?newClassUri rdfs:label ?atomConcept.
       ?newClassUri sys:from_structure ?req.
    }
    WHERE {
       # net1: atom
       ?net1 a net:Instance.
       ?net1 net:type net:atom.
       ?net1 net:has_structure ?req.
       ?net1 net:has_atom ?atomObject1.
       ?atomObject1 net:has_parent_class_uri ?parentUri.
       ?atomObject1 net:has_class_uri ?newClassUri.	
       ?atomObject1 net:has_concept ?atomConcept.	
       # Filter: atom not present in a composite list
       FILTER NOT EXISTS  { 
          ?net2 net:type net:list.
          ?net2 net:listOf net:composite.
          ?net2 net:has_item ?atomObject1.
       }
    }
    """
}


rule['generate-atom-instance'] = {
    'label': "generate-atom-instance",
    'comment': "Generate atom instances in the target ontology",
    'order': 3.12,
    'query': """
    CONSTRUCT { 
       # Instantiation
       ?newInstanceUri a ?classUri.
       ?newInstanceUri rdfs:label ?atomInstance.
       ?newInstanceUri sys:from_structure ?req.
    }
    WHERE {
       # net1: atom
       ?net1 a net:Instance.
       ?net1 net:type net:atom.
       ?net1 net:has_structure ?req.
       ?net1 net:has_atom ?atomObject1.
       ?atomObject1 net:has_class_uri ?classUri.	
       ?atomObject1 net:has_instance ?atomInstance. 
       ?atomObject1 net:has_instance_uri ?newInstanceUri.
       # Filter: entity not present in a class list
       FILTER NOT EXISTS  { ?net2 net:has_subClass ?atomConcept}
    }
    """
}


# ---------------------------------------------
# OWL Generation: Composite Class
# ---------------------------------------------

rule['generate-composite-class-from-list-net'] = {
    'label': "generate-composite-class-from-list-net",
    'comment': "Add Composite Class in Ontology (from Composite List net)",
    'order': 3.210,
    'query': """
    CONSTRUCT { 
       # Classification
       ?newClassUri rdfs:subClassOf ?parentUri.
       ?newClassUri rdfs:label ?compositeConcept.
       ?newClassUri sys:from_structure ?req.
       # Instantiation (extension)
       ?instanceUri rdf:type ?newClassUri.
       ?instanceUri sys:from_structure ?req.
    }
    WHERE {
       # net1: Composite List
       ?net1 a net:Instance.
       ?net1 net:type net:list.
       ?net1 net:listOf net:composite.
       ?net1 net:has_structure ?req.
       ?net1 net:has_parent ?parentObject.
       ?parentObject net:has_class_uri ?parentUri.
       # -- old --- ?parentObject net:has_instance_uri ?instanceUri.	
       ?net1 net:has_item ?compositeObject.	
       ?compositeObject net:has_class_uri ?newClassUri.		
       ?compositeObject net:has_concept ?compositeConcept.
    }
    """
}


rule['complement-composite-class'] = {
    'label': "complement-composite-class",
    'comment': "Complement composite classes with feature relation",
    'order': 3.22,
    'query': """
    CONSTRUCT { 
       # Complement with feature relation
       ?compositeClassUri sys:has_feature ?featureUri.
    }
    WHERE {
       # net1: Composite List
       ?net1 a net:Instance.
       ?net1 net:type net:list.
       ?net1 net:listOf net:composite.
       # -- Item
       ?net1 net:has_item ?itemObject.	
       ?itemObject net:has_class_uri ?compositeClassUri.	
       ?itemObject net:has_feature ?featureObject.	
       # -- Feature
       ?featureObject a net:Object.
       ?featureObject net:has_class_uri ?featureUri.	
    }
    """
}


rule['add-conjunctive-classes-from-list-net'] = {
    'label': "add-conjunctive-entity-classes",
    'comment': "Add disjunctive classes in Ontology",
    'order': 3.23,
    'query': """
    CONSTRUCT { 
       # Classification
       ?newClassUri rdfs:subClassOf ?parentUri.
       ?newClassUri rdfs:label ?newClassLabel.
       ?newClassUri sys:from_structure ?req.
       ?newClassUri  
    		owl:equivalentClass [	a owl:Class ;
                                    owl:intersectionOf ( ?item1Uri ?item2Uri ) ] .
       # Instantiation (extension)
       ?instanceUri rdf:type ?newClassUri.
    }
    WHERE {
       # net1: Composite List
       ?net1 a net:Instance.
       ?net1 net:type net:list.
       ?net1 net:listOf net:composite.
       ?net1 net:has_structure ?req.
       ?net1 net:has_node ?uw1, ?uw2.
       ?net1 net:has_parent ?parentObject.	
       ?parentObject net:has_class_uri ?parentUri.
       # -- old --- ?parentObject net:has_instance_uri ?instanceUri.	
       ?parentObject net:has_concept ?parentConcept.
       ?net1 net:has_item ?itemObject1, ?itemObject2.	
       ?itemObject1 net:has_class_uri ?item1Uri.
       ?itemObject1 net:has_node ?uw1.
       ?itemObject2 net:has_class_uri ?item2Uri.
       ?itemObject2 net:has_node ?uw2.
       # extension: disjunction of UW
       ?uw1 unl:and ?uw2.
       # Label(s) / URI (for classes)
       ?uw1 rdfs:label ?uw1Label.
       ?uw2 rdfs:label ?uw2Label.
       BIND (strbefore(?uw1Label, '(') AS ?concept1)
       BIND (strbefore(?uw2Label, '(') AS ?concept2)
       BIND (concat(?concept1, '-or-', ?concept2) AS ?disjunctiveConcept)
       BIND (concat(?disjunctiveConcept, '_', ?parentConcept) AS ?newClassLabel).
       # URI (for classes)
       cprm:Config_Parameters cprm:targetOntologyURI ?targetOntologyURI.
       BIND (concat(?targetOntologyURI, ?newClassLabel) AS ?s5).
       BIND (uri(?s5) AS ?newClassUri).
    }
    """
}

    
rule['add-disjunctive-classes-from-list-net'] = {
    'label': "add-disjunctive-entity-classes",
    'comment': "Add disjunctive classes in Ontology",
    'order': 3.23,
    'query': """
    CONSTRUCT { 
        # Classification
        ?newClassUri rdfs:subClassOf ?parentUri.
        ?newClassUri rdfs:label ?newClassLabel.
        ?newClassUri sys:from_structure ?req.
        ?newClassUri  
      		owl:equivalentClass [	a owl:Class ;
                                      owl:unionOf ( ?item1Uri ?item2Uri ) ] .
        # Instantiation (extension)
        ?instanceUri rdf:type ?newClassUri.
    }
    WHERE {
        # net1: Composite List
        ?net1 a net:Instance.
        ?net1 net:type net:list.
        ?net1 net:listOf net:composite.
        ?net1 net:has_structure ?req.
        ?net1 net:has_node ?uw1, ?uw2.
        ?net1 net:has_parent ?parentObject.	
        ?parentObject net:has_class_uri ?parentUri.
        # -- old --- ?parentObject net:has_instance_uri ?instanceUri.	
        ?parentObject net:has_concept ?parentConcept.
        ?net1 net:has_item ?itemObject1, ?itemObject2.	
        ?itemObject1 net:has_class_uri ?item1Uri.
        ?itemObject1 net:has_node ?uw1.
        ?itemObject2 net:has_class_uri ?item2Uri.
        ?itemObject2 net:has_node ?uw2.
        # extension: disjunction of UW
        ?uw1 unl:or ?uw2.
        # Label(s) / URI (for classes)
        ?uw1 rdfs:label ?uw1Label.
        ?uw2 rdfs:label ?uw2Label.
        BIND (strbefore(?uw1Label, '(') AS ?concept1)
        BIND (strbefore(?uw2Label, '(') AS ?concept2)
        BIND (concat(?concept1, '-or-', ?concept2) AS ?disjunctiveConcept)
        BIND (concat(?disjunctiveConcept, '_', ?parentConcept) AS ?newClassLabel).
        # URI (for classes)
        	cprm:Config_Parameters cprm:targetOntologyURI ?targetOntologyURI.
        BIND (concat(?targetOntologyURI, ?newClassLabel) AS ?s5).
        BIND (uri(?s5) AS ?newClassUri).
    }
    """
}


rule['generate-event-class'] = {
    'label': "generate-event-class",
    'comment': "Add Event Class in Target Ontology",
    'order': 3.25,
    'query': """
    CONSTRUCT { 
       # Classification
       ?newEventUri rdfs:subClassOf ?eventClassUri.
       ?newEventUri rdfs:label ?eventLabel.
       ?newEventUri sys:from_structure ?req.
    }
    WHERE {
       # Relation Net (net1)
       ?net1 a net:Instance.
       ?net1 net:type net:relation.
       ?net1 net:relationOf ?relationMother.
       ?net1 net:has_structure ?req.
       ?net1 net:has_node ?uw1.
       # -- Relation Object
       ?net1 net:has_relation ?relationObject.	
       ?relationObject net:has_concept ?relationConcept.
       # -- Source Object
       ?net1 net:has_source ?sourceObject.
       ?sourceObject net:has_concept ?sourceConcept.	
       # -- Target Object
       ?net1 net:has_target ?targetObject1.
       ?targetObject1 net:has_concept ?targetConcept.
       # Label: event
       BIND (concat(?sourceConcept, '-', ?relationConcept) AS ?e1).
       BIND (concat(?e1, '-', ?targetConcept) AS ?eventLabel).
       # URI (for classes)
    	cprm:Config_Parameters cprm:targetOntologyURI ?targetOntologyURI.
       sys:Event sys:is_class ?eventClass.
       BIND (concat( ?targetOntologyURI, ?eventClass) AS ?c1).
       BIND (concat(?c1, '_', ?eventLabel) AS ?c2).
       BIND (uri( ?c1) AS ?eventClassUri).
       BIND (uri(?c2) AS ?newEventUri).
    }
    """
}


# ---------------------------------------------
# OWL Generation: Relation Property 
# ---------------------------------------------

rule['generate-relation-property'] = {
    'label': "generate-relation-property",
    'comment': "Generate Relation Property in Target Ontology",
    'order': 3.31,
    'query': """
    CONSTRUCT { 
       # update: Relation Property
       ?newPropertyUri a owl:ObjectProperty.
       ?newPropertyUri rdfs:subPropertyOf ?parentProperty.
       ?newPropertyUri rdfs:label ?relationConcept.
       ?newPropertyUri sys:from_structure ?req.
    }
    WHERE {
       # Relation Net (net1)
       ?net1 a net:Instance.
       ?net1 net:type net:relation.
       ?net1 net:relationOf ?relationMother.
       ?net1 net:has_structure ?req.
       # -- Relation Object
       ?net1 net:has_relation ?relationObject.	
       ?relationObject net:has_parent_property ?parentProperty.
       ?relationObject net:has_property_uri ?newPropertyUri.
    }
    """
}


rule['compute-domain-of-relation-property'] = {
    'label': "compute-domain-of-relation-property",
    'comment': "Compute Domain of Relation Property",
    'order': 3.32,
    'query': """
    CONSTRUCT { 
       # update Relation Property: add domain
       ?newPropertyUri rdfs:domain ?domainClassUri.
    }
    WHERE {
       # Relation Net (net1)
       ?net1 a net:Instance.
       ?net1 net:type net:relation.
       ?net1 net:relationOf ?relationMother.
       ?net1 net:has_relation ?relationObject.	
       ?relationObject net:has_property_uri ?newPropertyUri.
       # -- Domain  
       ?net1 net:has_possible_domain ?possibleDomainLabel1.
    	?domainClass rdfs:label ?possibleDomainLabel1.
       FILTER (
    		NOT EXISTS {
    			?net1 net:has_possible_domain ?possibleDomainLabel2 .
    	        ?anotherDomainClass rdfs:label ?possibleDomainLabel2 .
    			?anotherDomainClass rdfs:subClassOf ?domainClass.
    		}
    	)
       BIND (uri( ?domainClass) AS ?domainClassUri).
    }
    """
}


rule['compute-range-of-relation-property'] = {
    'label': "compute-range-of-relation-property",
    'comment': "Compute Range of Relation Property",
    'order': 3.32,
    'query': """
    CONSTRUCT { 
       # update Relation Property: add range
       ?newPropertyUri rdfs:range ?rangeClassUri.
    }
    WHERE {
       # Relation Net (net1)
       ?net1 a net:Instance.
       ?net1 net:type net:relation.
       ?net1 net:relationOf ?relationMother.
       ?net1 net:has_relation ?relationObject.	
       ?relationObject net:has_property_uri ?newPropertyUri.
       # -- Range  
       ?net1 net:has_possible_range ?possibleRangeLabel1.
       ?rangeClass rdfs:label ?possibleRangeLabel1 .
       FILTER (
    		NOT EXISTS {
    			?net1 net:has_possible_domain ?possibleRangeLabel2 .
    	       ?anotherRangeClass rdfs:label ?possibleRangeLabel2 .
    			?anotherRangeClass rdfs:subClassOf ?rangeClass.
    		}
       )
       BIND (uri( ?rangeClass) AS ?rangeClassUri).
    }
    """
}


rule['link-instances-by-relation-property'] = {
    'label': "link-instances-by-relation-property",
    'comment': "Link instances by relation property",
    'order': 3.33,
    'query': """
    CONSTRUCT { 
       ?sourceInstanceUri ?newPropertyUri ?targetInstanceUri.
    }
    WHERE {
       # Relation Net (net1)
       ?net1 a net:Instance.
       ?net1 net:type net:relation.
       ?net1 net:relationOf ?relationMother.
       ?net1 net:has_structure ?req.
       # -- Relation Object
       ?net1 net:has_relation ?relationObject.	
       ?relationObject net:has_parent_property ?parentProperty.
       ?relationObject net:has_property_uri ?newPropertyUri.
       # -- Source Object
       ?net1 net:has_source ?sourceObject.
       ?sourceObject net:has_instance_uri ?sourceInstanceUri.  
       # -- Target Object
       ?net1 net:has_target ?targetObject.
       ?targetObject net:has_instance_uri ?targetInstanceUri. 
    }
    """
}