#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Net
#------------------------------------------------------------------------------
# Class to handle semantic nets 
#==============================================================================

import rdflib
from rdflib import URIRef, Literal, BNode
from rdflib.namespace import FOAF, RDF
from rdflib.namespace import NamespaceManager
from rdflib.term import _is_valid_uri

import copy

from transduction.net import SemanticNetReferenceHandle
from transduction.rdfterm_computer import produce_uriref, produce_literal
from transduction.query_builder import generate_select_query, generate_ask_query


#==============================================================================
# Class
#==============================================================================

class Net:
    """ Class to handle semantic net.
    """    
    
    anonymous_number = 0
    cloudy_number = 0
    DEPRECATED_NET = 'net:Deprecated_Net'
    
    
    #--------------------------------------------------------------------------
    # Constructor
    #--------------------------------------------------------------------------
              
    def __init__(self, support_graph, uri=None):
        
        # -- Net Support
        self.support_graph = support_graph
        self.referenceHandle = SemanticNetReferenceHandle()
        
        # -- Net Type
        self.type_name = 'default'
        self.type_id = 'Net'
        self.type_uri = produce_uriref(self.support_graph, f'net:{self.type_id}')
        self._deprecated_state = None 
        
        # -- Net URI
        self._uri = produce_uriref(self.support_graph, uri)        
           
        # -- Net Attributes
        self.attr_list = ['node', 'base_node', 'net_composition', 'structure', 'naming']
        self._node = []
        self._base_node = None
        self._net_composition = []
        self._structure = None
        self._naming = None
        
        # -- Net Relations
        self._input_relation_list = None
        self._output_relation_list = None
  
        
  
    #--------------------------------------------------------------------------
    # Accessor Base Methods
    #--------------------------------------------------------------------------
    
    # -- getter
    
    def __find_values_in_graph(self, attr_key):
        select_data_list = ['?attr_value']
        predicate = self.referenceHandle.get_predicate_uriref(self.support_graph, attr_key)
        clause_list = [(self._uri, predicate, '?attr_value')]
        query_code = generate_select_query(self.support_graph, select_data_list, clause_list)
        query_result = self.support_graph.query(query_code)
        return query_result
    
    
    def get_value_list_from_graph(self, attr_key):
        
        res = []
        
        if (self.uri is not None):
            value_set = self.__find_values_in_graph(attr_key)
            for v in value_set:
                res.append(v.attr_value)    
                
        return res
    
    
    def get_attribute_first_value(self, attribute):
        res = None
        if isinstance(attribute, list) and len(attribute) > 0:
            res = attribute[0]
        return res
    

    # -- setter
        
    def set_attribute_value_list(self, attr_value_list, rdfterm_function):
        
        res = []
        
        if attr_value_list is not None:
            
            if not(isinstance(attr_value_list, list)): 
                attr_value_list = [attr_value_list]
                
            for attr_value in attr_value_list:
                attr_value = rdfterm_function(self.support_graph, attr_value)
                res.append(attr_value)
              
        return res


    #--------------------------------------------------------------------------
    # Accessors for Definition Information (net URI, deprecated state, ...)
    #--------------------------------------------------------------------------
        
    # -- Net URI
    
    @property
    def uri(self):
        return self._uri       
    
    @uri.setter
    def uri(self, new_value):
        self._uri = produce_uriref(self.support_graph, new_value) 
    

    # -- Anonymous or cloudy identifier 
    
    @staticmethod
    def get_anonymous_identifier():
        Net.anonymous_number += 1
        return f'anonymous{Net.anonymous_number}'
        
    
    def set_cloudy_uri(self):
        Net.cloudy_number += 1
        self.uri = f'{self.uri}_cloudy{Net.cloudy_number}'
        
        
    # -- Deprecated State
    
    def __define_deprecated_clause(self):
        rdf_type_uri = produce_uriref(self.support_graph, RDF.type)
        deprecated_net_class_uri = produce_uriref(self.support_graph, Net.DEPRECATED_NET)
        return (self._uri, rdf_type_uri, deprecated_net_class_uri)
    
    
    def is_deprecated(self):
        
        if self._deprecated_state is None:
            clause_list = [self.__define_deprecated_clause()]
            query_code = generate_ask_query(self.support_graph, clause_list)
            self._deprecated_state = self.support_graph.query(query_code).askAnswer
            
        return self._deprecated_state
    
    
    def deprecate(self):
        self._deprecated_state = True
        return [self.__define_deprecated_clause()]
 
       
    #--------------------------------------------------------------------------
    # Accessors for Net Attributes
    #--------------------------------------------------------------------------
        
    @property
    def node(self):
        if self._node == []: 
            self._node = self.get_value_list_from_graph('node')
        return self._node
    
    @node.setter
    def node(self, new_value):
        self._node = self.set_attribute_value_list(new_value, produce_uriref)
        
        
    @property
    def base_node(self):
        if self._base_node is None: 
            self._base_node = self.get_value_list_from_graph('base_node')
        return self._base_node
    
    @base_node.setter
    def base_node(self, new_value):
        self._base_node = self.set_attribute_value_list(new_value, produce_uriref)
        
        
    @property
    def net_composition(self):
        if self._net_composition == []: 
            self._net_composition = self.get_value_list_from_graph('net_composition')
        return self._net_composition
    
    @net_composition.setter
    def net_composition(self, new_value):
        self._net_composition = self.set_attribute_value_list(new_value, produce_uriref)
        
        
    @property
    def structure(self):
        if self._structure is None: 
            self._structure = self.get_value_list_from_graph('structure')
        return self._structure
    
    @structure.setter
    def structure(self, new_value):
        self._structure = self.set_attribute_value_list(new_value, produce_literal)
        
    @property
    def naming(self):
        if self._naming is None: 
            self._naming = self.get_value_list_from_graph('naming')
        return self._naming
    
    @naming.setter
    def naming(self, new_value):
        self._naming = self.set_attribute_value_list(new_value, produce_literal)
    
        
    #--------------------------------------------------------------------------
    # Accessors for Net Relations
    #--------------------------------------------------------------------------
    
    # -- input relations
    
    def __find_input_relations_in_graph(self):
        select_data_list = ['?in_net' '?in_rel']
        rdf_type_uri = produce_uriref(self.support_graph, RDF.type)
        clause_list = [('?in_rel', rdf_type_uri, 'net:Relation')]
        clause_list.append(('?in_net', '?in_rel', self.uri))
        # print(f' *** DEVTEST *** {clause_list}')
        query_code = generate_select_query(self.support_graph, select_data_list, clause_list)
        # print(f' *** DEVTEST *** {query_code}')
        query_result = self.support_graph.query(query_code)
        return query_result
    
    @property
    def input_relation_list(self):
        if self._input_relation_list is None: 
            self._input_relation_list = []
            if self.uri is not None:
                query_result = self.__find_input_relations_in_graph()
                for row in query_result:
                    relation = (row.in_net, row.in_rel, self.uri)
                    self._input_relation_list.append(relation)
        return self._input_relation_list
    
    @input_relation_list.setter
    def input_relation_list(self, relation_list):
        self._input_relation_list = []
        for (in_net, in_rel, _) in relation_list:
            relation = (in_net, in_rel, self.uri)
            self._input_relation_list.append(relation)
        
        
    # -- output relations
    
    def __find_output_relations_in_graph(self):
        select_data_list = ['?out_rel', '?out_net']
        rdf_type_uri = produce_uriref(self.support_graph, RDF.type)
        clause_list = [('?out_rel', rdf_type_uri, 'net:Relation')]
        clause_list.append((self.uri, '?out_rel', '?out_net'))
        # print(f' *** DEVTEST *** {clause_list}')
        query_code = generate_select_query(self.support_graph, select_data_list, clause_list)
        # print(f' *** DEVTEST *** {query_code}')
        query_result = self.support_graph.query(query_code)
        return query_result
    
    @property
    def output_relation_list(self):
        if self._output_relation_list is None: 
            self._output_relation_list = []
            if self.uri is not None:
                query_result = self.__find_output_relations_in_graph()
                for row in query_result:
                    relation = (self.uri, row.out_rel, row.out_net)
                    self._output_relation_list.append(relation)
        return self._output_relation_list
    
    @output_relation_list.setter
    def output_relation_list(self, relation_list):
        self._output_relation_list = []
        for (_, out_rel, out_net) in relation_list:
            relation = (self.uri, out_rel, out_net)
            self._output_relation_list.append(relation)
        
        
    #--------------------------------------------------------------------------
    # Composition Method
    #--------------------------------------------------------------------------
    
    def compose(self, *nets):
        
        assert len(nets) > 0, 'Houston, we have a problem: at least one net is needed to compose a new net'
        
        # -- Base Node and Structure Setting
        self.base_node = nets[0].base_node
        self.structure = nets[0].structure
        
        # -- Node(s) Setting
        net_node_list = []
        for n in nets:
            net_node_list += n.node
        self.node = net_node_list
        
        # -- Net Composition
        self._net_composition = []
        for n in nets:
            self._net_composition.append(n.uri) 
        
        
    #--------------------------------------------------------------------------
    # Consolidation Method (finalize)
    #--------------------------------------------------------------------------
    
    def __get_node_value(self, node_uri):
        
        node_value = f'blankNode3'
        graph = self.support_graph
        
        # -- query
        select_data_list = ['?value']
        clause_list = []
        clause_list.append((node_uri, 'amr:hasVariable','?variable'))
        clause_list.append(f'?variable amr:label ?value.')
        query_code = generate_select_query(graph, select_data_list, clause_list)
        
        # -- searching
        result_set = graph.query(query_code)
        for res in result_set:
            node_value = res.value
        
        return node_value

    
    def __consolidate_uri(self):
        
        if self.uri is None:
            
            # -- Net Naming
            if not (isinstance(self.naming, list) and len(self.naming) > 0):
                self.naming = Net.get_anonymous_identifier()
            net_naming = self.naming[0]
            
            # -- Net Node Reference
            if self.base_node is None:
                net_node_ref = f'blankNode'
            elif len(self.base_node) <= 0:
                net_node_ref = f'blankNode'
            else:
                assert len(self.base_node) > 0, f'Houston, we have a problem!'
                net_node_ref = self.__get_node_value(self.base_node[0])                
            
            self.uri = URIRef(f'net:{self.type_name}_{net_naming}_{net_node_ref}')
    
    
    def __update_relations(self):
        self.input_relation_list = self.input_relation_list
        self.output_relation_list = self.output_relation_list
        
        
    def finalize(self):
        self.__consolidate_uri()
        self.type_uri = produce_uriref(self.support_graph, self.type_uri)
        self.__update_relations()
        
        
        
    #--------------------------------------------------------------------------
    # Triple Generation Method
    #-------------------------------------------------------------------------- 

    # -- Net Declaration
    
    def generate_net_declaration_triple(self):
        
        triple_list = []
        
        rdf_type_uri = produce_uriref(self.support_graph, RDF.type)
        triple_list.append((self.uri, rdf_type_uri, self.type_uri))
        
        if self.is_deprecated():
            triple_list.append(self.__define_deprecated_clause())
        
        return triple_list
      
        
    # -- Net Attribute(s)  
            
    def __get_attr_value_list(self, attr_key):
        attr_value_list = []
        
        attr_key_str = f'_{attr_key}'
        
        if attr_key_str in self.__dict__.keys():
            attr_value_list = eval(f'self.{attr_key_str}')
            
        if attr_value_list is None: attr_value_list = []
        
        return attr_value_list
    
    
    def __define_attribute_triple_list(self, attr_key):
        
        attr_triple_list = []
        
        attr_predicate = self.referenceHandle.get_predicate_uriref(self.support_graph, attr_key)
        attr_value_list = self.__get_attr_value_list(attr_key)
        for attr_value in attr_value_list:
            attr_triple_list.append((self.uri, attr_predicate, attr_value))
            
        return attr_triple_list
    
    
    def generate_net_attribute_triples(self):
        
        triple_list = []
        
        for attr_key in self.attr_list:
            attr_triple_list = self.__define_attribute_triple_list(attr_key)
            triple_list += attr_triple_list
        
        return triple_list
    
    
    # -- Net Relation(s)
    
    def generate_net_relation_triples(self): 
        
        triple_list = []
        
        rdf_type_uri = produce_uriref(self.support_graph, RDF.type) 
        net_relation_uri = produce_uriref(self.support_graph, 'net:Relation')
        for triple in self.input_relation_list + self.output_relation_list:
            _, relation, _ = triple
            triple_list.append((relation, rdf_type_uri, net_relation_uri))
            triple_list.append(triple)
            
        return triple_list
    
    
    # -- Generation Triple Definition
    
    def generate_triple_definition(self):
        
        triple_list = []
        
        if self.uri is not None:
            triple_list += self.generate_net_declaration_triple()
            triple_list += self.generate_net_attribute_triples()
            triple_list += self.generate_net_relation_triples()
            
        return triple_list
        
        
        
    #--------------------------------------------------------------------------
    # Managing Method(s)
    #-------------------------------------------------------------------------- 
    
    # -- Method to copy the net
    
    def duplicate_as_cloudy_net(self):
        new_net = copy.copy(self)
        new_net.set_cloudy_uri()
        new_net.node = None
        new_net.base_node = None
            
        return new_net