#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: SPARQL Query Builder
#------------------------------------------------------------------------------
# ...
#==============================================================================

from rdflib import Graph, Namespace
from rdflib.plugins.sparql import prepareQuery
from transduction.rdfterm_computer import produce_sparql_term

# -- Useful Constant(s)
DEFAULT_ATTRIBUTE_VALUE = f'\"NA\"'
INDENT_STR = '            '
SENTENCE_REF = '?sentenceRef'

# -- RDF Prefix List
PREFIX_LIST = [('owl', 'http://www.w3.org/2002/07/owl#'),
               ('rdf', 'http://www.w3.org/1999/02/22-rdf-syntax-ns#'),
               ('rdfs', 'http://www.w3.org/2000/01/rdf-schema#'),
               ('xsd', 'http://www.w3.org/2001/XMLSchema#'),
               ('amr', 'https://amr.tetras-libre.fr/rdf/schema#'),
               ('ns1', 'http://amr.isi.edu/frames/ld/v1.2.2/'),
               ('ns2', 'http://amr.isi.edu/rdf/amr-terms#'),
               ('ns3', 'http://amr.isi.edu/rdf/core-amr#'),
               ('ns4', 'http://amr.isi.edu/entity-types#'),
               ('net', 'https://tenet.tetras-libre.fr/semantic-net#'),
               ('cprm', 'https://tenet.tetras-libre.fr/config/parameters#'),
               ('fprm', 'https://tenet.tetras-libre.fr/frame/parameters#'),
               ('base-out', 'https://tenet.tetras-libre.fr/base-ontology#'),
               ('ext-out', 'https://tenet.tetras-libre.fr/extract-result#')]


# RDF Prefix Definitions
rdf = Namespace("http://www.w3.org/1999/02/22-rdf-syntax-ns#")
rdfs = Namespace("http://www.w3.org/2000/01/rdf-schema#")
net = Namespace("https://tenet.tetras-libre.fr/semantic-net#")
               


#==============================================================================
# Method to generate a generic SPARQL SELECT query
#==============================================================================

def get_prefix_def():
    prefix_def = ''
    for (ident, iri) in PREFIX_LIST:
        prefix_def += f'PREFIX {ident}: <{iri}>\n'
    return prefix_def


def __define_clause_part(graph, clause_list):
    
    clause_part = ''
    
    for clause in clause_list:
        
        if isinstance(clause, tuple):
            (subject, predicate, object) = clause
            subject = produce_sparql_term(graph, subject)
            predicate = produce_sparql_term(graph, predicate)
            object = produce_sparql_term(graph, object)
            clause_part += f'\n    {subject} {predicate} {object}.'
            
        if isinstance(clause, str):
            clause_part += f'\n    {clause}'
            
    return clause_part


def generate_select_query(graph, select_data_list, clause_list):
    """ Generate a SELECT Sparql query with a select part (from select_data_list)
    and a clause part (from clause_list).

    Parameters
    ----------
    graph : RDFLIB Graph
        target graph of the query.
    select_data_list : List
        list of sparql variables for the select part.
    clause_list : List
        list of clauses (as rdf triples or string) for the clause part.

    Returns
    -------
    query_code : SPARQL query (string)
        SELECT Sparql query.

    """
    
    prefix = get_prefix_def()
    
    first = True
    select_data_part = ''
    for data in select_data_list:
        if not first: select_data_part += ' '
        select_data_part += f'{data}' 
        first = False
    
    clause_part = __define_clause_part(graph, clause_list)       
    
    query_code = f'{prefix}'
    query_code += f'\n SELECT {select_data_part}'
    query_code += f'\n WHERE {{ {clause_part} \n }}'
    
    return query_code   


def generate_select_distinct_query(graph, select_data_list, clause_list):
    """ Generate a SELECT Sparql query with a select part (from select_data_list)
    and a clause part (from clause_list).

    Parameters
    ----------
    graph : RDFLIB Graph
        target graph of the query.
    select_data_list : List
        list of sparql variables for the select part.
    clause_list : List
        list of clauses (as rdf triples or string) for the clause part.

    Returns
    -------
    query_code : SPARQL query (string)
        SELECT Sparql query.

    """
    
    prefix = get_prefix_def()
    
    first = True
    select_data_part = ''
    for data in select_data_list:
        if not first: select_data_part += ' '
        select_data_part += f'{data}' 
        first = False
    
    clause_part = __define_clause_part(graph, clause_list)       
    
    query_code = f'{prefix}'
    query_code += f'\n SELECT DISTINCT {select_data_part}'
    query_code += f'\n WHERE {{ {clause_part} \n }}'
    
    return query_code  



#==============================================================================
# Method to generate a generic SPARQL ASK query
#==============================================================================


def generate_ask_query(graph, clause_list):
    
    prefix = get_prefix_def()
    
    clause_part = __define_clause_part(graph, clause_list)
    
    query_code = f'{prefix}'
    query_code += f'\n ASK {{ {clause_part} \n }}'
    
    return query_code   
