#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Naming Computer
#------------------------------------------------------------------------------
# Class grouping methods to compute URI for semantic nets (and their 
# attributes)
#==============================================================================

import rdflib
from rdflib import URIRef, Graph
from rdflib.term import _is_valid_uri

PREFIX_LIST = [('owl', 'http://www.w3.org/2002/07/owl#'),
               ('odrl', 'http://www.w3.org/ns/odrl/2/'),
               ('cc', 'https://creativecommons.org/ns#'),
               ('rdf', 'http://www.w3.org/1999/02/22-rdf-syntax-ns#'),
               ('rdfs', 'http://www.w3.org/2000/01/rdf-schema#'),
               ('xsd', 'http://www.w3.org/2001/XMLSchema#'),
               ('amr', 'https://amr.tetras-libre.fr/rdf/schema#'),
               ('ns1', 'http://amr.isi.edu/frames/ld/v1.2.2/'),
               ('ns2', 'http://amr.isi.edu/rdf/amr-terms#'),
               ('ns3', 'http://amr.isi.edu/rdf/core-amr#'),
               ('ns4', 'http://amr.isi.edu/entity-types#'),
               ('net', 'https://tenet.tetras-libre.fr/semantic-net#'),
               ('cprm', 'https://tenet.tetras-libre.fr/config/parameters#'),
               ('fprm', 'https://tenet.tetras-libre.fr/frame/parameters#'),
               ('base-out', 'https://tenet.tetras-libre.fr/base-ontology#'),
               ('ext-out', 'https://tenet.tetras-libre.fr/extract-result#')]



#==============================================================================
# Method to produce URIRef
#==============================================================================

def __update_uri_with_prefix(graph, uri):
    assert isinstance(uri, str), f'Houston, we have a problem: uri is not a string'
    
    # nsm = graph.namespace_manager
    # if uri.startswith("http") and uri in nsm:
    #     uri = rdflib.URIRef(uri.n3(nsm))
      
    for (prefix, namespace) in PREFIX_LIST:
        assert len(prefix) > 0, f'Houston, we have a problem: prefix is None'
        assert len(namespace) > 0, f'Houston, we have a problem: namespace is None'
        if uri.startswith("http") and uri.startswith(namespace):
            uri = uri.replace(namespace, f'{prefix}:')
    
    return uri


def __update_uri_with_namespace(graph, uri):
    assert isinstance(uri, str), f'Houston, we have a problem: uri is not a string'
    
    # nsm = graph.namespace_manager
    # for (prefix, namespace) in nsm.namespaces():
    #     if len(prefix) > 0 and uri.startswith(prefix):
    #         uri = uri.replace(f'{prefix}:', namespace)
            
    for (prefix, namespace) in PREFIX_LIST:
        assert len(prefix) > 0, f'Houston, we have a problem: prefix is None'
        assert len(namespace) > 0, f'Houston, we have a problem: namespace is None'
        if uri.startswith(f'{prefix}:'):
            uri = uri.replace(f'{prefix}:', namespace)
    
    return uri


def produce_uriref(graph, uri):
    """Create a valid URIRef with some necessary checks and adaptations from an
    URI string instance. 
    
    None is returned is input URI is None.
    
    The RDFLib _is_valid_uri only checks for invalid 
    characters. Some additional specific checks are also performed.
    """
    
    if uri is not None:
        assert isinstance(uri, str), f'Houston, we have a problem: uri is not a string'
        
        if (uri.startswith("<") & uri.endswith(">")): 
            uri = uri[1:-1]
        
        uri = __update_uri_with_namespace(graph, uri)
        
        if not isinstance(uri, rdflib.term.URIRef):
            uri = rdflib.URIRef(uri)
           
        assert isinstance(uri, rdflib.term.URIRef), f'Houston, we have a problem: result uri is not an URIRef'
        assert _is_valid_uri(uri), f'Houston, we have a problem: URI is not a valid uri'
    
    return uri



#==============================================================================
# Method to produce Literal, Boolean
#==============================================================================

def produce_literal(graph, value):
    literal = None
    if isinstance(value, str):
        literal = rdflib.Literal(value)
    return literal


# def produce_boolean(graph, value):
#     boolean = None
#     if isinstance(value, str):
#         boolean = rdflib.Boolean(value)
#     elif (isinstance(value, tuple) & len(value) > 0):
#         value = value[0]
#         assert isinstance(value, str), f'Houston, we have a problem: value is not a string' 
#         boolean = rdflib.Boolean(value)
#     return boolean



#==============================================================================
# Method to produce RDF or SPARQL term
#==============================================================================

def produce_rdf_term(graph, term):
    """Create a term for RDF graph managing with some necessary checks and 
    adaptations from a RDF term (uriref or literal). 
    """
    
    is_literal = isinstance(term, rdflib.term.Literal)
    is_uriref = isinstance(term, rdflib.term.URIRef)
    is_valid_term = is_literal or is_uriref
    assert is_valid_term, f'Houston, we have a problem: term is not valid'
    
    if is_uriref:
        __update_uri_with_namespace(graph, uri)
        
    return term
    
    
    
def produce_sparql_term(graph, term):
    """Create a term for SPARQL query with some necessary checks and 
    adaptations from a string (starting with ?) or RDF term (uriref or literal). 
    """
    
    is_sparql_variable = isinstance(term, str) and term.startswith('?')
    is_literal = isinstance(term, rdflib.term.Literal)
    is_uristr = isinstance(term, str) and not term.startswith('?')
    is_uriref = isinstance(term, rdflib.term.URIRef)
    is_valid_term = is_sparql_variable or is_literal or is_uristr or is_uriref
    assert is_valid_term, f'Houston, we have a problem: term is not valid'
    
    if is_uristr or is_uriref:
        term = __update_uri_with_prefix(graph, term)
        
    if is_literal:
        term = f'"{term}"'
        
    return term
    