#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to conjunctive phenomena and (rule 2)
#------------------------------------------------------------------------------
# Net Expansion AMR rule to analyse conjunctive phenomena (and)
# Rule: phenomena(_, and_phenomena) => relation(phenomena, argument)
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.rdfterm_computer import produce_uriref, produce_literal


#==============================================================================
# Pattern Search: modality(_, and_phenomena)
#==============================================================================

CONJUNCTION_PHENOMENA_URI = 'amr:phenomena_conjunction_and'  
       
def __search_pattern(graph):
    select_data_list = ['?left_phenomena_net', '?net_relation', '?right_phenomena_net']
    clause_list = [f'?left_phenomena_net a [rdfs:subClassOf* net:Phenomena_Net].',
                   f'?right_phenomena_net a [rdfs:subClassOf* net:Phenomena_Net].',
                   f'?right_phenomena_net net:hasPhenomenaType {CONJUNCTION_PHENOMENA_URI}.',
                   f'?net_relation a [rdfs:subClassOf* net:Relation].',
                   f'?left_phenomena_net ?net_relation ?right_phenomena_net.']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set



#==============================================================================
# Useful Additional Search
#==============================================================================

def __search_op_class_pattern(graph, phenomena_net_uri, num):
    assert 1 <= num <= 9, f'invalid number ({num})'
    select_data_list = ['?class_net']
    clause_list = [f'?class_net a [rdfs:subClassOf* net:Class_Net].',
                   (phenomena_net_uri, f'amr:role_op{num}', '?class_net')]
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set

def __search_phenomena_operator_class(graph, phenomena_net_uri):
    query_code = ''
    op_set = []
    for num in range(1, 9+1):
        query_code, result_set = __search_op_class_pattern(graph, phenomena_net_uri, num)
        op_set += result_set
    return query_code, op_set

    
def __search_op_property_pattern(graph, phenomena_net_uri, num):
    assert 1 <= num <= 9, f'invalid number ({num})'
    select_data_list = ['?property_net']
    clause_list = [f'?property_net a [rdfs:subClassOf* net:Property_Net].',
                   (phenomena_net_uri, f'amr:role_op{num}', '?property_net')]
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set

def __search_phenomena_operator_property(graph, phenomena_net_uri):
    query_code = ''
    op_set = []
    for num in range(1, 9+1):
        query_code, result_set = __search_op_property_pattern(graph, phenomena_net_uri, num)
        op_set += result_set
    return query_code, op_set



#==============================================================================
# Useful Computation Method(s)
#==============================================================================  

def __propagate_relation(left_net, amr_relation_uri, right_net_uri):
    new_relation = (left_net.uri, amr_relation_uri, right_net_uri)
    up_relation_list = left_net.output_relation_list + [new_relation]
    left_net.output_relation_list = up_relation_list
    
    
    
#==============================================================================
# Construct Method(s)
#==============================================================================

# None
  
    
#==============================================================================
# Main Method
#==============================================================================

def analyze_phenomena_and_2(graph):
    
    # -- Rule Initialization 
    rule_label = 'analyze "and" phenomena (2)' 
    rule_triple_list = []
    
    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Pattern Analysis
    for pattern in pattern_set:
        property_net = net.PhenomenaNet(graph, uri=pattern.left_phenomena_net)
        phenomena_net = net.PhenomenaNet(graph, uri=pattern.right_phenomena_net)
        
        # -- Search for phenomena operators
        
        # -- Relation Propagation for Operator Class Net
        _, operator_class_set = __search_phenomena_operator_class(graph, phenomena_net.uri)
        for operator in operator_class_set:
            amr_relation_uri = produce_uriref(graph, pattern.net_relation)
            right_net_uri = produce_uriref(graph, operator.class_net)
            __propagate_relation(property_net, amr_relation_uri, right_net_uri)
            rule_triple_list += property_net.generate_net_relation_triples()
            
        # -- Relation Propagation for Operator Class Net
        _, operator_property_set = __search_phenomena_operator_property(graph, phenomena_net.uri)
        for operator in operator_property_set:
            amr_relation_uri = produce_uriref(graph, pattern.net_relation)
            right_net_uri = produce_uriref(graph, operator.property_net)
            __propagate_relation(property_net, amr_relation_uri, right_net_uri)
            rule_triple_list += property_net.generate_net_relation_triples()
        
        
        # -- New Net Construction (as union of properties)
        # _, triple_list = __construct_class_union_net(graph, class_net_1, class_net_list)
        # rule_triple_list += triple_list
    
    return rule_label, rule_triple_list