#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to analyze phenomena mod (rule 1)
#------------------------------------------------------------------------------
# Net Expansion AMR rule to analyse modifier phenomena (mod)
# Rule: mod(class, class) => compositeClass
#==============================================================================

import rdflib
from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.naming_computer import define_axiom_naming
from transduction.naming_computer import define_composite_naming_2


#==============================================================================
# Pattern Search: mod(class, class)
#==============================================================================

MOD_RELATION = 'amr:role_mod' 

def __search_pattern(graph):
    select_data_list = ['?class_net_1', '?class_net_2']
    clause_list = [f'?class_net_1 a [rdfs:subClassOf* net:Class_Net].',
                   f'?class_net_2 a [rdfs:subClassOf* net:Class_Net].',
                   f'?class_net_1 {MOD_RELATION} ?class_net_2.']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set

    

#==============================================================================
# Useful Computation Method(s)
#============================================================================== 
  
def __extract_naming(net, default='none'):
    naming = f'{default}'
    naming_list = net.naming
    if len(naming_list) > 0:
        assert len(naming_list) == 1, f'Houston, we have a problem: too many naming'
        naming = naming_list[0]
    return naming

def __define_feature_restriction_naming(class_net):
    name = __extract_naming(class_net, default='something')
    return f'{name}-feature'

def __define_composite_naming(class_net_1, class_net_2):
    name_1 = __extract_naming(class_net_1, default='something')
    name_2 = __extract_naming(class_net_2, default='something')
    return f'{name_2}-{name_1}'


def __filter_relation(relation_list):
    result_list = []
    for relation in relation_list:
        check = True
        (s, p, o) = relation
        if s == o: check = False
        if p == MOD_RELATION: check = False
        if check: result_list.append(relation)
    return result_list

def __propagate_relation(target_net, base_net):
    target_net.input_relation_list = base_net.input_relation_list
    out_relation_list = __filter_relation(base_net.output_relation_list)
    target_net.output_relation_list = out_relation_list
    
    
    
#==============================================================================
# Construct Method(s)
#============================================================================== 

def __construct_feature_restriction_net(graph, class_net): # TODO
    
    restriction_net = net.RestrictionNet(graph)
    restriction_net.compose(class_net)
    
    # -- Data Computation
    restriction_net.restriction_property = '--hasFeature--' # TODO
    restriction_net.restriction_net_value = class_net.uri

    # -- Relation Propagation: None

    # -- Net Naming
    restriction_net.naming = __define_feature_restriction_naming(class_net)
    
    # -- Finalization
    restriction_net.finalize()
    triple_list = restriction_net.generate_triple_definition()
    
    return restriction_net, triple_list
    
    

def __construct_composite_class_net(graph, class_net_1, class_net_2):

    # -- Net Composition
    composite_class_net = net.CompositeClassNet(graph)
    composite_class_net.compose(class_net_1, class_net_2)
    
    # -- Data Computation
    composite_class_net.mother_class_net = class_net_1.uri

    # -- Restriction Computation
    triple_list_1 = []
    # restriction_net, triple_list_1 = __construct_feature_restriction_net(graph, class_net)
    # composite_class_net.restriction = restriction_net.uri 

    # -- Net Naming
    composite_class_net.naming = __define_composite_naming(class_net_1, class_net_2)
    
    # -- Relation Propagation
    __propagate_relation(composite_class_net, class_net_1)
    
    # -- Finalization
    composite_class_net.finalize()
    triple_list_2 = composite_class_net.generate_triple_definition()
    result_triple_list = triple_list_1 + triple_list_2
    
    return composite_class_net, result_triple_list
    
    

def __construct_individual_net(graph, class_net):

    # -- Net Composition
    individual_net = net.IndividualNet(graph)
    individual_net.compose(class_net)
    
    # -- Data Computation
    individual_net.base_class_name = f'Feature'
    individual_net.mother_class_net = class_net.uri

    # -- Net Naming
    individual_net.naming = f'{__extract_naming(class_net, default="something")}'
    individual_net.individual_label = f'{__extract_naming(class_net, default="something")}'
    
    # -- Finalization
    individual_net.finalize()
    result_triple_list = individual_net.generate_triple_definition()
    
    return individual_net, result_triple_list

  
    
#==============================================================================
# Main Method: analyze_phenomena_or_1
#==============================================================================

def analyze_phenomena_mod_1(graph):
    
    # -- Rule Initialization 
    rule_label = 'analyze modifier phenomena (mod)' 

    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Selection Analyzing (1)
    rule_triple_list = []
    for pattern in pattern_set:
        class_net_1 = net.ClassNet(graph, uri=pattern.class_net_1)
        class_net_2 = net.ClassNet(graph, uri=pattern.class_net_2)
        
        # -- New Net Construction(s)
        _, triple_list_1 = __construct_composite_class_net(graph, class_net_1, class_net_2)  
        _, triple_list_2 = __construct_individual_net(graph, class_net_2)
        
        # -- Resulting List Update
        rule_triple_list += triple_list_1 + triple_list_2
        
        # -- Deprecation: Origin Class Net
        rule_triple_list += class_net_1.deprecate()
    
    return rule_label, rule_triple_list