#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to conjunctive phenomena or (rule 1)
#------------------------------------------------------------------------------
# Net Expansion AMR rule to analyse conjunctive phenomena (or)
# Rule: property(class, or_phenomena) => compositeClass
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.naming_computer import define_composite_naming_1, define_restriction_naming


#==============================================================================
# Pattern Search: property(class, or_phenomena)
#==============================================================================

def __rule_pattern_query_code(graph):
    select_data_list = ['?property_net', '?class_net', '?phenomena_net']
    clause_list = []
    clause_list.append(f'?property_net a [rdfs:subClassOf* net:Property_Net].')
    clause_list.append(f'?class_net a [rdfs:subClassOf* net:Class_Net].')
    clause_list.append(f'?phenomena_net a [rdfs:subClassOf* net:Phenomena_Net].')
    clause_list.append(f'?phenomena_net net:hasPhenomenaType amr:phenomena_conjunction_or.')
    clause_list.append(f'?property_net amr:role_ARG0 ?class_net.')
    clause_list.append(f'?property_net amr:role_ARG1 ?phenomena_net.')
    query_code = generate_select_query(graph, select_data_list, clause_list)
    return query_code
    
       
def __search_pattern(graph):
    query_code = __rule_pattern_query_code(graph)
    rule_pattern_set = graph.query(query_code) 
    return rule_pattern_set



#==============================================================================
# Useful Additional Search
#==============================================================================

def __class_op_pattern_query_code(graph, phenomena_net_uri, num):
    assert 1 <= num <= 9, f'invalid number ({num})'
    select_data_list = ['?class_net']
    clause_list = [f'?class_net a [rdfs:subClassOf* net:Class_Net].']
    clause_list.append((phenomena_net_uri, f'amr:role_op{num}', '?class_net'))
    query_code = generate_select_query(graph, select_data_list, clause_list)
    return query_code


def __search_class_phenomena_operator(graph, phenomena_net_uri):
    op_set = []
    for num in range(1, 9+1):
        query_code = __class_op_pattern_query_code(graph, phenomena_net_uri, num)
        op_set += graph.query(query_code) 
    return op_set

    

#==============================================================================
# Useful Computation Method(s)
#==============================================================================  

def __filter_relation(relation_list):
    result_list = []
    for relation in relation_list:
        check = True
        (s, p, o) = relation
        if s == o: check = False
        if check: result_list.append(relation)
    return result_list


def __propagate_relation(target_net, base_net):
    # -- target_net.input_relation_list = base_net.input_relation_list
    out_relation_list = __filter_relation(base_net.output_relation_list)
    target_net.output_relation_list = out_relation_list
    
    
    
#==============================================================================
# Construct Method(s)
#==============================================================================

def __construct_restriction_net(graph, property_net_1, property_net_2):
    
    restriction_net = net.RestrictionNet(graph)
    restriction_net.compose(property_net_1, property_net_2)
    
    # -- Data Computation
    restriction_net.restriction_property = property_net_1.uri
    restriction_net.restriction_net_value = property_net_2.uri

    # -- Relation Propagation: None

    # -- Net Naming
    restriction_net.naming = define_restriction_naming(property_net_1, property_net_2)
    
    # -- Finalization
    restriction_net.finalize()
    triple_list = restriction_net.generate_triple_definition()
    
    return restriction_net, triple_list
    
    

def __construct_composite_class_net_from_3_nets(
        graph, base_class_net, core_property_net, target_class_net):

    # -- Net Composition
    composite_class_net = net.CompositePropertyNet(graph)
    composite_class_net.compose(base_class_net, core_property_net, target_class_net)
    
    # -- Data Computation
    composite_class_net.mother_class_net = base_class_net.uri

    # -- Restriction Computation
    restriction_net, triple_list_1 = __construct_restriction_net(graph, core_property_net, target_class_net)
    composite_class_net.restriction = restriction_net.uri
    
    # -- Relation Propagation
    __propagate_relation(composite_class_net, base_class_net) 

    # -- Net Naming
    composite_class_net.naming = define_composite_naming_1(
        base_class_net, core_property_net, target_class_net)
    
    # -- Finalization
    composite_class_net.finalize()
    triple_list_2 = composite_class_net.generate_triple_definition()
    result_triple_list = triple_list_1 + triple_list_2
    
    return composite_class_net, result_triple_list



def __construct_class_union_net(graph, base_class_net, class_net_list):

    # -- Net Composition
    class_union_net = net.PropertyUnionNet(graph)
    class_union_net.compose(base_class_net, class_net_list)
    
    # -- Data Computation: None

    # -- Restriction Computation: None
    
    # -- Relation Propagation
    __propagate_relation(class_union_net, base_class_net) 

    # -- Net Naming
    class_union_net.naming = define_union_naming(base_class_net)
    
    # -- Finalization
    class_union_net.finalize()
    triple_list = class_union_net.generate_triple_definition()
    
    return class_union_net, triple_list
  
    
#==============================================================================
# Main Method: analyze_phenomena_or_1
#==============================================================================

def analyze_phenomena_or_1(graph):
    
    # -- Rule Initialization 
    rule_label = 'analyze "or" phenomena (1)' 

    # -- Search for patterns 
    rule_pattern_set = __search_pattern(graph)
    
    # -- Selection Analyzing (1)
    rule_triple_list = []
    for selection_1 in rule_pattern_set:
        
        # -- Net Selection
        class_net_1 = net.ClassNet(graph, uri=selection_1.class_net)
        property_net = net.PropertyNet(graph, uri=selection_1.property_net)
        phenomena_net = net.PhenomenaNet(graph, uri=selection_1.phenomena_net)
        
        # -- Search for phenomena operators
        class_operator_set = __search_class_phenomena_operator(graph, phenomena_net.uri)
        
        # -- Selection Analyzing (2)
        class_net_list = []
        for selection_2 in class_operator_set:
            # print(f' *** DEVTEST *** {selection_2.class_net}')
            
            # -- Net Selection
            class_net_2 = net.ClassNet(graph, uri=selection_2.class_net)
            
            # -- New Net Construction (from 3 nets)
            new_class, triple_list = __construct_composite_class_net_from_3_nets(
                graph, class_net_1, property_net, class_net_2)
            
            # -- Resulting List Update
            class_net_list.append(new_class)
            rule_triple_list += triple_list
        
        # -- Deprecation: Origin Class Net
        rule_triple_list += class_net_1.deprecate()
        
        # -- New Net Construction (as union of properties)
        # _, triple_list = __construct_class_union_net(graph, class_net_1, class_net_list)
        # rule_triple_list += triple_list
    
    return rule_label, rule_triple_list