#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Extraction Rule Test
#------------------------------------------------------------------------------
# Script to test rules under development
#==============================================================================

import subprocess, os
from rdflib import Graph, Namespace
from rdflib.namespace import NamespaceManager, FOAF, RDF
from rdflib import URIRef, Literal, BNode

FILE_PATH = f'{os.path.dirname(os.path.abspath(__file__))}'
INPUT_DIR_PATH = f'{FILE_PATH}/test_data/'
OUTPUT_DIR_PATH = f'{FILE_PATH}/test_data/'

TEST_FILE_NAME_1 = 'atom-extraction-devGraph-1'
TEST_FILE_NAME_2 = 'atom-extraction-devGraph-2'
TEST_FILE_NAME_3 = 'atom-extraction-devGraph-3'

from context import tenet
from tenet.scheme.amr_master_rule.transduction.extractor import atom_class_extractor as test_rule_1
from tenet.scheme.amr_master_rule.transduction.extractor import atom_individual_extractor as test_rule_2
from tenet.scheme.amr_master_rule.transduction.extractor import atom_property_extractor as test_rule_3
from tenet.scheme.amr_master_rule.transduction.extractor import atom_value_extractor as test_rule_4
from tenet.scheme.amr_master_rule.transduction.extractor import atom_phenomena_extractor as test_rule_5
from tenet.scheme.amr_master_rule.transduction.extractor import atom_relation_propagator as test_rule_6
from tenet.scheme import amr_master_rule

from tenet.transduction.rdfterm_computer import __update_uri_with_prefix
from tenet.transduction import rdfterm_computer, prefix_handle
from tenet.transduction import net



#==============================================================================
# Useful Methods
#==============================================================================

def load_test_graph(test_file_name):
    print(f'\n -- Test Graph Loading')
    graph = Graph()
    prefix_handle.update_graph_namespacemanager(graph)
    graph_path = f'{INPUT_DIR_PATH}{test_file_name}.ttl'
    graph.parse(graph_path)
    print(f" ----- Graph Loaded ({len(graph)})")
    return graph


def print_triple(graph, triple, num=-1):
    num_str = f'[{num}]' if num > -1 else '[-]'
    (s, p, o) = triple
    s = __update_uri_with_prefix(graph, s)
    p = __update_uri_with_prefix(graph, p)
    o = __update_uri_with_prefix(graph, o)
    print(f' {num_str} {s} {p} {o}')
    

def add_triples_in_graph(test_file_name, graph, triple_list):
    print(f'\n -- Adding triple(s) in graph')       
    print(f" ----- Graph length before update: {len(graph)}")
    print(f" ----- Number of triples to add: {len(triple_list)}")
    
    print(f" ----- Added triples:")
    n = 0
    graph_length = len(graph)
    for triple in triple_list:
        graph.add(triple)
        if graph_length < len(graph):
            n += 1
            graph_length = len(graph)
            print_triple(graph, triple, num=n)      
        
    print(f" ----- Graph length after update: {len(graph)}")
    
    output_graph_path = f'{OUTPUT_DIR_PATH}{test_file_name}.result.ttl'
    output_graph_uri = f'https://amr.tetras-libre.fr/rdf/{test_file_name}/result'
    print(f'\n -- Serialize test graph to {output_graph_path}')
    graph.serialize(destination=output_graph_path, 
                    format='turtle',
                    base=output_graph_uri)



#==============================================================================
# Development Test
#==============================================================================
        
def test_search_pattern_1(graph):
    _, pattern_set = test_rule_1.__search_pattern(graph)
    print(f'\n ----- number of selection found: {len(pattern_set)}')
    for row in pattern_set: 
        result_str = f'>>> '
        result_str += f'{row.leaf.n3(graph.namespace_manager)}'
        result_str += f' {row.conceptName.n3(graph.namespace_manager)}'
        print(result_str) 
    return pattern_set

        
def test_search_pattern_2(graph):
    _, pattern_set = test_rule_2.__search_pattern(graph)
    print(f'\n ----- number of selection found: {len(pattern_set)}')
    for row in pattern_set: 
        result_str = f'>>> '
        result_str += f'{row.baseLeaf.n3(graph.namespace_manager)}'
        result_str += f' {row.valueLabel.n3(graph.namespace_manager)}'
        result_str += f' {row.classNet.n3(graph.namespace_manager)}'
        print(result_str) 
    return pattern_set

        
def test_search_pattern_3(graph):
    _, pattern_set = test_rule_3.__search_pattern(graph)
    print(f'\n ----- number of selection found: {len(pattern_set)}')
    for row in pattern_set: 
        result_str = f'>>> '
        result_str += f'{row.baseLeaf.n3(graph.namespace_manager)}'
        result_str += f' {row.predicateLabel.n3(graph.namespace_manager)}'
        print(result_str) 
    return pattern_set

        
def test_search_pattern_4(graph):
    _, pattern_set = test_rule_4.__search_pattern(graph)
    print(f'\n ----- number of selection found: {len(pattern_set)}')
    for row in pattern_set: 
        result_str = f'>>> '
        result_str += f'{row.valueLabel.n3(graph.namespace_manager)}'
        print(result_str) 
    return pattern_set

        
def test_search_pattern_5(graph):
    _, pattern_set = test_rule_5.__search_pattern(graph)
    print(f'\n ----- number of selection found: {len(pattern_set)}')
    for row in pattern_set: 
        result_str = f'>>> '
        result_str += f'{row.baseLeaf.n3(graph.namespace_manager)}'
        result_str += f' {row.phenomenaLabel.n3(graph.namespace_manager)}'
        result_str += f' {row.relationConceptPhenomena.n3(graph.namespace_manager)}'
        result_str += f' {row.relationConceptLabel.n3(graph.namespace_manager)}'
        print(result_str) 
    return pattern_set

        
def test_search_pattern_6(graph):
    _, pattern_set = test_rule_6.__search_pattern(graph)
    print(f'\n ----- search 1 result: {len(pattern_set)}')
    for row in pattern_set: 
        result_str = f'>>> '
        result_str += f'{row.net.n3(graph.namespace_manager)}'
        print(result_str) 
        
    
    for pattern in pattern_set:
        target_net = net.Net(graph, pattern.net)
        
        if len(target_net.base_node) > 0:
            base_leaf = target_net.base_node[0]
            
            _, result_set = test_rule_6.__search_leaf_in_leaf_relation(graph, base_leaf) 
            print(f'\n ----- search 2 result: {len(result_set)}')
            for row in result_set: 
                result_str = f'>>> '
                result_str += f'( {row.inNet.n3(graph.namespace_manager)}'
                result_str += f', {row.inRelationRole.n3(graph.namespace_manager)} )'
                print(result_str) 
                
            _, result_set = test_rule_6.__search_leaf_in_value_relation(graph, base_leaf) 
            print(f'\n ----- search 3 result: {len(result_set)}')
            for row in result_set: 
                result_str = f'>>> '
                result_str += f'( {row.inNet.n3(graph.namespace_manager)}'
                result_str += f', {row.inRelationRole.n3(graph.namespace_manager)} )'
                print(result_str)
                
            _, result_set = test_rule_6.__search_leaf_out_leaf_relation(graph, base_leaf) 
            print(f'\n ----- search 4 result: {len(result_set)}')
            for row in result_set: 
                result_str = f'>>> '
                result_str += f'( {row.outRelationRole.n3(graph.namespace_manager)}'
                result_str += f', {row.outNet.n3(graph.namespace_manager)} )'
                print(result_str)
                
            _, result_set = test_rule_6.__search_leaf_out_value_relation(graph, base_leaf) 
            print(f'\n ----- search 5 result: {len(result_set)}')
            for row in result_set: 
                result_str = f'>>> '
                result_str += f'( {row.outRelationRole.n3(graph.namespace_manager)}'
                result_str += f', {row.outNet.n3(graph.namespace_manager)} )'
                print(result_str)
            
    return pattern_set

        
def test_search_structure(graph):
    result_set = test_rule_3.__search_structure(graph)
    print(f'\n ----- number of selection found: {len(result_set)}')
    for row in result_set: 
        result_str = f'>>> '
        result_str += f'{row.sentenceId.n3(graph.namespace_manager)}'
        print(result_str) 
    return result_set
    

#==============================================================================
# Unit Test
#==============================================================================

def test_rule_application(test_file_name, graph, rule):    
    print('\n -- Rule Test')
    
    rule_label, new_triple_list = rule(graph)
    print(f' ----- label: {rule_label}')
    
    add_triples_in_graph(test_file_name, graph, new_triple_list)
    


#==============================================================================
# Test Script
#==============================================================================

if __name__ == '__main__':
      
    print('\n *** Test Preparation ***')
    graph_1 = load_test_graph(TEST_FILE_NAME_1)
    graph_2 = load_test_graph(TEST_FILE_NAME_2)
    graph_3 = load_test_graph(TEST_FILE_NAME_3)
    print('\n \n')
    
    print('\n ///////////////////// Extraction Rule 1')
    
    print('\n *** Step Test ***')
    print('\n -- Step 1: Search Pattern')
    pattern_set = test_search_pattern_1(graph_1)
    test_search_structure(graph_1)
    print('\n \n')
    
    print('\n *** Unit Test ***')
    test_rule_application(TEST_FILE_NAME_1, graph_1, test_rule_1.extract_atom_class)
    print('\n \n')
    
    print('\n ///////////////////// Extraction Rule 2')
    
    print('\n *** Step Test ***')
    print('\n -- Step 1: Search Pattern')
    pattern_set = test_search_pattern_2(graph_1)
    print('\n \n')
    
    print('\n *** Unit Test ***')
    test_rule_application(TEST_FILE_NAME_1, graph_1, test_rule_2.extract_atom_individual)
    print('\n \n')
    
    print('\n ///////////////////// Extraction Rule 3')
    
    print('\n *** Step Test ***')
    print('\n -- Step 1: Search Pattern')
    pattern_set = test_search_pattern_3(graph_1)
    print('\n \n')
    
    print('\n *** Unit Test ***')
    test_rule_application(TEST_FILE_NAME_1, graph_1, test_rule_3.extract_atom_property)
    print('\n \n')
    
    print('\n ///////////////////// Extraction Rule 4')
    
    print('\n *** Step Test ***')
    print('\n -- Step 1: Search Pattern')
    pattern_set = test_search_pattern_4(graph_1)
    print('\n \n')
    
    print('\n *** Unit Test ***')
    test_rule_application(TEST_FILE_NAME_1, graph_1, test_rule_4.extract_atom_value)
    print('\n \n')
    
    print('\n ///////////////////// Extraction Rule 5')
    
    print('\n *** Step Test ***')
    print('\n -- Step 1: Search Pattern')
    pattern_set = test_search_pattern_5(graph_1)
    print('\n \n')
    
    print('\n *** Unit Test ***')
    test_rule_application(TEST_FILE_NAME_1, graph_1, test_rule_5.extract_atom_phenomena)
    print('\n \n')
    
    print('\n ///////////////////// Extraction Rule 6')
    
    print('\n *** Step Test ***')
    print('\n -- Step 1: Search Pattern')
    pattern_set = test_search_pattern_6(graph_3)
    print('\n \n')
    
    print('\n *** Unit Test ***')
    test_rule_application(TEST_FILE_NAME_3, graph_3, test_rule_6.propagate_atom_relation)
    print('\n \n')
    print('\n \n')
    
    print('\n ///////////////////// Additional Tests')
    print('\n *** Unit Test ***')
    # test_rule_application(graph_1, test_rule_1.extract_atom_class)
    # test_rule_application(graph_1, test_rule_3.extract_atom_property)
    test_rule_application(TEST_FILE_NAME_1, graph_1, test_rule_6.propagate_atom_relation)
    test_rule_application(TEST_FILE_NAME_2, graph_2, test_rule_6.propagate_atom_relation)
    print('\n \n')

    print('\n *** - ***')