#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Extraction Rule Test
#------------------------------------------------------------------------------
# Script to test rules under development
#==============================================================================

import subprocess, os
from rdflib import Graph, Namespace
from rdflib.namespace import NamespaceManager, FOAF, RDF
from rdflib import URIRef, Literal, BNode

FILE_PATH = f'{os.path.dirname(os.path.abspath(__file__))}'
INPUT_DIR_PATH = f'{FILE_PATH}/test_data/'
OUTPUT_DIR_PATH = f'{FILE_PATH}/test_data/'

TEST_FILE_NAME_1 = 'devGraph-classes-to-generate-1'
TEST_FILE_NAME_2 = 'devGraph-classes-to-generate-2'
TEST_FILE_NAME_3 = 'devGraph-classes-to-generate-3'

from context import tenet
from tenet.transduction import rdfterm_computer, prefix_handle
from tenet.scheme.amr_master_rule.owl_generation import owl_class_generator as dev_rule
from tenet.scheme import amr_master_rule as amr_rule


from tenet.transduction.rdfterm_computer import __update_uri_with_prefix



#==============================================================================
# Useful Methods
#==============================================================================

def load_test_graph(test_file_name):
    print(f'\n -- Test Graph Loading')
    graph = Graph()
    prefix_handle.update_graph_namespacemanager(graph)
    graph_path = f'{INPUT_DIR_PATH}{test_file_name}.ttl'
    graph.parse(graph_path)
    print(f" ----- Graph Loaded ({len(graph)})")
    return graph


def print_triple(graph, triple, num=-1):
    num_str = f'[{num}]' if num > -1 else '[-]'
    (s, p, o) = triple
    s = __update_uri_with_prefix(graph, s)
    p = __update_uri_with_prefix(graph, p)
    o = __update_uri_with_prefix(graph, o)
    print(f' {num_str} {s} {p} {o}')
    

def add_triples_in_graph(test_file_name, graph, triple_list):
    print(f'\n -- Adding triple(s) in graph')       
    print(f" ----- Graph length before update: {len(graph)}")
    print(f" ----- Number of triples to add: {len(triple_list)}")
    
    print(f" ----- Added triples:")
    n = 0
    graph_length = len(graph)
    for triple in triple_list:
        graph.add(triple)
        if graph_length < len(graph):
            n += 1
            graph_length = len(graph)
            print_triple(graph, triple, num=n)      
        
    print(f" ----- Graph length after update: {len(graph)}")
    
    output_graph_path = f'{OUTPUT_DIR_PATH}{test_file_name}.result.ttl'
    output_graph_uri = f'https://amr.tetras-libre.fr/rdf/{test_file_name}/result'
    print(f'\n -- Serialize test graph to {output_graph_path}')
    graph.serialize(destination=output_graph_path, 
                    format='turtle',
                    base=output_graph_uri)

#==============================================================================
# Development Test
#==============================================================================

# -- Test: Methods to search for patterns
        
def test_search_pattern_1(graph):
    print('\n -- Search patterns (1)')
    pattern_result_set = dev_rule.__search_pattern_1(graph)
    print(f'\n ----- number of selection found: {len(pattern_result_set)}')
    for selection in pattern_result_set: 
        result_str = f'>>> '
        result_str += f'{selection.class_net.n3(graph.namespace_manager)}'
        print(result_str) 
                           
        
def test_search_pattern_2(graph):
    print('\n -- Search patterns (2)')
    pattern_result_set = dev_rule.__search_pattern_2(graph)
    print(f'\n ----- number of selection found: {len(pattern_result_set)}')
    for selection in pattern_result_set: 
        result_str = f'>>> '
        result_str += f'{selection.class_net.n3(graph.namespace_manager)}'
        print(result_str) 
   

# -- Test: Methods to get useful net list 
                        
def __test_get_list_function(graph, get_function, f_name): 
    print(f'\n -- Get net list with {f_name}')
    net_list =  get_function(graph)
    print(f'\n ----- number of selection found: {len(net_list)}')
    for net in net_list: 
        print(f'>>> {net.uri.n3(graph.namespace_manager)}') 
    return net_list
        
def test_get_atom_class_net_list(graph):
    get_function = dev_rule.__get_atom_class_net_list
    f_name = '__get_atom_class_net_list'
    return __test_get_list_function(graph, get_function, f_name)
        
def test_get_composite_class_net_list(graph):
    get_function = dev_rule.__get_composite_class_net_list
    f_name = '__get_composite_class_net_list'
    return __test_get_list_function(graph, get_function, f_name)
        
def test_get_mother_class_net_list(graph):
    list_1 =  dev_rule.__get_atom_class_net_list(graph)
    list_2 =  dev_rule.__get_composite_class_net_list(graph)
    print(f'\n -- Get net list with __get_mother_class_net_list')
    net_list =  dev_rule.__get_mother_class_net_list(list_1, list_2)
    print(f'\n ----- number of selection found: {len(net_list)}')
    for net in net_list: 
        print(f'>>> {net.uri.n3(graph.namespace_manager)}') 
    return net_list
      

# -- Test: Check Method(s)
        
def check_net_not_deprecated(graph, atom_class_net_list, composite_class_net_list):
    not_deprecated_net_list = []
    for class_net in (atom_class_net_list + composite_class_net_list):
        if not class_net.is_deprecated():
            not_deprecated_net_list.append(class_net)
    print(f'\n ----- number of not deprecated net: {len(not_deprecated_net_list)}')
    for net in not_deprecated_net_list: 
        print(f'>>> {net.uri.n3(graph.namespace_manager)}') 
        
        
# -- Test: Generator Methods
        
def test_generate_owl_triple_definition(graph, class_net):
    print(f' ----- Generate OWL Triple Definition ({class_net.uri.n3(graph.namespace_manager)})')
    triple_list =  dev_rule.__generate_owl_triple_definition(graph, class_net)
    print(f' -------- Net Class URI: {class_net.class_uri}')
    print(f' -------- number of triples: {len(triple_list)}')
    n = 0
    for triple in triple_list:
        n += 1
        print_triple(graph, triple, num=n)
    return triple_list


#==============================================================================
# Unit Test
#==============================================================================

def test_rule_application(test_file_name, graph, rule):    
    print('\n -- Rule Test')
    
    rule_label, new_triple_list = rule(graph)
    print(f' ----- label: {rule_label}')
    
    add_triples_in_graph(test_file_name, graph, new_triple_list)
    


#==============================================================================
# Test Script
#==============================================================================

if __name__ == '__main__':
      
    print('\n *** Test Preparation ***')
    graph_1 = load_test_graph(TEST_FILE_NAME_1)
    graph_2 = load_test_graph(TEST_FILE_NAME_2)
    graph_3 = load_test_graph(TEST_FILE_NAME_3)
    print('\n \n')
    
    print('\n ///////////////////// Generator Rule 1')
    
    print('\n *** Step Test (1) ***')
    
    print('\n -- Step 1a: search pattern')
    pattern_set_1 = test_search_pattern_1(graph_1)
    pattern_set_2 = test_search_pattern_2(graph_1)
    
    print('\n -- Step 1b: get semantic net list')
    atom_class_net_list =  test_get_atom_class_net_list(graph_1)
    composite_class_net_list =  test_get_composite_class_net_list(graph_1)
    mother_class_net_list =  test_get_mother_class_net_list(graph_1)
    
    print('\n -- Step 2: check class net not deprecated')
    check_net_not_deprecated(graph_1, atom_class_net_list, composite_class_net_list)
       
    print('\n -- Step 3: generate OWL triple definition for class net')
    total_triple_list = []
    for class_net in (atom_class_net_list + composite_class_net_list):
        if not class_net.is_deprecated():    
            total_triple_list += test_generate_owl_triple_definition(graph_1, class_net)
    print(f' ----- Total Number of triples: {len(total_triple_list)}')
    
    print('\n \n')
    
    
    print('\n *** Step Test (2) ***')
    
    print('\n -- Step 1a: search pattern')
    pattern_set_1 = test_search_pattern_1(graph_2)
    pattern_set_2 = test_search_pattern_2(graph_2)
    
    print('\n -- Step 1b: get semantic net list')
    atom_class_net_list =  test_get_atom_class_net_list(graph_2)
    composite_class_net_list =  test_get_composite_class_net_list(graph_2)
    mother_class_net_list =  test_get_mother_class_net_list(graph_2)
    
    print('\n -- Step 2: check class net not deprecated')
    check_net_not_deprecated(graph_2, atom_class_net_list, composite_class_net_list)
       
    print('\n -- Step 3: generate OWL triple definition for class net')
    total_triple_list = []
    for class_net in composite_class_net_list:
        if not class_net.is_deprecated():
            total_triple_list = test_generate_owl_triple_definition(graph_1, class_net)
    print(f' ----- Total Number of triples: {len(total_triple_list)}')
    
    print('\n \n')
    
    print('\n *** Unit Test ***')
    test_rule_application(TEST_FILE_NAME_3, graph_3, amr_rule.generate_owl_class)
    print('\n \n')
    

    print('\n *** - ***')