#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Extraction Rule Test
#------------------------------------------------------------------------------
# Script to test rules under development
#==============================================================================

import subprocess, os
from rdflib import Graph, Namespace
from rdflib.namespace import NamespaceManager, FOAF, RDF
from rdflib import URIRef, Literal, BNode

FILE_PATH = f'{os.path.dirname(os.path.abspath(__file__))}'
INPUT_DIR_PATH = f'{FILE_PATH}/test_data/'
OUTPUT_DIR_PATH = f'{FILE_PATH}/test_data/'

TEST_FILE_NAME = 'devGraph-properties-to-generate'
INPUT_GRAPH_PATH = f'{INPUT_DIR_PATH}{TEST_FILE_NAME}.ttl'
OUTPUT_GRAPH_PATH = f'{OUTPUT_DIR_PATH}{TEST_FILE_NAME}.result.ttl'
OUTPUT_GRAPH_URI = f'https://amr.tetras-libre.fr/rdf/{TEST_FILE_NAME}/result'

from context import tenet
from tenet.transduction import rdfterm_computer, prefix_handle
from tenet.scheme.amr_master_rule.owl_generation import owl_property_generator as dev_rule
from tenet.scheme import amr_master_rule as amr_rule




#==============================================================================
# Useful Methods
#==============================================================================

def load_test_graph():
    print(f'\n -- Test Graph Loading')
    graph = Graph()
    prefix_handle.update_graph_namespacemanager(graph)
    graph.parse(INPUT_GRAPH_PATH)
    print(f" ----- Graph Loaded ({len(graph)})")
    return graph


def define_clause_list(composition_pattern_list):
    clause_list = []
    for (net_1, relation, net_2) in composition_pattern_list:
        clause_list.append(f'{net_1} {relation} {net_2}.')
    return clause_list


def print_triple(triple, num=-1):
    num_str = f'[{num}]' if num > -1 else '[-]'
    (s, p, o) = triple
    s = rdfterm_computer.__update_uri_with_prefix(graph, s)
    p = rdfterm_computer.__update_uri_with_prefix(graph, p)
    o = rdfterm_computer.__update_uri_with_prefix(graph, o)
    print(f' {num_str} {s} {p} {o}')
    


def add_triples_in_graph(graph, triple_list):
    print(f'\n -- Adding triple(s) in graph')       
    print(f" ----- Graph length before update: {len(graph)}")
    print(f" ----- Number of triples to add: {len(triple_list)}")
    
    print(f" ----- Added triples:")
    n = 0
    for triple in triple_list:
        n += 1
        print_triple(triple, num=n)
        graph.add(triple)      
        
    print(f" ----- Graph length after update: {len(graph)}")
    
    print(f'\n -- Serialize test graph to {OUTPUT_GRAPH_PATH}')
    graph.serialize(destination=OUTPUT_GRAPH_PATH, 
                    format='turtle',
                    base=OUTPUT_GRAPH_URI)


#==============================================================================
# Development Test
#==============================================================================
        
def test_search_pattern_1(graph):
    print('\n -- Search patterns (1)')
    pattern_result_set = dev_rule.__search_pattern_1(graph)
    print(f'\n ----- number of selection found: {len(pattern_result_set)}')
    for selection in pattern_result_set: 
        result_str = f'>>> '
        result_str += f'{selection.property_net.n3(graph.namespace_manager)}'
        print(result_str) 
                           
        
def test_search_pattern_2(graph):
    print('\n -- Search patterns (2)')
    pattern_result_set = dev_rule.__search_pattern_2(graph)
    print(f'\n ----- number of selection found: {len(pattern_result_set)}')
    for selection in pattern_result_set: 
        result_str = f'>>> '
        result_str += f'{selection.property_net.n3(graph.namespace_manager)}'
        print(result_str) 
                           
def __test_get_list_function(graph, get_function, f_name): 
    print(f'\n -- Get net list with {f_name}')
    net_list =  get_function(graph)
    print(f'\n ----- number of selection found: {len(net_list)}')
    for net in net_list: 
        print(f'>>> {net.uri.n3(graph.namespace_manager)}') 
    return net_list
        
def test_get_atom_property_net_list(graph):
    get_function = dev_rule.__get_atom_property_net_list
    f_name = '__get_atom_property_net_list'
    return __test_get_list_function(graph, get_function, f_name)
        
def test_get_composite_property_net_list(graph):
    get_function = dev_rule.__get_composite_property_net_list
    f_name = '__get_composite_property_net_list'
    return __test_get_list_function(graph, get_function, f_name)
        
def test_get_mother_property_net_list(graph):
    list_1 =  dev_rule.__get_atom_property_net_list(graph)
    list_2 =  dev_rule.__get_composite_property_net_list(graph)
    print(f'\n -- Get net list with __get_mother_property_net_list')
    net_list =  dev_rule.__get_mother_property_net_list(list_1, list_2)
    print(f'\n ----- number of selection found: {len(net_list)}')
    for net in net_list: 
        print(f'>>> {net.uri.n3(graph.namespace_manager)}') 
    return net_list
        
        
def check_net_not_deprecated(atom_property_net_list, composite_property_net_list):
    not_deprecated_net_list = []
    for property_net in (atom_property_net_list + composite_property_net_list):
        if not property_net.is_deprecated():
            not_deprecated_net_list.append(property_net)
    print(f'\n ----- number of not deprecated net: {len(not_deprecated_net_list)}')
    for net in not_deprecated_net_list: 
        print(f'>>> {net.uri.n3(graph.namespace_manager)}') 
        
     
        
def test_generate_owl_triple_definition(graph, property_net):
    print(f' ----- Generate OWL Triple Definition ({property_net.uri.n3(graph.namespace_manager)})')
    print(f' -------- Net property_uri: {property_net.property_uri}')
    triple_list =  dev_rule.__generate_owl_triple_definition(graph, property_net)
    print(f' -------- number of triples: {len(triple_list)}')
    n = 0
    for triple in triple_list:
        n += 1
        print_triple(triple, num=n)
    return triple_list


#==============================================================================
# Unit Test
#==============================================================================

def unittest_run_rule(graph, rule):    
    print('\n -- Rule Test')
    
    rule_label, new_triple_list = rule(graph)
    print(f' ----- label: {rule_label}')
    print(f' ----- new_triple_list ({len(new_triple_list)}):')
    
    add_triples_in_graph(graph, new_triple_list)
    


#==============================================================================
# Test Script
#==============================================================================

if __name__ == '__main__':
      
    print('\n *** Test Preparation ***')
    graph = load_test_graph()
    uriref = URIRef('net:compositeClass_orbit_hasManner_conjunction-OR')
    type_uriref = URIRef('net:Composite_Class_Net')
    triple = (uriref, RDF.type, type_uriref)
    phenomena_net_uri = 'net:phenomena_conjunction-OR_o3'
    print('\n \n')
    
    print('\n *** Step Test ***')
    
    print('\n -- Step 1a: search pattern')
    pattern_set_1 = test_search_pattern_1(graph)
    pattern_set_2 = test_search_pattern_2(graph)
    
    print('\n -- Step 1b: get semantic net list')
    atom_property_net_list =  test_get_atom_property_net_list(graph)
    composite_property_net_list =  test_get_composite_property_net_list(graph)
    mother_property_net_list =  test_get_mother_property_net_list(graph)
    
    print('\n -- Step 2: check property net not deprecated')
    check_net_not_deprecated(atom_property_net_list, composite_property_net_list)
       
    print('\n -- Step 3: generate OWL triple definition for property net')
    total_triple_list = []
    for property_net in (atom_property_net_list + composite_property_net_list):
        if not property_net.is_deprecated():    
            total_triple_list += test_generate_owl_triple_definition(graph, property_net)
    print(f' ----- Total Number of triples: {len(total_triple_list)}')
        
    print('\n \n')
    
    print('\n *** Unit Test ***')
    unittest_run_rule(graph, amr_rule.generate_owl_property)
    print('\n \n')

    print('\n *** - ***')