#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Extraction Rule Test
#------------------------------------------------------------------------------
# Script to test rules under development
#==============================================================================

import subprocess, os
from rdflib import Graph, Namespace
from rdflib.namespace import NamespaceManager, FOAF, RDF
from rdflib import URIRef, Literal, BNode

FILE_PATH = f'{os.path.dirname(os.path.abspath(__file__))}'
INPUT_DIR_PATH = f'{FILE_PATH}/test_data/'
OUTPUT_DIR_PATH = f'{FILE_PATH}/test_data/'

TEST_FILE_NAME_1 = 'and-devGraph-1'
TEST_FILE_NAME_2 = 'and-devGraph-2'

from context import tenet
from tenet.scheme.amr_master_rule.transduction.phenomena_analyzer import and_analyzer_1 as rule_1
from tenet.scheme.amr_master_rule.transduction.phenomena_analyzer import and_analyzer_2 as rule_2
from tenet.scheme import amr_master_rule as rule

from tenet.transduction.rdfterm_computer import __update_uri_with_prefix
from tenet.transduction import query_builder
from tenet.transduction import prefix_handle


#==============================================================================
# Useful Methods
#==============================================================================

def load_test_graph(test_file_name):
    print(f'\n -- Test Graph Loading')
    graph = Graph()
    prefix_handle.update_graph_namespacemanager(graph)
    graph_path = f'{INPUT_DIR_PATH}{test_file_name}.ttl'
    graph.parse(graph_path)
    print(f" ----- Graph Loaded ({len(graph)})")
    return graph


def print_triple(graph, triple, num=-1):
    num_str = f'[{num}]' if num > -1 else '[-]'
    (s, p, o) = triple
    s = __update_uri_with_prefix(graph, s)
    p = __update_uri_with_prefix(graph, p)
    o = __update_uri_with_prefix(graph, o)
    print(f' {num_str} {s} {p} {o}')
    


def add_triples_in_graph(test_file_name, graph, triple_list):
    print(f'\n -- Adding triple(s) in graph')       
    print(f" ----- Graph length before update: {len(graph)}")
    print(f" ----- Number of triples to add: {len(triple_list)}")
    
    print(f" ----- Added triples:")
    n = 0
    graph_length = len(graph)
    for triple in triple_list:
        graph.add(triple)
        if graph_length < len(graph):
            n += 1
            graph_length = len(graph)
            print_triple(graph, triple, num=n)      
        
    print(f" ----- Graph length after update: {len(graph)}")
    
    output_graph_path = f'{OUTPUT_DIR_PATH}{test_file_name}.result.ttl'
    output_graph_uri = f'https://amr.tetras-libre.fr/rdf/{test_file_name}/result'
    print(f'\n -- Serialize test graph to {output_graph_path}')
    graph.serialize(destination=output_graph_path, 
                    format='turtle',
                    base=output_graph_uri)


#==============================================================================
# Development Test
#==============================================================================
        
def test_search_pattern_1(graph):    
    query_code, pattern_set = rule_1.__search_pattern(graph)
    print(f'\n ----- query code: {query_code}')
    print(f'\n ----- number of selection found: {len(pattern_set)}')
    for selection in pattern_set: 
        result_str = f'>>> '
        result_str += f'{selection.property_net.n3(graph.namespace_manager)}'
        result_str += f' {selection.net_relation.n3(graph.namespace_manager)}'
        result_str += f' {selection.phenomena_net.n3(graph.namespace_manager)}'
        print(result_str)        
    return pattern_set


def test_search_pattern_2(graph):    
    query_code, pattern_set = rule_2.__search_pattern(graph)
    print(f'\n ----- query code: {query_code}')
    print(f'\n ----- number of selection found: {len(pattern_set)}')
    for selection in pattern_set: 
        result_str = f'>>> '
        result_str += f'{selection.left_phenomena_net.n3(graph.namespace_manager)}'
        result_str += f' {selection.net_relation.n3(graph.namespace_manager)}'
        result_str += f' {selection.right_phenomena_net.n3(graph.namespace_manager)}'
        print(result_str)        
    return pattern_set

     
def test_search_operators_1(graph, phenomena_net_uri):
    print('\n *** DEVTEST *** Search for operators')
    query_code, result_set = rule_1.__search_phenomena_operator_class(graph, phenomena_net_uri)
    print(f' -- query code: \n{query_code}')
    print(f'\n ----- result number: {len(result_set)}')
    for selection in result_set: 
        result_str = f'>>> '
        result_str += f'{selection.class_net.n3(graph.namespace_manager)}'
        print(result_str) 
        
     
def test_search_operators_2(graph, phenomena_net_uri):
    print('\n *** DEVTEST *** Search for operators')
    query_code, result_set = rule_1.__search_phenomena_operator_property(graph, phenomena_net_uri)
    print(f' -- query code: \n{query_code}')
    print(f'\n ----- result number: {len(result_set)}')
    for selection in result_set: 
        result_str = f'>>> '
        result_str += f'{selection.property_net.n3(graph.namespace_manager)}'
        print(result_str) 
                      
        
def devtest_add_triple(graph, triple):
    print(f'\n -- Adding triple in a graph')
    print(f"----- Graph length before update: {len(graph)}")
    print(f"----- Triple added: {triple}")
    graph.add(triple)
    print(f"----- Graph length after update: {len(graph)}")
    
    

#==============================================================================
# Unit Test
#==============================================================================

def test_rule_application(test_file_name, graph, rule):    
    print('\n -- Rule Test')
    
    rule_label, new_triple_list = rule(graph)
    print(f' ----- label: {rule_label}')
    
    add_triples_in_graph(test_file_name, graph, new_triple_list)
    


#==============================================================================
# Test Script
#==============================================================================

if __name__ == '__main__':
      
    print('\n *** Test Preparation ***')
    graph_1 = load_test_graph(TEST_FILE_NAME_1)
    graph_2 = load_test_graph(TEST_FILE_NAME_2)
    uriref = URIRef('net:compositeClass_orbit_hasManner_conjunction-OR')
    type_uriref = URIRef('net:Composite_Class_Net')
    triple = (uriref, RDF.type, type_uriref)
    phenomena_net_uri = 'net:phenomena_conjunction-AND_a'
    print('\n \n')
    
    
    print('\n ///////////////////// Extraction Rule 1')
    
    # print('\n *** Step Test ***')
    
    print('\n -- Step 1: Search Pattern')
    pattern_set = test_search_pattern_1(graph_1) 
    test_search_operators_1(graph_1, phenomena_net_uri) 
    test_search_operators_2(graph_1, phenomena_net_uri) 
    print('\n \n')
    
    print('\n *** Unit Test ***')
    test_rule_application(TEST_FILE_NAME_1, graph_1, rule.analyze_phenomena_and_1)
    print('\n \n')
    
    
    # print('\n ///////////////////// Extraction Rule 2')
    
    print('\n *** Step Test ***')
    
    print('\n -- Step 1: Search Pattern')
    pattern_set = test_search_pattern_2(graph_2)
    test_search_operators_1(graph_2, phenomena_net_uri)  
    test_search_operators_2(graph_2, phenomena_net_uri)    
    print('\n \n')
    
    print('\n *** Unit Test ***')
    test_rule_application(TEST_FILE_NAME_2, graph_2, rule.analyze_phenomena_and_2)
    print('\n \n')
    

    print('\n *** - ***')