#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Extraction Rule Test
#------------------------------------------------------------------------------
# Script to test rules under development
#==============================================================================

import subprocess, os
from rdflib import Graph, Namespace
from rdflib.namespace import NamespaceManager, FOAF, RDF
from rdflib import URIRef, Literal, BNode

FILE_PATH = f'{os.path.dirname(os.path.abspath(__file__))}'
INPUT_DIR_PATH = f'{FILE_PATH}/test_data/'
OUTPUT_DIR_PATH = f'{FILE_PATH}/test_data/'
INPUT_GRAPH_PATH = f'{INPUT_DIR_PATH}devGraph1.ttl'
OUTPUT_GRAPH_PATH = f'{OUTPUT_DIR_PATH}devGraph1.result.ttl'
OUTPUT_GRAPH_URI = f'https://amr.tetras-libre.fr/rdf/devGraph1/result'

from context import tenet
from tenet.scheme.amr_master_rule.transduction.phenomena_analyzer import or_analyzer_1 as rule_1
from tenet.scheme.amr_master_rule.transduction.phenomena_analyzer import or_analyzer_2 as rule_2
from tenet.scheme import amr_master_rule as rule

from tenet.transduction.rdfterm_computer import __update_uri_with_prefix
from tenet.transduction import query_builder
from tenet.transduction import prefix_handle

INDENT_STR = '            '



#==============================================================================
# Useful Methods
#==============================================================================

def load_test_graph():
    print(f'\n -- Test Graph Loading')
    graph = Graph()
    prefix_handle.update_graph_namespacemanager(graph)
    graph.parse(INPUT_GRAPH_PATH)
    print(f" ----- Graph Loaded ({len(graph)})")
    return graph


def define_clause_list(composition_pattern_list):
    clause_list = []
    for (net_1, relation, net_2) in composition_pattern_list:
        clause_list.append(f'{net_1} {relation} {net_2}.')
    return clause_list


def print_triple(triple, num=-1):
    num_str = f'[{num}]' if num > -1 else '[-]'
    (s, p, o) = triple
    s = __update_uri_with_prefix(graph, s)
    p = __update_uri_with_prefix(graph, p)
    o = __update_uri_with_prefix(graph, o)
    print(f' {num_str} {s} {p} {o}')
    


def add_triples_in_graph(graph, triple_list):
    print(f'\n -- Adding triple(s) in graph')       
    print(f" ----- Graph length before update: {len(graph)}")
    print(f" ----- Number of triples to add: {len(triple_list)}")
    
    print(f" ----- Added triples:")
    n = 0
    for triple in triple_list:
        n += 1
        print_triple(triple, num=n)
        graph.add(triple)      
        
    print(f" ----- Graph length after update: {len(graph)}")
    
    print(f'\n -- Serialize test graph to {OUTPUT_GRAPH_PATH}')
    graph.serialize(destination=OUTPUT_GRAPH_PATH, 
                    format='turtle',
                    base=OUTPUT_GRAPH_URI)


#==============================================================================
# Development Test
#==============================================================================
        
def devtest_select_pattern_application_1(graph):
    print('\n -- Select Pattern Application 1')
    query_code = rule_1.__rule_pattern_query_code(graph)
    print(query_code)
    pattern_result_set = rule_1.__search_pattern(graph)
    print(f'\n ----- number of selection found: {len(pattern_result_set)}')
    for selection in pattern_result_set: 
        result_str = f'>>> '
        result_str += f'{selection.property_net.n3(graph.namespace_manager)}'
        result_str += f' {selection.class_net.n3(graph.namespace_manager)}'
        result_str += f' {selection.phenomena_net.n3(graph.namespace_manager)}'
        print(result_str) 
        

def devtest_select_pattern_application_2(graph):
    print('\n -- Select Pattern Application 2')
    query_code = rule_2.__rule_pattern_query_code(graph)
    print(query_code)
    pattern_result_set = rule_2.__search_pattern(graph)
    print(f'\n ----- number of selection found: {len(pattern_result_set)}')
    for selection in pattern_result_set: 
        result_str = f'>>> '
        result_str += f'{selection.property_net_core.n3(graph.namespace_manager)}'
        result_str += f' {selection.property_net_arg0.n3(graph.namespace_manager)}'
        result_str += f' {selection.phenomena_net.n3(graph.namespace_manager)}'
        print(result_str) 
        

def devtest_search_operators(graph, phenomena_net_uri):
    print('\n *** DEVTEST *** Search for operators')
    query_code = rule_2.__property_op_pattern_query_code(graph, phenomena_net_uri, 1)
    print(f' -- query code for operator 1: \n{query_code}')
    result_set = rule_2.__search_property_phenomena_operator(graph, phenomena_net_uri)
    print(f'\n ----- number of selection found: {len(result_set)}')
    for selection in result_set: 
        result_str = f'>>> '
        result_str += f'{selection.property_net.n3(graph.namespace_manager)}'
        print(result_str) 
                      
        
def devtest_add_triple(graph, triple):
    print(f'\n -- Adding triple in a graph')
    print(f"----- Graph length before update: {len(graph)}")
    print(f"----- Triple added: {triple}")
    graph.add(triple)
    print(f"----- Graph length after update: {len(graph)}")
    
    

#==============================================================================
# Unit Test
#==============================================================================

def unittest_run_rule(graph, rule):    
    print('\n -- Rule Test')
    
    rule_label, new_triple_list = rule(graph)
    print(f' ----- label: {rule_label}')
    print(f' ----- new_triple_list ({len(new_triple_list)}):')
    
    add_triples_in_graph(graph, new_triple_list)
    


#==============================================================================
# Test Script
#==============================================================================

if __name__ == '__main__':
      
    print('\n *** Test Preparation ***')
    graph = load_test_graph()
    uriref = URIRef('net:compositeClass_orbit_hasManner_conjunction-OR')
    type_uriref = URIRef('net:Composite_Class_Net')
    triple = (uriref, RDF.type, type_uriref)
    phenomena_net_uri = 'net:phenomena_conjunction-OR_o3'
    print('\n \n')
    
    print('\n *** Development Test ***')
    devtest_select_pattern_application_1(graph)
    devtest_select_pattern_application_2(graph)
    devtest_search_operators(graph, phenomena_net_uri)
    devtest_add_triple(graph, triple)
    print('\n \n')
    
    print('\n *** Unit Test ***')
    unittest_run_rule(graph, rule.analyze_phenomena_or_1)
    unittest_run_rule(graph, rule.analyze_phenomena_or_2)
    print('\n \n')

    print('\n *** - ***')