#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Extraction Rule Test
#------------------------------------------------------------------------------
# Script to test rules under development
#==============================================================================

import subprocess, os
import rdflib
from rdflib import URIRef, Literal, BNode
from rdflib.namespace import FOAF, RDF, RDFS
from rdflib import Graph, Namespace, URIRef
from rdflib.namespace import NamespaceManager
from rdflib.term import _is_valid_uri

FILE_PATH = f'{os.path.dirname(os.path.abspath(__file__))}'
INPUT_DIR_PATH = f'{FILE_PATH}/test_data/'
OUTPUT_DIR_PATH = f'{FILE_PATH}/test_data/'

TEST_FILE_NAME = 'devGraph1'
INPUT_GRAPH_PATH = f'{INPUT_DIR_PATH}{TEST_FILE_NAME}.ttl'
OUTPUT_GRAPH_PATH = f'{OUTPUT_DIR_PATH}{TEST_FILE_NAME}.result.ttl'
OUTPUT_GRAPH_URI = f'https://amr.tetras-libre.fr/rdf/{TEST_FILE_NAME}/result'

from context import tenet
import utility

from tenet.transduction import rdfterm_computer
from tenet.transduction.rdfterm_computer import produce_uriref, produce_literal



#==============================================================================
# Useful Methods
#==============================================================================

def load_test_graph():
    print(f'\n -- Test Graph Loading')
    graph = Graph()
    graph.bind('amr', Namespace('https://amr.tetras-libre.fr/rdf/schema#'))
    graph.bind('net', Namespace('https://tenet.tetras-libre.fr/semantic-net#'))
    graph.bind('base-out', Namespace('https://tenet.tetras-libre.fr/base-ontology#'))
    graph.parse(INPUT_GRAPH_PATH)
    print(f" ----- Graph Loaded ({len(graph)})")
    return graph


def generate_empty_test_graph():
    print(f'\n -- Test Graph Loading')
    graph = Graph()
    graph.bind('amr', Namespace('https://amr.tetras-libre.fr/rdf/schema#'))
    graph.bind('net', Namespace('https://tenet.tetras-libre.fr/semantic-net#'))
    graph.bind('base-out', Namespace('https://tenet.tetras-libre.fr/base-ontology#'))
    graph.bind('rdf', Namespace('http://www.w3.org/1999/02/22-rdf-syntax-ns#'))
    print(f" ----- Graph Loaded ({len(graph)})")
    graph.serialize(destination=OUTPUT_GRAPH_PATH, format='turtle')
    return graph


#==============================================================================
# Development Test
#==============================================================================

def devtest_check_uri(uri):
    print(f'\n -- Test URI checking')
    print(f' ----- uri: {uri}')
    print(f' ----- check 1 (valid uri): {_is_valid_uri(uri)}')
    print(f' ----- check 2 (URIRef instance): {isinstance(uri, rdflib.term.URIRef)}')
    print(f' ----- check 3 (str instance): {isinstance(uri, str)}')
    print(f' ----- check 4 (not http uri): {uri.startswith("http")}')
    print(f' ----- check 5 (not single quote uri): {uri.startswith("<") & uri.endswith(">")}')
    
    
def devtest_create_uriref(graph, uri):
    print(f'\n -- Creation of URI Reference')
    assert isinstance(uri, str), f'URI is not a string'
    
    origin_uri = uri
    
    if (uri.startswith("<") & uri.endswith(">")):
        uri = uri[1:-1]
        
    nsm = graph.namespace_manager
    for (prefix, namespace) in nsm.namespaces():
        if len(prefix) > 0 and uri.startswith(prefix):
            print(f' ----- Replace "{prefix}" by "{namespace}"')
            uri = uri.replace(f'{prefix}:', namespace)
            uri = rdflib.URIRef(uri)
    
    if not isinstance(uri, rdflib.term.URIRef):
        uri = rdflib.URIRef(uri)
    assert isinstance(uri, rdflib.term.URIRef), f'Houston, we have a problem (1)'
    
    # nsm = graph.namespace_manager
    # if uri.startswith("http") and uri in nsm:
    #     new_uri = rdflib.URIRef(uri.n3(nsm))
    #     assert isinstance(new_uri, rdflib.term.URIRef), f'Houston, we have a problem (2)'
    # else:
    #     new_uri = uri
    #     assert isinstance(new_uri, rdflib.term.URIRef), f'Houston, we have a problem (3)'

    assert isinstance(uri, rdflib.term.URIRef), f'Houston, we have a problem: URI is not an URIRef instance'
    assert _is_valid_uri(uri), f'Houston, we have a problem: URI is not a valid uri'
    print(f' ----- origin uri: {origin_uri}')
    print(f' ----- new uri: {uri}')
    print(f' ----- new uri checking as URIRef instance: {isinstance(uri, rdflib.term.URIRef)}')



#==============================================================================
# Unit Test
#==============================================================================

def unittest_produce_uriref(graph, uri):
    print(f'\n -- Produce valid URI Reference')
    try:
        uriref = rdfterm_computer.produce_uriref(graph, uri)
        if uriref is None:
            print(f' ----- None')
        else: # uriref is not None
            print(f' ----- origin uri: {uri}')
            print(f' ----- result uri: {uriref}')
            c_1 = _is_valid_uri(uriref)
            c_2 = isinstance(uriref, rdflib.term.URIRef)
            c_3 = isinstance(uriref, str)
            c_4 = uriref.startswith("http")
            c_5 = not(uriref.startswith("<") & uriref.endswith(">"))
            assert c_1 & c_2 & c_3 & c_4 & c_5
            print(f' ----- result uri check: ok')
        return uriref
    except:
        print(f' ----- check 1 (valid uri): {c_1}')
        print(f' ----- check 2 (URIRef instance): {c_2}')
        print(f' ----- check 3 (str instance): {c_3}')
        print(f' ----- check 4 (not http uri): {c_4}')
        print(f' ----- check 5 (not single quote uri): {c_5}')
        return None
    

def unittest_failure_produce_uriref(graph, uri):
    print(f'\n -- * Failure to produce URI Reference *')
    try:
        uriref = rdfterm_computer.produce_uriref(graph, uri)
        print(f'Houston, we have a problem: production is successful')
    except:
        print(f' ----- OK FAILURE!')
        print(f' ----- origin uri: {uri}')


def unittest_add_uriref_in_graph(graph, *uriref_set):
    print(f'\n -- * TEST uriref adding *')   
    
    print(f" ----- Graph length before update: {len(graph)}")
    
    print(f'\n -- Adding triples in test graph')
    n = 0
    for uriref in uriref_set:
        if uriref is not None:
            n += 1
            rdf_type_uriref = produce_uriref(graph, RDF.type)
            rdf_class_uriref = produce_uriref(graph, RDFS.Class)
            triple = (uriref, rdf_type_uriref, rdf_class_uriref)
            print(f' [{n}] {uriref} {rdf_type_uriref} {rdf_class_uriref}')
            graph.add(triple)      
        
    print(f" ----- Graph length after update: {len(graph)}")
    
    print(f'\n -- Serialize test graph to {OUTPUT_GRAPH_PATH}')
    graph.serialize(destination=OUTPUT_GRAPH_PATH, format='turtle')
    

#==============================================================================
# Test Script
#==============================================================================

if __name__ == '__main__':
      
    print('\n *** Test Preparation ***')
    graph = generate_empty_test_graph()
    graph = load_test_graph()
    uri_1 = 'net:compositeClass_orbit_hasManner_conjunction-OR'
    uri_2 = '<net:compositeClass_object_orbit_sun>'
    uri_3 = 'https://tenet.tetras-libre.fr/semantic-net#atomClass_sun_s2'
    uri_4 = 'https://unknow-url#atomClass_object_o1'
    uri_5 = URIRef('net:atomProperty_orbit_o2')
    print('\n \n')
    
    print('\n *** Development Test ***')
    devtest_check_uri(uri_1)
    devtest_check_uri(uri_2)
    devtest_check_uri(uri_3)
    devtest_check_uri(uri_4)
    devtest_check_uri(uri_5)
    devtest_create_uriref(graph, uri_1)
    devtest_create_uriref(graph, uri_1)
    devtest_create_uriref(graph, uri_2)
    devtest_create_uriref(graph, uri_3)
    devtest_create_uriref(graph, uri_4)
    devtest_create_uriref(graph, uri_5)
    print('\n \n')
    
    print('\n *** Unit Test ***')
    uriref_1 = unittest_produce_uriref(graph, uri_1)
    uriref_2 = unittest_produce_uriref(graph, uri_2)
    uriref_3 = unittest_produce_uriref(graph, uri_3)
    unittest_failure_produce_uriref(graph, uri_4)
    uriref_5 = unittest_produce_uriref(graph, uri_5)
    unittest_produce_uriref(graph, None)
    uriref_set = (uriref_1, uriref_1, uriref_2, uriref_3)
    unittest_add_uriref_in_graph(graph, *uriref_set)    
    print('\n \n')

    print('\n *** - ***')